/* build.c:  build tide database
 *
 * Pilot utility to build location databases for Tide Tool
 * (c) 1997, 2000 Walt Bilofsky
 * Modified 7/00 to generate database files under Windows.
 *
 * Modified from Xtide 1.5 - (c) 1996, 1997 D. Jeff Dionne
 * This is free software, under the GNU public license v2
 */

#include "config.h"
#include "DataPrv.h"	/* Some Palm definitions for generating database file */
#include "tidelib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

char *fitfilters[] = {					/* These used to make fit in font 1 */
	"California",		"Calif.",
	"Washington",		"Wash.",
	"Oregon",			"Ore.",
	"New Hampshire",		"N.H.",
	"Massachusetts",		"Mass.",
	"Rhode Island",		"R.I.",
	"Pennsylvania",		"Pa.",
	"New Jersey",		"N.J.",
	"Maryland",			"Md.",
	"Delaware",			"Del.",
	"New York",			"N.Y.",
	"Connecticut",		"Ct.",
	"Virginia",			"Va.",
	"Virgina",	/* (sic) */	"Va.",
	"North Carolina",		"N.C.",
	"South Carolina",		"S.C.",
	"Georgia",			"Ga.",
	"Louisiana",		"La.",
	"Mississippi",		"Miss.",
	"Florida",			"Fla.",
	"British Columbia",	"B.C.",
	"New Brunswick",		"N.B.",
	"Northeast",		"NE.",		/* Discretionary substitutions */
	"Northwest",		"NW.",		/* (Also used in tidelib) */
	"Southeast",		"SE.",
	"Southwest",		"SW.",
	"North",			"N.",
	"East",			"E.",
	"South",			"S.",
	"West",			"W.",
	"northeast",		"ne.",
	"northwest",		"nw.",
	"southeast",		"se.",
	"southwest",		"sw.",
	"north",			"n.",
	"east",			"e.",
	"south",			"s.",
	"west",			"w.",
	"miles",			"mi.",
	"mile",			"mi.",
	"San Francisco Bay",	"S.F. Bay",
	"Fort",			"Ft.",
	"Canada",			"Can.",
	"Point",			"Pt.",	
	"Entrance",			"Ent.",
	"entrance",			"ent.",
	"Terminal",			"Term.",
	"Channel",			"Chan.",
	"Island",			"Is.",
	"River",			"R.",
	"Norte",			"N.",
	"Mexico",			"Mex.",
	"Boulevard",		"Blvd.",
	NULL, NULL };

static char *cfilters[] = {				/* These used to make fit in font 0 */
	"Terminal",			"Term.",		/* These abbreviations if really desperate */
	" and ",			" & ",
	"Calif.",			"CA",	
	"Mass.",			"MA",
	"Del.",			"DE",
	"S.C.",			"SC",
	"Alaska",			"Alas.",
	"Harbor",			"Hbr.",
	"Harbour",			"Hbr.",
	"mi.",			"mi",
	"Chan.",			"Ch.",
	"Beach",			"Bch.",
	"Bridge",			"Br.",
	"Shores",			"Shrs.",
	"Institute",		"Inst.",
	"Oceanographic",		"Oceanog.",
	"Pond",			"Pd.",
	"Lake",			"Lk.",
	"Vineyard",			"Viny'd",
	"Midchannel",		"Midchan.",
	"Shipyard",			"Shipyd.",
	"Station",			"Sta.",
	"Landing",			"Ldg.",
	"Sound",			"Snd.",
	"Bay of",			"B.",
	"Narrows",			"Narr.",
	"Light",			"Lt.",
	"Strait",			"Str.",
	"Creek",			"Ck.",
	"Islands",			"Is.",
	"Str.",			"St.",
	"Republic",			"Rep.",
	"Canal",			"Can.",
	"Fla.",			"FL",
	"Va.",			"VA",
	"Key",			"K.",
	"Marine Corps",		"USMC",
	"Management",		"Mgmt.",
	NULL, NULL };


static long amp[MAX_NUM_CSTS];
static long epoch[MAX_NUM_CSTS];

char FontWidths[2][256] = { 
 { 	5, 5, 5, 5, 5, 5, 5, 5, 5, 2, 5, 5, 5, 5, 5, 5, 
	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 
	2, 2, 4, 8, 6, 8, 7, 2, 4, 4, 6, 6, 3, 4, 2, 5, 
	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 2, 3, 6, 6, 6, 5, 
	8, 5, 5, 5, 6, 4, 4, 6, 6, 2, 4, 6, 5, 8, 6, 7, 
	5, 7, 5, 5, 6, 6, 6, 8, 6, 6, 6, 3, 5, 3, 6, 4, 
	3, 5, 5, 4, 5, 5, 4, 5, 5, 2, 3, 5, 2, 8, 5, 5, 
	5, 5, 4, 4, 4, 5, 5, 6, 6, 6, 4, 4, 2, 4, 7, 5, 
	5, 5, 3, 8, 5, 6, 6, 6, 4, 11, 5, 4, 8, 10, 10, 10, 
	10, 3, 3, 5, 5, 4, 4, 7, 7, 10, 4, 4, 8, 8, 8, 6, 
	2, 2, 6, 6, 8, 6, 2, 5, 4, 8, 5, 6, 6, 4, 8, 6, 
	5, 6, 4, 4, 3, 5, 6, 2, 4, 2, 5, 6, 8, 8, 8, 5, 
	5, 5, 5, 5, 5, 5, 7, 5, 4, 4, 4, 4, 3, 2, 3, 3, 
	7, 6, 7, 7, 7, 7, 7, 5, 8, 6, 6, 6, 6, 6, 6, 6, 
	5, 5, 5, 5, 5, 5, 8, 4, 5, 5, 5, 5, 2, 2, 3, 3, 
	5, 5, 5, 5, 5, 5, 5, 6, 7, 5, 5, 5, 5, 6, 5, 6, 
},
 { 	5, 5, 5, 5, 5, 5, 5, 5, 5, 2, 5, 5, 5, 5, 5, 5, 
	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 
	2, 3, 6, 10, 6, 13, 9, 5, 5, 6, 6, 3, 5, 3, 6, 
	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 3, 3, 6, 6, 6, 6, 
	10, 7, 7, 6, 7, 5, 5, 8, 8, 3, 5, 7, 6, 10, 7, 8, 
	7, 8, 8, 6, 7, 7, 8, 11, 7, 7, 7, 4, 6, 4, 6, 5, 
	4, 6, 6, 5, 6, 6, 5, 6, 6, 3, 4, 6, 3, 9, 6, 6, 
	6, 6, 5, 5, 6, 6, 6, 9, 6, 6, 5, 5, 3, 5, 7, 5, 
	6, 5, 3, 9, 5, 6, 5, 5, 4, 17, 6, 5, 10, 10, 10, 10, 
	2, 3, 6, 7, 8, 7, 3, 6, 4, 8, 6, 8, 6, 5, 8, 6, 
	5, 6, 4, 4, 4, 6, 7, 2, 4, 2, 6, 8, 9, 9, 9, 6, 
	7, 7, 7, 7, 7, 7, 9, 6, 5, 5, 5, 5, 3, 3, 3, 3, 
	6, 6, 6, 6, 6, 6, 9, 5, 6, 6, 6, 6, 3, 3, 3, 3, 
	6, 6, 6, 6, 6, 6, 6, 7, 8, 6, 6, 6, 6, 6, 6, 6, 
} };

FILE *	TideDB;
int numrecords;			/* Number of records in the database */
int noffsets = 0;
int ncurroffsets = 0;
int nhidden = 0;

#define TEMP_FILE "database.tmp"	/* Temporary file for pre-writing the data records */
static FILE * tfile = NULL;
static FILE * lfile = NULL;

#define BatchName "renamedb.bat"
char        TideLocsDBName[] = "temp-pdb";
#define TideAppID   "Ttbl"
#define TideDBType  "Ttbl"					/* Was Data until 2.1b2 */

static int font0 = 0, sw = 0;
char too_many_places = 0;

#if 0
static char *append(char *p, char *str);
static int sfpr(char *buf, float f, int width, int prec, int minus);
#endif

static void crevedonc(char *s) {
	fprintf(stderr,"Fatal Error in build.cpp: %s\n", s);
	barf(OTHER);
	}

long swab4(long l) {
	return (0xFF & (l >> 24)) | (0xFF00 & (l >> 8)) | ((0xFF00 & l) << 8) | ((0xFF & l) << 24);
	}

long swabf(float f) {
	union {float ff; long ll; } foo;
	foo.ff = f;
	return swab4(foo.ll);
	}

int swab2(int i) {
	return (0xFF & (i >> 8)) | ((0xFF & i) << 8);
	}

/**** Substitutes for Palm OS functions ****/
int font = 0;
void 	FntSetFont(int i) { font = i; }
int FntCharsWidth (char *c, int len) {
	int i = 0;
	while (len--) {
		i += FontWidths[font][*c++ & 0xff];
		}
	return i;
	}
long TimGetSeconds(void) {		/* Convert from UNIX time to Palm OS time */
	return 2082844800L + time(NULL); }

#define MAXCOMMAS 50
static unsigned char bestbreak;
/* Get width of c when it is broken into two strings for display */
static int stringwidth(char *c, int iscurrent, char comma) {
	int i, j, ncommas, width, mindif = 2000;
	char *p, *pp, *commas[MAXCOMMAS];
	bestbreak = 0;
	if (iscurrent) return FntCharsWidth (c, strlen(c));
	FntSetFont(1);
	if (!iscurrent && (width = FntCharsWidth (c, strlen(c))) > 160) {
		/* Find all commas */
		for (p = c, ncommas = 0; ncommas < MAXCOMMAS; ++ncommas )
			if (!*p || (p = commas[ncommas] = strchr(p,comma)) == NULL) 
				break; else ++p;
		/* Find the one which breaks the string most evenly */
		for (i = 0; i < ncommas; ++i) {
			*commas[i] = 0;
			if (*(pp = commas[i]+1) == ' ') ++pp;		/* Skip space after comma */
			j = FntCharsWidth(c,strlen(c)) -
				FntCharsWidth(pp,strlen(commas[i]+1));
			if (j < 0) j = -j;
			if (j < mindif) { 
				bestbreak = commas[i] - c;			/* Remember where break came */
				mindif = j; }
			*commas[i] = comma;
			}
		if (mindif != 2000) width = (mindif + width + 1) / 2;
		}
	return width;
}

static int filterit(char *s) {
	/* Rewrite location string or reject certain locations */
	/* Return 1 for filtered, -1 for reject else 0 */
	int a,b; char *p;
	for (a = b = 0; fitfilters[2*a]; ++a) {
	    while ((p = strstr(s, fitfilters[2*a])) != 0
			&& !isalpha(p[strlen(fitfilters[2*a])])) {/* Ignore if didn't match whole word */
		if (!fitfilters[2*a+1]) { b = -1; break; }	/* NULL means reject */
		strcpy(p, fitfilters[2*a+1]);				/* else substitute */
		strcpy(p + strlen(fitfilters[2*a+1]), p + strlen(fitfilters[2*a]));
		return 1;
	}	}
	return b;
}
static int cfilterit(char *s) {
	/* Rewrite location string or reject certain locations */
	/* Return 1 for cfiltered, -1 for reject else 0 */
	int a,b; char *p;
	for (a = b = 0; cfilters[2*a]; ++a) {
	    while ((p = strstr(s, cfilters[2*a])) != 0) {
		if (!cfilters[2*a+1]) { b = -1; break; }	/* NULL means reject */
		strcpy(p, cfilters[2*a+1]);			/* else substitute */
		strcpy(p + strlen(cfilters[2*a+1]), p + strlen(cfilters[2*a]));
		return 1;
	}	}
	return b;
}

/* Location database creation.
   Record 0 = header (see struct) 
   Remainder of records are:
	Record n+1: struct place for nth place (lat/long, current flag and name)
	Record n+2: data for the place
*/
static int diddit = 0;

struct LocDBHdr {			/* Header for the locations database files */
	char db_type;				/* Always 2 for place databases */
	char db_ID;					/* Unique identifier */
	time_t db_created_date;
	int db_num_places;			/* Number of visible places */
	int db_default_tide_place;
	int db_default_current_place;
	int db_num_currents;
	int db_num_offsets;
	int db_num_curroffsets;
	int db_num_total_places;		/* Number of total places (including hidden) */
	long latfrom, latto, longfrom, longto;
	int db_first_year, 			/* To check for compatibility */
		db_num_csts, db_num_nodes; 
	char db_name[MAXLOCSTR];		/* Name of database file */
	} LocDBHeader = {0};

static void MakeHeader(void) {
	/* Check size limitations not exceeded */
	if (nplaces >= (1 << ID_SHIFT) - 1) {
		fprintf(stderr,"Fatal error - number of places is %d, can not exceed %d\n",
			nplaces,(1 << ID_SHIFT) - 1);
		exit(1);
		}
	if (dbID & ~((1 << (16 - ID_SHIFT)) - 1) ) {
		fprintf(stderr,"Fatal error - database id is %d, can not exceed %d\n",
			dbID,((1 << (16 - ID_SHIFT)) - 1) );
		exit(1);
		}
	LocDBHeader.db_type = DBTYPE_LOC;
	LocDBHeader.db_ID = dbID;
	LocDBHeader.db_created_date = swab4(TimGetSeconds());
	LocDBHeader.db_num_places = swab2(nplaces - nhidden);
	LocDBHeader.db_default_tide_place = swab2(deftideindex);
	LocDBHeader.db_default_current_place = swab2(defcurrindex);
	LocDBHeader.db_num_currents = swab2(ncurrent);
	LocDBHeader.db_num_offsets = swab2(noffsets);
	LocDBHeader.db_num_curroffsets = swab2(ncurroffsets);
	LocDBHeader.db_num_total_places = swab2(nplaces);
	LocDBHeader.db_num_csts = swab2(num_csts);
	LocDBHeader.db_first_year = swab2(dumpyear);
	LocDBHeader.db_num_nodes = swab2(nyears);
	LocDBHeader.latfrom = swab4((long)(mlat * 3600));
	LocDBHeader.latto = swab4((long)(nlat * 3600));
	LocDBHeader.longfrom = swab4((long)(mlon * 3600));
	LocDBHeader.longto = swab4((long)(nlon * 3600));
	strncpy(LocDBHeader.db_name,dbNAM,MAXLOCSTR);
#ifdef OFFSETS
	LocDBHeader.db_num_places += NOFF;
	LocDBHeader.db_num_currents += NCOFF;
#endif
	if (fwrite(&LocDBHeader,sizeof(LocDBHeader),1,TideDB) != 1)
			crevedonc("Can't write DB header in database file");
	}

/*
**	Write the record entry (index) for a record with the given offset
*/
static void WriteRecordEntry(ULong whererec) {
	RecordEntryType r;
	static UInt index = 0;

	r.localChunkID = swab4(whererec);
	r.attributes = 0;
	r.uniqueID = 0;
	r.uniqueIDa = swab2(1 + index);
	if (fwrite(&r,sizeof(RecordEntryType),1,TideDB) != 1) 
		crevedonc("Could not write header record entry to database file");
	++index;
    }

/*
**	Write the header for a Palm OS database file
*/
static void WriteDBHeader(void) {
	DatabaseHdrType d = { 0};
	int i;
	int p = 0;
	long dataoffset;

	printf("Creating %s database file\n",dbNAM);
	strncpy(d.name,dbNAM,dmDBNameLength);
	d.attributes = 0;							/* Do not back up */
/* Note: If we ever allow user editing, that will need to be:
 *	d.attributes = swab2(dmHdrAttrBackup);
 */
	d.version = 0;
	d.lastBackupDate = d.creationDate = d.modificationDate = swab4(TimGetSeconds());
	d.modificationNumber = 0;
	d.appInfoID = d.sortInfoID = 0;
	strncpy((char *)&d.type,TideDBType,4);
	strncpy((char *)&d.creator, TideAppID,4);
	d.uniqueIDSeed = 0;
	numrecords = (2 * nplaces + 1);	/* Compute number of records in database */
	d.recordList.nextRecordListID = 0;
	d.recordList.numRecords = swab2(numrecords);
	if (fwrite(&d,sizeof(d),1,TideDB) != 1) crevedonc("Can't write header in database file");

	dataoffset = ftell(TideDB);		/* Compute offset of first data record */
	dataoffset += numrecords * sizeof (RecordEntryType) + sizeof(p);
	WriteRecordEntry(dataoffset);		/* Offset of first (tide header) record */
	dataoffset += sizeof(LocDBHeader);
	for (i = 0; i < nplaces; ++i) {
		WriteRecordEntry(dataoffset + placeinfo[i].record1);
		WriteRecordEntry(dataoffset + placeinfo[i].record2);
		}
	if (fwrite(&p,sizeof(p),1,TideDB) != 1) 	/* For some reason this is needed */
			crevedonc("Can't write header in database file");
	}
static int pla = 0, cur = 0, off = 0, offcur = 0;

int WriteLocationDatabase(void)
{	int c;
	FILE *b;

	if (too_many_places) {				/* Fatal error - don't make the file */
		fclose(tfile);
		unlink(TEMP_FILE);
		return 1;
		}

	unlink(BatchName);
	if (!(b = fopen(BatchName,"w"))) crevedonc("Can't create batch file.");
	fprintf(b,"echo off\n");
	fprintf(b,"del \"%s.pdb\"\n",dbNAM);
	fprintf(b,"rename %s \"%s.pdb\"\n",TideLocsDBName,dbNAM);
	fclose(b);

//  	sprintf(TideLocsDBName,"TideDB%dLocs",dbID);

	if (!(TideDB = fopen(TideLocsDBName,"wb"))) crevedonc("Can't create database file.");
	WriteDBHeader();
	MakeHeader();

	fseek(tfile,0,0);					/* Copy data records */
	while ((c = getc(tfile)) != EOF) putc(c,TideDB);
	fclose(TideDB);
	fclose(tfile);
	unlink(TEMP_FILE);

	printf("DBase %d generated: %d locations (including %d current and %d offsets, plus %d hidden)\n",
		dbID,nplaces-nhidden,ncurrent,noffsets,nhidden);
	if (sw) printf("\t%d names were too wide for the screen and will display truncated.\n",sw);
	if (font0) printf("\t%d location names were pretty long and will display in font 0\n", font0);
	/* Double check: */
	if (pla != nplaces || cur != ncurrent || off != noffsets) {
		printf("*** ERROR *** - Record flag counts disagree with header data!\n");
		printf("Flags show: %d places, %d currents, %d offsets, %d curr off\n",pla,cur,off,offcur);
		}
	printf("\n");
  	return 0;
}

static char ISCURRENT;

/*
**	Write a single database record onto the temp file.  
**		This happens during the pass through the harmonics file.
**		Then at the end, the data is copied over to the final database file.
*/
void WriteLocationRecord(int n, struct placeoff *offset)
{
  struct place *pp;
  struct placedat pd;
#define BFSIZ 256
  char buf[BFSIZ]; 			/* Actually used for variable size struct place */
  char buff[linelen];			/* Line buffer for input string */
	char savebuf[linelen];
	int len = 0;
  int i,j;
  long l;
  int font0wid, font1wid;
  LocalID lid;
  unsigned long date;
  char *p;
  int k, m, realdat;			/* For accumulating the non-zero data and mask */

  ++pla;
  assert(MAXLOCSTR + sizeof(struct place) < BFSIZ);
  pp = (struct place *)buf;
	if (!tfile) {			/* First time through, create temp file */
		unlink(TEMP_FILE);	/* In case it's there */
		if (!(tfile = fopen(TEMP_FILE,"wb+"))) 
			crevedonc("Can't create temporary database file.");
		}

	/* Pack latitude and longitude */
	l = 3600 * places[n].latitude;
	i = l % 10; if (l < 0) i = -i;
	l /= 10;
	assert (l <= 65535L && l >= -65535L);
	pp->latitude = swab2(l);
	l = 3600 * places[n].longitude ;
	j = l % 20; if (l < 0) j = -j;
	l /= 20;
	assert (l <= 65535L && l >= -65535L);
	pp->longitude = swab2(l);
	pp->lloflow = 10 * j + i;
	pp->absmax = swab2((int)(100 * (placeinfo[n].absmax + 0.01)));
	pp->absmin = swab2((int)(100 * (placeinfo[n].absmin - 0.01)));
	ISCURRENT = (places[n].iscurrent != 0);
	pp->flags = places[n].units | (ISCURRENT ? P_ISCURRENT : 0);
	if (places[n].issilent) {
		pp->flags |= P_HIDDEN;
		++nhidden;
		}
	pp->timezone = places[n].tzindex;

	strcpy(buff, places[n].location);
	if (offset) {
		pp->flags |= P_ISOFFSET;
		if (places[n].iscurrent) {
			++ncurrent;
			++ncurroffsets;
			}
		strcat(buff, "\260");						/* Mark offsets */
		}
	if (ISCURRENT) ++cur;
	if (pp->flags & P_ISOFFSET) { ++off; if (ISCURRENT) ++offcur; }
//printf("Location %d: (%x) %s%s\n",n,pp->flags ,places[n].location,(ISCURRENT)?" CURRENT" : "");

	while (1) {			/* Actually only do this once */
/* If location name too wide for screen, make it fit line in following order: 
	1. Bold font, nice abbreviations, break at commas (for tides which have 2 lines)
	2. Regular font, nice abbreviations, break at commas
	3. Same criteria except break at spaces.
	4. Regular font, break at spaces, allow nasty abbreviations (but as few as possible)
*/
		FntSetFont(i = 1);					/* Bold font */
		strcpy(savebuf,buff);					/* Save the input string */
		while ((len = stringwidth(buff,ISCURRENT, ',')) > 160) /* Try breaking at commas */
			if (!filterit(buff)) break;			/* Nice ones first */
		if (len <= 160) break;					/* If that works, return location of best break */

		FntSetFont(i = 0);					/* Normal Font */
		strcpy(buff,savebuf);					/* Restore input string */
		while ((len = stringwidth(buff,ISCURRENT, ',')) > 160) /* Try breaking at comma */
			if (!filterit(buff)) break;			/* Nice ones first */
		if (len <= 160) break;					/* If that works, return location of best break */

		FntSetFont(i = 1);					/* Bold font */
		strcpy(buff,savebuf);					/* Restore input string */
		while ((len = stringwidth(buff,ISCURRENT, ' ')) > 160)/* Try breaking at spaces */
			if (!filterit(buff)) break;			/* Nice ones first */
		if (len <= 160) break;					/* If that works, return location of best break */

		FntSetFont(i = 0);					/* Normal Font */
		strcpy(buff,savebuf);					/* Restore input string */
		while ((len = stringwidth(buff,ISCURRENT, ' ')) > 160)	/* Try breaking at space */
			if (!filterit(buff)) break;			/* Nice ones first */
		if (len <= 160) break;					/* If that works, return location of best break */
		while ((len = stringwidth(buff,ISCURRENT, ',')) > 160) /* Ugly abbreviations too - break at commas */
			if (!cfilterit(buff)) break;
		while ((len = stringwidth(buff,ISCURRENT, ' ')) > 160) /* and spaces */
			if (!cfilterit(buff)) break;
		break;
		}
	if (bestbreak) {							/* If the line has to be split */
		if (buff[bestbreak+1] == ' ') ++bestbreak; }	/* If break at comma, move to following space*/
//if (bestbreak && j)  {
//i = buff[bestbreak] ; buff[bestbreak] = '\\'; 
//printf("BREAK: %d %s\n",j,buff); buff[bestbreak] = i; }
	strcpy(pp->location, buff);
#if 0
	pp->split.loc = bestbreak;
	pp->split.font = i;
#else
	pp->split = bestbreak | (i << 7);
#endif

     	if (len > 160) {							/* If wide, report the fact */
		printf("ERROR: Location name \"%s\" %s too wide (%d), will display truncated\n",
			buff,(ISCURRENT)?"(CURRENT)" : "",len);
		sw++; }
	  else if (i == 0) {
//		printf("Warning: Location name \"%s\" too wide (%d), will display in font 0\n",buff,len);
		font0++; 
		}
	if (printplaces) {
		printf("%s", pp->location);
		if (ISCURRENT) printf(" (current)");
		printf(": %d\n",i+1);
		}
	if (listfile) {
		if (!lfile) {			/* First time through, open or create temp file */
			if (!(lfile = fopen(listfile,"a")) && !(lfile = fopen(listfile,"w"))) 
				crevedonc("Can't create listing file.");
			}
		if (!(pp->flags & P_HIDDEN))
		   fprintf(lfile, "%s\t%g\t%g\t%s\t%s\n", placeinfo[n].origlocation,places[n].latitude,
			places[n].longitude,dbNAM,offset ? placeinfo[offset->primary].origlocation : "");
		}
	placeinfo[n].record1 = ftell(tfile);	/* Remember offset of record 1 */
	if (fwrite(pp, sizeof(struct place) + strlen(pp->location),1,tfile) != 1) 
		crevedonc("Could not write record to database file");
	placeinfo[n].record2 = ftell(tfile);	/* Remember offset of record 2 */

	if (offset) {					/* Write an offset record 2 */
		offset->primary = swab2(offset->primary);
#if 0
		offset->minoff = swab2(offset->minoff);
		offset->maxoff = swab2(offset->maxoff);
#endif
		for (i = 1; i >= 0; i--) {
			offset->mulfactors[i] = swab2(offset->mulfactors[i]);
			offset->addfactors[i] = swab2(offset->addfactors[i]);
			}
		for (i = 3; i >= 0; i--) {
			offset->offsets[i] = swab2(offset->offsets[i]);
			}
		if (fwrite(offset, sizeof (struct placeoff),1,tfile) != 1) 
			crevedonc("Could not write record to database file");
		++noffsets;
		return;
		}

	pd.datum = swabf(places[n].datum);		/* Write a regular record */
	pd.meridian = places[n].meridian;

	/* Copy the actual data */
	for (j = 0; j < CST_ARRAY_SIZE; ++j) pd.datamask[j] = swab4(places[n].datamask[j]);
	pd.ndata = realdat = places[n].ndata;
	for (j = 0; j < realdat; ++j) {
		amp[j] = swabf(places[n].amp[j]);			/* Store a nonzero entry */
		/*** Now scale for cosine function here to save time ***/
		epoch[j] = swabf(places[n].epoch[j] * (1024.0 / 90.0));
		}
	if (fwrite(&pd, sizeof (pd),1,tfile) != 1) 
		crevedonc("Could not write record to database file");
	/* Got to write the data arrays separately */
	if (fwrite(amp,sizeof(amp[0]),realdat,tfile) != realdat) 
		crevedonc("Can't write data array in database file");
	if (fwrite(epoch,sizeof(epoch[0]),realdat,tfile) != realdat) 
		crevedonc("Can't write data array in database file");

	/* Now free up the data arrays */
	free(places[n].amp); places[n].amp = NULL;
	free(places[n].epoch); places[n].epoch = NULL;
}

char TideArgsDBName[dmDBNameLength] = "TideArgsDB";
struct ArgsDBHeader {		/* Header for tide factors database */
	char db_type;		/* Always 1 for factors DB */
	int params[4];		/* first_year, num_epochs, num_csts, num_nodes */
	time_t created;
	} argsheader = { DBTYPE_ARGS };

/*
**	Make factors database.  Simpler than locations because we know all record sizes.
*/
void WriteFactorsDatabase(void)
{
	DatabaseHdrType d = { 0};
	int i, y;
	int p = 0;
	long dataoffset;
	float ff;
	long f[MAX_NUM_CSTS];

	if (!(TideDB = fopen(TideArgsDBName,"wb"))) crevedonc("Can't create offsets file.");
	printf("Creating %s database file\n",dbNAM);
	strncpy(d.name,TideArgsDBName,dmDBNameLength);			/* Write database header
	d.attributes = 0;							/* Do not back up */
/* Note: If we ever allow user editing, that will need to be:
 *	d.attributes = swab2(dmHdrAttrBackup);
 */
	d.version = 0;
	d.lastBackupDate = d.creationDate = d.modificationDate = swab4(TimGetSeconds());
	d.modificationNumber = 0;
	d.appInfoID = d.sortInfoID = 0;
	strncpy((char *)&d.type,TideDBType,4);
	strncpy((char *)&d.creator, TideAppID,4);
	d.uniqueIDSeed = 0;
	numrecords = (2 * nyears + 2);	/* Compute number of records in database */
	d.recordList.nextRecordListID = 0;
	d.recordList.numRecords = swab2(numrecords);
	if (fwrite(&d,sizeof(d),1,TideDB) != 1) crevedonc("Can't write header in offsets file");

	dataoffset = ftell(TideDB);		/* Compute offset of first data record */
	dataoffset += (2 * nyears + 2) * sizeof (RecordEntryType) + sizeof(p);
	WriteRecordEntry(dataoffset);		/* Offset of first (tide header) record */
	dataoffset += sizeof(ArgsDBHeader);
	WriteRecordEntry(dataoffset);	
	for (i = 0; i < 2 * nyears; ++i) {
		dataoffset += num_csts * sizeof(float);
		WriteRecordEntry(dataoffset);		/* Offset of additional records */
		}
	if (fwrite(&p,sizeof(p),1,TideDB) != 1) 	/* For some reason this is needed */
			crevedonc("Can't write header in database file");

	argsheader.params[0] = swab2(dumpyear);		/* Write first record */
	argsheader.params[1] = swab2(nyears);
	argsheader.params[2] = swab2(num_csts);
	argsheader.params[3] = swab2(nyears);
	argsheader.created = swab4(TimGetSeconds());
	if (fwrite(&argsheader,sizeof(argsheader),1,TideDB) != 1) 
			crevedonc("Can't write header in offsets file");
	for (i = 0; i < num_csts; ++i) {
		ff = cst_speeds[i];
		f[i] = swabf(ff);
		}
	if (fwrite(&f, num_csts * sizeof(float),1,TideDB) != 1) 
			crevedonc("Can't write record in offsets file");
	for (y = 0; y < nyears; ++y) {
		for (i = 0; i < num_csts; ++i)  /*** Scale for cosine function here to save time ***/
			f[i] = swabf((float)(cst_epochs[i][dumpyear-first_year+y] * (1024.0 / 90.0)));
		if (fwrite(&f, num_csts * sizeof(float),1,TideDB) != 1) 
			crevedonc("Can't write record in offsets file");
		for (i = 0; i < num_csts; ++i) {
			f[i] = swabf((float)cst_nodes[i][dumpyear-first_year+y]);
			}
		if (fwrite(&f, num_csts * sizeof(float),1,TideDB) != 1) 
			crevedonc("Can't write record in offsets file");
		}
	fclose(TideDB);
	}		
