/*  Tide  TTY client for XTide
    Last modified 1/26/96

    MODIFIED VERSION TO DUMP HARMONIC DATA FOR USE IN PILOT XTIDE.

*/

#include "config.h"

#include <stdio.h>
#include <math.h>
#ifndef PILOT
#include <stdlib.h>
#include <signal.h>
// Not in Borland c #include <unistd.h>
#include <time.h>
#include <assert.h>
#endif
#include <string.h>
// Maybe timeb.h??  #include <sys/time.h>

#ifdef PILOT
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "sysinclude.h"
#endif

#include "tidelib.h"
#include "arglib.h"

/* External variables */
extern int num_csts, num_nodes, num_epochs, first_year;
extern char lastline[linelen];
extern char lastloc[linelen];

extern char hfile_name[MAXARGLEN+1], location[MAXARGLEN+1], *ppm,
  tzfile[MAXARGLEN+1], *geometry;
extern char *fgrise_color_arg, *fgfall_color_arg,
  *bg_color_arg, *fgtext_color_arg, *fgmark_color_arg,
  *fgmllw_color_arg, *fgmiddle_color_arg;
extern int tadjust, utc, list, checkyear, text, now, graphmode, httimeoff,
  lttimeoff, tstep, mark, middle, mllw, lines, skinny, PPMWIDTH,
  PPMHEIGHT, hinc, tinc, loctz, iscurrent, curonly, toplines, hincmagic,
  calendar, banner,precise;

/* CONTROL VARIABLES FOR DUMPING PALM PILOT DATA FILE */
char dumpdata = 0;    	/* Flag to dump place data for Pilot to compile in */
char dumpfactors = 0;   	/* Flag to dump factors data for Pilot to compile in */
int dumpyear = 0;		/* Year to start dumping data */
int nyears = 0;		/* How many years to dump */
double latfrom = 37.5;	/* Bounds of places to dump */
double latto = 38.5;	/*  (eventually these must be command line args) */
double longfrom = -122.0;
double longto = -123.0;
int dbID = 0;
char dbNAM[MAXLOCSTR] = "Database Name";
char deftidename[MAXLOCSTR] , defcurrname[MAXLOCSTR] ;
char *exclude[10];
char *excluden[10];
char *include[10];
char *offfiles[10];
int nexclude = 0, nexcluden = 0, ninclude = 0, nofffiles = 0;
char printplaces = 0;
char *listfile = NULL, listflag = 0;		/* Flag = 1 to list included files, 2 for all */

extern time_t faketime;
extern double htleveloff, ltleveloff, marklev;

#define getarg if (!(arg = poparg())) barf (WANTMOREARGS)
#define getint(a) if (sscanf (arg, "%d", &a) != 1) barf (BADINTEGER)
#define getfloat(a) if (sscanf (arg, "%lf", &a) != 1) {fprintf(stderr,"\nNot a float: \"%s\"\n",arg);barf(BADFLOAT);}

void
main(int argc, char **argv)
{
  char *arg;
  int argl;

  /* Exit without core dump on assertion failures. */
//  signal (SIGABRT, sigabort);

  /* From config.h */
  assert ((int)(strlen (deflocation)) <= MAXARGLEN);
  strcpy (location, deflocation);
  assert ((int)(strlen (hfile)) <= MAXARGLEN);
  strcpy (hfile_name, hfile);
  /* Check environment */
  if ((arg = getenv ("LOCATION")) != 0) {
    assert ((int)(strlen (arg)) <= MAXARGLEN);
    strcpy (location, arg);
  }
  if ((arg = getenv ("HFILE")) != 0) {
    assert ((int)(strlen (arg)) <= MAXARGLEN);
    strcpy (hfile_name, arg);
  }

  text = -1;
  push_command_line (argc, argv);
  /* Read default config file only if none is explicitly specified */
  for (argl=1;argl<argc;argl++) {
    if (!strcmp (argv[argl], "-config")) {
      argl = 0;
      break;
    }
  }
  /* Try to read file specified by TIDERC */
  if (argl) {
    if ((arg = getenv ("TIDERC")) != 0) {
      if (push_config_file (arg))
        argl = 0;
    }
  }
  /* Try to read ~/.tiderc */
  if (argl) {
    if ((arg = getenv ("HOME")) != 0) {
      char tfile[MAXARGLEN+1];
      assert ((int)(strlen (arg)) + 8 <= MAXARGLEN);
      sprintf (tfile, "%s/.tiderc", arg);
      if (push_config_file (tfile))
        argl = 0;
    }
  }
  /* If failed, try to read system config file */
  if (argl)
    push_config_file (sysconfig);

  while ((arg = poparg ()) != 0) {
    if (!strcmp (arg, "-hfile")) {
      getarg;
      strcpy (hfile_name, arg);
#ifndef COMPILEDATA
    } else if (!strcmp (arg, "-dump")) {  /* Pilot arg */
      dumpdata = 1;
    } else if (!strcmp (arg, "-dumpfactors")) {  /* Pilot arg */
      dumpfactors = 1;
    } else if (!strcmp (arg, "-fyear")) {
      getarg;
      getint (dumpyear);
    } else if (!strcmp (arg, "-nyears")) {
      getarg;
      getint (nyears);
    } else if (!strcmp (arg, "-ltf")) {
      getarg;
      getfloat (latfrom);
    } else if (!strcmp (arg, "-ltt")) {
      getarg;
      getfloat (latto);
    } else if (!strcmp (arg, "-lnf")) {
      getarg;
      getfloat (longfrom);
    } else if (!strcmp (arg, "-lnt")) {
      getarg;
      getfloat (longto);
    } else if (!strcmp (arg, "-dID")) {
      getarg;
      getint (dbID);
    } else if (!strcmp (arg, "-dNM")) {
      getarg;
	if (strlen(arg) >= MAXLOCSTR) {
		fprintf(stderr,"DB name too long: %s\n",arg); exit(-1);
		}
      strcpy(dbNAM, arg);
    } else if (!strcmp (arg, "-dtide")) {
      getarg;
      strcpy(deftidename , arg);
    } else if (!strcmp (arg, "-pp")) {
      printplaces = 1;
    } else if (!strcmp (arg, "-dcur")) {
      getarg;
      strcpy(defcurrname , arg);
    } else if (!strcmp (arg, "-xn")) {
      getarg;
      excluden[nexcluden++] = strdup(arg);
//fprintf(stderr,"\nExclude not %d = <%s>\n",nexcluden-1,excluden[nexcluden-1]);
    } else if (!strcmp (arg, "-x")) {
      getarg;
      exclude[nexclude++] = strdup(arg);
    } else if (!strcmp (arg, "-of")) {
      getarg;
      offfiles[nofffiles++] = strdup(arg);
    } else if (!strcmp (arg, "-i")) {
      getarg;
      include[ninclude++] = strdup(arg);
    } else if (!strcmp (arg, "-lf")) {
      getarg;
      listfile = strdup(arg);
	listflag = 1;
    } else if (!strcmp (arg, "-precise")) {
      precise = 1;
#endif
    } else if (!strcmp (arg, "-check")) {
      getarg;
      getint (checkyear);
    } else if (!strcmp (arg, "-ppm")) {
      getarg;
      if (ppm)
        free (ppm);
      ppm = stradoop (arg);
    } else if (!strcmp (arg, "-config") || !strcmp (arg, "-cf")) {
      getarg;
      if (!(push_config_file (arg)))
        barf (CONFIGFAIL);
    } else if (!strcmp (arg, "-gstart")) {
      getarg;
      faketime = parse_time_string (arg);
    } else if (!strcmp (arg, "-location")) {
      getarg;
      strcpy (location, arg);
    } else if (!strcmp (arg, "-text")) {
      if (dataarg ()) {
        getarg;
        getint (text);
      }
    } else if (!strcmp (arg, "-hinc")) {
      if (dataarg ()) {
        getarg;
        getint (hinc);
      }
      if (hinc < 1)
        hincmagic = 1;
    } else if (!strcmp (arg, "-tinc")) {
      if (dataarg ()) {
        getarg;
        getint (tinc);
      }
      if (tinc < 1)
        tinc = 1;
    } else if (!strcmp (arg, "-tz")) {
      getarg;
      tadjust = hhmm2seconds (arg);
      if (!tadjust)
        utc = 1;
    } else if (!strcmp (arg, "-list")) {
      list = 1;
    } else if (!strcmp (arg, "-listtz")) {
      list = 2;
    } else if (!strcmp (arg, "-loctz")) {
      loctz = 1;
    } else if (!strcmp (arg, "-graph")) {
      graphmode = 1;
    } else if (!strcmp (arg, "-gstretch")) {
      double gstretch;
      getarg;
      getfloat (gstretch);
      if (gstretch <= 0.0)
        barf (STRETCHTOOSMALL);
      tstep = (int)(180.0 / gstretch);
      if (tstep <= 0)
        barf (STRETCHTOOBIG);
    } else if (!strcmp (arg, "-utc")) {
      utc = 1;
    } else if (!strcmp (arg, "-mark")) {
      mark = 1;
      getarg;
      getfloat (marklev);
    } else if (!strcmp (arg, "-middle")) {
      middle = 1;
    } else if (!strcmp (arg, "-mllw")) {
      mllw = 1;
    } else if (!strcmp (arg, "-nolines")) {
      lines = 0;
    } else if (!strcmp (arg, "-now")) {
      now = 1;
    } else if (!strcmp (arg, "-calen")) {
      calendar = 1;
    } else if ((!strcmp (arg, "-geometry"))||(!strcmp (arg, "-ppmgeom"))) {
      getarg;
      if (geometry)
        free (geometry);
      geometry = stradoop (arg);
    } else if (!strcmp (arg, "-cur")) {
      curonly = 1;
    } else if (!strcmp (arg, "-toplines")) {
      toplines = 1;
    } else if (!strcmp (arg, "-skinny")) {
      skinny = 1;
    } else if (!strcmp (arg, "-banner")) {
      banner = 1;
    } else if (!strcmp (arg, "-hloff")) {
      getarg;
      getfloat (htleveloff);
    } else if (!strcmp (arg, "-lloff")) {
      getarg;
      getfloat (ltleveloff);
    } else if (!strcmp (arg, "-htoff")) {
      getarg;
      httimeoff = hhmm2seconds (arg);
    } else if (!strcmp (arg, "-ltoff")) {
      getarg;
      lttimeoff = hhmm2seconds (arg);
    } else if (!strcmp (arg, "-bg")) {
      getarg;
      if (bg_color_arg)
        free (bg_color_arg);
      bg_color_arg = stradoop(arg);
    } else if (!strcmp (arg, "-fgrise")) {
      getarg;
      if (fgrise_color_arg)
        free (fgrise_color_arg);
      fgrise_color_arg = stradoop(arg);
    } else if (!strcmp (arg, "-fgfall")) {
      getarg;
      if (fgfall_color_arg)
        free (fgfall_color_arg);
      fgfall_color_arg = stradoop(arg);
    } else if (!strcmp (arg, "-fgtext")) {
      getarg;
      if (fgtext_color_arg)
        free (fgtext_color_arg);
      fgtext_color_arg = stradoop(arg);
    } else if (!strcmp (arg, "-fgmark")) {
      getarg;
      if (fgmark_color_arg)
        free (fgmark_color_arg);
      fgmark_color_arg = stradoop(arg);
    } else if (!strcmp (arg, "-fgmllw")) {
      getarg;
      if (fgmllw_color_arg)
        free (fgmllw_color_arg);
      fgmllw_color_arg = stradoop(arg);
    } else if (!strcmp (arg, "-fgmiddle")) {
      getarg;
      if (fgmiddle_color_arg)
        free (fgmiddle_color_arg);
      fgmiddle_color_arg = stradoop(arg);
    } else if (!strcmp (arg, "-version")) {
      if (PATCHLEVEL)
        printf ("Tide %s.%d\n", VERSION, PATCHLEVEL);
      else
        printf ("Tide %s\n", VERSION);
      exit (0);
    } else {
fprintf(stderr, "\nBad arg: %s\n",arg); 
if (!strcmp(arg,"-h") || !strcmp(arg,"-help") || !strcmp(arg,"-?"))
      printf (
"Usage:  tide [-banner]              Sideways ASCII tide graph\n\
             [-bg color]            Background color\n\
             [-calen]               Make 1 month tide calendar (slow)\n\
             [-check YYYY]          Check for errors in equilibrium arguments\n\
             [-config {filename | -}]  Read configuration file / stdin\n\
             [-cur]                 Restrict -location search to currents\n\
             [-fgfall color]        Color of ebb current\n\
             [-fgmark color]        Color of mark line\n\
             [-fgmiddle color]      Color of middle line\n\
             [-fgmllw color]        Color of mllw line\n\
             [-fgrise color]        Color of normal graph / flood current\n\
             [-fgtext color]        Color of text and lines\n\
             [-geometry NxN]        PPM geometry\n\
             [-graph]               ASCII graph mode\n\
             [-gstart YYYY:MM:DD:HH:MM]  Time at which to start\n\
             [-gstretch N.NN]       Adjust graph detail vs. time span\n\
             [-hfile ...]           Location and name of harmonics file\n\
             [-hinc [N]]            Label depth marks in increments of N\n\
             [-hloff [-]N.NN]       (Text)  Tide level offset for high tide\n\
             [-htoff [-]HH:MM]      (Text)  Time offset for high tide\n\
             [-list]                List all locations in harmonics file\n\
             [-listtz]              List with meridians and time zone info\n\
             [-lloff [-]N.NN]       (Text)  Tide level offset for low tide\n\
             [-location \"Name\"]     Location to show tides for\n\
             [-loctz]               Try to use time zone of location (Moof!)\n\
             [-ltoff [-]HH:MM]      (Text)  Time offset for low tide\n\
             [-mark [-]N.NN]        Draw line at specified tide level\n\
             [-middle]              Draw line near mean tide level\n\
             [-mllw]                Draw line at mean lower low water level\n\
             [-nolines]             Suppress tick marks / depth lines\n\
             [-now]                 Mark current time on graph\n\
             [-ppm {filename | -}]  Write graph as PPM to file / stdout\n\
             [-skinny]              Abbreviate timestamps\n\
             [-text N]              List N tides to standard output\n\
             [-tinc [N]]            Label each N hours\n\
             [-toplines]            (PPM)  Depth lines on top of graph\n\
             [-tz [-]HH:MM]         Fixed time zone offset for timestamps\n\
             [-utc]                 Show timestamps in UTC\n\
             [-version]             Print Tide version\n\
\n\
Time format example -- half past midnight, June 1, 1995:  1995:06:01:00:30\n\
Colors must be specified as rgb:hh/hh/hh (where hh is a 2-digit hexadecimal\n\
number)\n\
\n\
	Arguments for generating Tide Tool databases: \n\
		[-dump]			Create a locations database file\n\
		[-dumpfactors]		Create C source for a TideArgs file\n\
		[-fyear N]			Start from year N\n\
		[-nyears N]			Number of years in database = N\n\
		[-ltf N] [-ltt N]		Set latitude limits to include\n\
		[-lnf N] [-lnt N]		Set longitude limits to include\n\
		[-dID N]			Unique database ID number\n\
		[-dNM namestr]		Database description string\n\
		[-dtide substr]		Default tide location (substring)\n\
		[-dcur substr]		Default current location (substring) \n\
		[-x substr]			Exclude eligible locations with this substring\n\
		[-xn substr]		Exclude eligible locations without this substr\n\
		[-i substr]			Include ineligible locations with this substr\n\
		[-pp]				Print place names included\n\
            [-lf filename]		List place names included on file\n\
            [-la filename]		List all place names in database on file\n\
            [-cf filename]		Read configuration file\n\
		[-of filename]		Read offsets file (up to 10)\n\
Up to 10 include and exclude substrings can be specified.\n\
Note there is a limit on command line length so -cf is used a lot.\n\
");
      exit (0);
    }
  }
  /* Remove glaring inconsistencies in args */
  if (!lines)
    hinc = tinc = 0;
  if (banner) {
    graphmode = 0;
    ppm = NULL;
  }
  if (graphmode && ppm)
    graphmode = 0;
  if (graphmode || ppm) {
    calendar = 0;
    if (!hinc)
      hincmagic = 1;
  }
  if (calendar || graphmode)
    skinny = 1;
  if ((httimeoff || lttimeoff || htleveloff != 0.0 || ltleveloff != 0.0)
      && (graphmode || ppm || banner || mark))
    barf (OFFSETSTEXTONLY);

  /* Adjust tstep for ASCII graph mode */
  if (graphmode)
    tstep *= AGTSTEP;
  /* Adjust tstep for banner mode */
  if (banner)
    tstep *= BANTSTEP;

  /* Parse geometry for PPM mode (it's needed here because of -now) */
  if (geometry && ppm) {
    if (sscanf (geometry, "%dx%d", &PPMWIDTH, &PPMHEIGHT) != 2)
      barf (BADGEOMETRY);
    if ((PPMWIDTH <= 20)||(PPMHEIGHT <= 20))
      barf (BADGEOMETRY);
    PPMHEIGHT -= 12; /* We take 12 extra, so compensate. */
  }

  if (!faketime) {
    faketime = time (NULL);
    /* Center graph around current time if -now */
    if (graphmode && now)
      faketime -= tstep * (TEXTWIDTH>>1);
    else if (ppm && now)
      faketime -= tstep * (PPMWIDTH>>1);
  }

  load_data ();
  if (checkyear) {
    check_epoch ();
    exit (0);
  }
  if (loctz)
    change_time_zone (tzfile); /* Moof! */
  if (iscurrent) {
    mark = 1;
    marklev = 0.0;
  }
  if (ppm) {
    tide2ppm (ppm);
    exit (0);
  }
  if (banner) {
    do_banner ();
    exit (0);
  }
  if (graphmode) {
    tide2ascii ();
    exit (0);
  }
  if (calendar) {
    do_calendar ();
    exit (0);
  }
  list_tides ();
  exit (0);
}
/* User-friendly error messages */
void
barf (enum tideerr err)
{
  fprintf(stderr, "******* TIDELIB FATAL ERROR *********\n");
  printf("Tidelib Fatal Error:  ");
  switch (err) {
  case BADCOLORSPEC:
    printf("BADCOLORSPEC\n");
    printf(
"One of your specified colors is not understood.  This may be because it is\n\
not in rgb.txt, or because you used the wrong syntax.  In PPM mode, colors\n\
must be specified as rgb:hh/hh/hh, where hh is a 2-digit hexadecimal number.\n\
Other formats are possible under X; type 'man X' for an explanation of them.\n");
    break;
  case BADGEOMETRY:
    printf("BADGEOMETRY\n");
    printf(
"In PPM mode, geometry must be specified as WxH, where W and H are width and\n\
height in pixels respectively.  Other formats are possible under X; type\n\
'man X' for an explanation of them.  This error is also generated if the\n\
geometry that you specify is too small or too large in one or both directions.\n");
    break;
  case CANTOPENDISPLAY:
    printf("CANTOPENDISPLAY\n");
    printf(
"Can't open the X display.  Check the setting of the DISPLAY environment\n\
variable or the value supplied with the -display switch.  You may also need\n\
to fix your X authorities with xhost or xauth.\n");
    break;
  case WANTMOREARGS:
    printf("WANTMOREARGS\n");
    printf(
      "Your last switch wants an argument that you did not provide.\n");
    break;
  case CONFIGFAIL:
    printf("CONFIGFAIL\n");
    perror ("fopen");
    printf(
      "Could not open the config file that you specified.\n");
    break;
  case CANTOPENFILE:
    printf("CANTOPENFILE\n");
    perror ("fopen");
    printf(
      "Could not open a file.\n");
    break;
  case BADINTEGER:
    printf("BADINTEGER\n");
    printf(
      "One of your switches wants an integer but didn't get one.\n");
    break;
  case BADFLOAT:
    printf("BADFLOAT\n");
    printf(
  "One of your switches wants a floating point number but didn't get one.\n");
    break;
  case BADHHMM:
    printf("BADHHMM\n");
    printf(
"Offset must be of the form [-]HH:MM where HH is hours and MM is minutes.\n"
    );
    break;
  case STRETCHTOOSMALL:
    printf("STRETCHTOOSMALL\n");
    printf("-gstretch needs a positive value.\n");
    break;
  case STRETCHTOOBIG:
    printf("STRETCHTOOBIG\n");
    printf("-gstretch wants a smaller number.\n");
    break;
  case BADTIMESTAMP:
    printf("BADTIMESTAMP\n");
    printf(
"-gstart wants a timestamp of the form YYYY:MM:DD:HH:MM.  Example -- half past\n\
midnight, June 1, 1995:  1995:06:01:00:30.  This error may also be raised if\n\
you specify a non-existent time, such as February 31st or the hour that is\n\
skipped because of Daylight Savings Time, or go outside the acceptable range\n\
of dates (i.e. before 1970 or after 2037 on most Unix machines).\n");
    break;
  case MISSINGYEAR:
    printf("MISSINGYEAR\n");
    printf(
"In order to calculate tides for a given year, the node factors and\n\
equilibrium arguments for that year must be in the harmonics file.  The\n\
harmonics file that you are using only supports the following years:\n");
    if (num_nodes < num_epochs)
      num_epochs = num_nodes;
    printf("    %d through %d\n", first_year,
      first_year+num_epochs-1);
    break;
  case OFFSETSTEXTONLY:
    printf("OFFSETSTEXTONLY\n");
    printf(
"-hloff, -htoff, -lloff, and -ltoff are only supported in text mode and cannot\n\
be used with -graph, -ppm, -mark, or -banner.\n");
    break;
  default:
    assert (0);
  }
  if (*lastline) printf("Last input line: %s\n",lastline);
  printf("Last location read was %s\n",lastloc);
  exit (-1);
}


