#include <time.h>
#include "tidelib.h"

/* IMPORTANT: Changes in the first block must also appear in the tidetool source! **/

struct TimeZone Zones[] = {
	"America/Pacific", 	"PST", "PDT", -8*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"America/Central", 	"CST", "CDT", -6*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"America/Eastern", 	"EST", "EDT", -5*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"US/Alaska", 		"AKST","AKDT",-9*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"US/Hawaii-Aleutian",	"HAST","HADT",-10*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"America/Mountain", 	"MST", "MDT", -7*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"Canada/Pacific", 	"PST", "PDT", -8*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"America/Bogota", 		"COST","CODT", -5*60,60,12,0*60,ON_DATE,31,5,0*60,ON_DATE,2,
	"Brazil/East",   		"EST","EDT", -3*60,60,10,0*60,ON_AFTER|dSun,1, 2,0*60,ON_AFTER|dSun,22,
	"Atlantic/Stanley", 	"FKST","FKDT", -4*60,60,9,0*60,ON_AFTER|dSun,8, 4,0*60,ON_AFTER|dSun,16,
	"America/Santiago", 	"CLST","CLDT", -4*60,60,10,0*60,ON_AFTER|dSun,9, 3,0*60,ON_AFTER|dSun,9,
	"America/Atlantic", 	"AST", "ADT", -4*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"Canada/Newfoundland", 	"NST", "NDT", -(3*60+30),60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"America/Godthab", 	"WGST","WGDT",-3*60,60,4,2*60,ON_AFTER|dSun,1,10,2*60,ON_AFTER|dSun,25,
	"Greenwich Mean Time",	"GMT", "",     0,0,0,0,0,0,0,0,0,0,
	"Europe/London",   	"GMT", "BST",        0,60,3,1*60,ON_AFTER|dSun,25, 10,1*60,ON_AFTER|dSun,25,
	"Europe/Dublin",   	"GMT", "IST",        0,60,3,1*60,ON_AFTER|dSun,25, 10,1*60,ON_AFTER|dSun,25,
	"Atlantic/Azores",   	"AZOST","AZODT",-1*60,60,3,2*60,ON_AFTER|dSun,25, 10,2*60,ON_AFTER|dSun,25,
	"Western Europe",   	"WEST","WEDT",    0,60,3,2*60,ON_AFTER|dSun,25, 10,2*60,ON_AFTER|dSun,25,
	"Central Europe",   	"CEST","CEDT", 1*60,60,3,2*60,ON_AFTER|dSun,25, 10,2*60,ON_AFTER|dSun,25,
	"Europe/Moscow",   	"MSK","MSD",   3*60,60,3,2*60,ON_AFTER|dSun,25, 10,2*60,ON_AFTER|dSun,25,
	"Egypt",   			"EEST","EEDT",   2*60,60,4,0*60,ON_AFTER|dFri,24, 9,0*60,ON_AFTER|dFri,24,
	"Asia/Tehran (appr.)", 	"IRST","IRDT", 3*60+30,60,3,0*60,ON_DATE,21, 9,0*60,ON_DATE,23,
	"Asia/Baghdad",   	"AST","ADT",   3*60,60,4,3*60,ON_DATE,1, 10,3*60,ON_DATE,1,
	"Namibia", 			"SAST", "SADT", 2*60,60,4,2*60,ON_AFTER|dSun,1,9,2*60,ON_AFTER|dSun,1,
	"Asia/Vladivostok",   	"VLAST","VLADT",10*60,60,3,2*60,ON_AFTER|dSun,25, 10,2*60,ON_AFTER|dSun,25,
	"Asia/Magadan",   	"MAGST","MAGDT",11*60,60,3,2*60,ON_AFTER|dSun,25, 10,2*60,ON_AFTER|dSun,25,
	"Asia/Kamchatka",   	"PETST","PETDT",12*60,60,3,2*60,ON_AFTER|dSun,25, 10,2*60,ON_AFTER|dSun,25,
	"Pacific/Auckland",	"NZST", "NZDT",12*60,60,10,2*60+45,ON_AFTER|dSun,1,3,2*60+45,ON_AFTER|dSun,25,
	"Australia/Eastern", 	"EST", "", 10*60,0,0,0,0,0,0,0,0,0,
	"Australia/Central", 	"CST", "", 9*60+30,0,0,0,0,0,0,0,0,0,
	"Australia/Western", 	"WST", "", 8*60,0,0,0,0,0,0,0,0,0,
	"Australia/S.Eastern",	"EST", "EDT", 10*60,60,10,2*60,ON_AFTER|dSun,25,3,2*60,ON_AFTER|dSun,25,
	"Australia/S.Central",	"CST", "CDT",9*60+30,60,10,2*60,ON_AFTER|dSun,25,3,2*60,ON_AFTER|dSun,25,

/* Note that from here on in, these zones are computed, not stored in database */
	"GMT-11", 			"GMT-11", "", -11*60,0,0,0,0,0,0,0,0,0,
	"GMT-10", 			"GMT-10", "", -10*60,0,0,0,0,0,0,0,0,0,
	"GMT-9:30", 		"GMT-9+", "", -9*60+30,0,0,0,0,0,0,0,0,0,
	"GMT-9", 			"GMT-9", "", -9*60,0,0,0,0,0,0,0,0,0,
	"GMT-7", 			"GMT-7", "", -6*60,0,0,0,0,0,0,0,0,0,
	"GMT-6", 			"GMT-6", "", -6*60,0,0,0,0,0,0,0,0,0,
	"GMT-5", 			"GMT-5", "", -5*60,0,0,0,0,0,0,0,0,0,
	"GMT-4", 			"GMT-4", "", -4*60,0,0,0,0,0,0,0,0,0,
	"GMT-3", 			"GMT-3", "", -3*60,0,0,0,0,0,0,0,0,0,
	"GMT-2", 			"GMT-2", "", -2*60,0,0,0,0,0,0,0,0,0,
	"GMT-1", 			"GMT-1", "", -1*60,0,0,0,0,0,0,0,0,0,
	"GMT+0", 			"GMT+0", "", 0,0,0,0,0,0,0,0,0,0,
	"GMT+1", 			"GMT+1",  "", 1*60,0,0,0,0,0,0,0,0,0,
	"GMT+2", 			"GMT+2",  "", 2*60,0,0,0,0,0,0,0,0,0,
	"GMT+3", 			"GMT+3",  "", 3*60,0,0,0,0,0,0,0,0,0,
	"GMT+4", 			"GMT+4",  "", 4*60,0,0,0,0,0,0,0,0,0,
	"GMT+5", 			"GMT+5",  "", 5*60,0,0,0,0,0,0,0,0,0,
	"GMT+5:30", 		"GMT+5+",  "", 5*60+30,0,0,0,0,0,0,0,0,0,
	"GMT+6", 			"GMT+6",  "", 6*60,0,0,0,0,0,0,0,0,0,
	"GMT+6:30", 		"GMT+6+",  "", 6*60+30,0,0,0,0,0,0,0,0,0,
	"GMT+7", 			"GMT+7",  "", 7*60,0,0,0,0,0,0,0,0,0,
	"GMT+8", 			"GMT+8",  "", 8*60,0,0,0,0,0,0,0,0,0,
	"GMT+9", 			"GMT+9",  "", 9*60,0,0,0,0,0,0,0,0,0,
	"GMT+10", 			"GMT+10", "", 10*60,0,0,0,0,0,0,0,0,0,
	"GMT+10:30", 		"UT+10+",  "", 10*60+30,0,0,0,0,0,0,0,0,0,
	"GMT+11", 			"GMT+11", "", 11*60,0,0,0,0,0,0,0,0,0,
	"GMT+11:30", 		"UT+11+",  "", 11*60+30,0,0,0,0,0,0,0,0,0,
	"GMT+12", 			"GMT+12", "", 12*60,0,0,0,0,0,0,0,0,0,
	"GMT+13", 			"GMT+13", "", 13*60,0,0,0,0,0,0,0,0,0,
  	"" };

char *TZequivs[250][2] = {
	"America/Los_Angeles",	"America/Pacific",
	"US/Central",		"America/Central",
	"America/Mazatlan",	"Mexico/BajaSur",
	"America/Tijuana",	"Mexico/BajaNorte",
	"America/Vancouver",	"America/Pacific",
	"America/Edmonton",	"America/Mountain",
	"America/New_York",	"America/Eastern",
	"America/Montreal",	"America/Eastern",
	"Canada/Eastern",		"America/Eastern",
	"Canada/Atlantic",	"America/Atlantic",
	"America/Goose_Bay",	"America/Atlantic",
	"US/Eastern",		"America/Eastern",
	"US/Pacific",		"America/Pacific",
	"America/Whitehorse",	"America/Pacific",
	"America/Yellowknife",	"America/Mountain",
	"America/Winnipeg",	"America/Central",
	"America/Halifax",	"America/Atlantic",
	"America/Iqaluit",	"America/Central", 
	"America/Havana",		"America/Eastern",
	"America/St_Johns",	"Canada/Newfoundland",
	"America/Nassau",		"America/Eastern",
	"America/Port-au-Prince","America/Eastern",
	"America/Grand_Turk",	"America/Eastern",
	"America/St_Kitts",	"GMT-4",
	"America/St_Lucia",	"GMT-4",
	"America/Santo_Domingo","GMT-4",
	"America/Guadeloupe",	"GMT-4",
	"America/Antigua",	"GMT-4",
	"America/Barbados",	"GMT-4",
	"Atlantic/Bermuda",	"America/Atlantic",
	"America/St_Thomas",	"GMT-4",
	"America/Grenada",	"GMT-4",
	"America/Caracas",	"GMT-4",
	"America/Port_of_Spain","GMT-4",
	"America/Martinique",	"GMT-4",
	"America/Aruba",		"GMT-4",
	"America/St_Vincent",	"GMT-4",
	"America/Curacao",	"GMT-4",
	"America/Puerto_Rico",	"GMT-4",
	"America/Martinique",	"GMT-4",
	"America/Jamaica",	"GMT-5",
	"America/Bogota",		"GMT-5",
	"America/Panama",		"GMT-5",
	"America/Costa_Rica",	"GMT-6",
	"America/Belize",		"GMT-6",
	"America/Guatemala",	"GMT-6",
	"America/El_Salvador",	"GMT-6",
	"America/Tegucigalpa",	"America/Central",
	"America/Managua",	"GMT-6",
	"America/Hermosillo",	"GMT-7", 
	"America/Ensenada",	"America/Pacific",
	"Mexico/BajaNorte",	"America/Pacific",
	"Mexico/BajaSur",		"America/Mountain",
	"Mexico/General",		"America/Central",
	"Pacific/Galapagos",	"GMT-6",
	"America/Guayaquil",	"GMT-5",
	"America/Lima",		"GMT-5",
	"America/Panama",		"GMT-5",
	"America/Guyana",		"GMT-4",
	"America/Belem",		"GMT-3",
	"America/Fortaleza",		"GMT-3",
	"America/Maceio",	"Brazil/East",
	"America/Sao_Paulo",	"Brazil/East",
	"America/Noronha",	"GMT-2",
	"America/Buenos_Aires",	"GMT-3",
	"America/Paramaribo",	"GMT-3",
	"America/Cayenne",	"GMT-3",
	"America/Montevideo",	"GMT-3",
	"Atlantic/South_Georgia", "GMT-2",
	"Antarctica/Palmer",	"GMT-4",
	"America/Mexico_City",	"America/Central",
	"MST7",			"America/Central", 		/* For Sonora, which appears to use DST */
	"America/Scoresbysund",	"Atlantic/Azores",
	"America/Thule",		"America/Atlantic", 
	"Atlantic/Reykjavik",	"Greenwich Mean Time",
	"Iceland",			"Greenwich Mean Time",
	"UTC",			"Greenwich Mean Time",
	"Etc/UTC",			"Greenwich Mean Time",
	"America/Chicago",	"America/Central",
	"America/Anchorage",	"US/Alaska",
	"America/Atka", 		"US/Hawaii-Aleutian",
	"Europe/Belfast",		"Europe/London",
	"Europe/Paris",		"Central Europe",
	"Europe/Brussels",	"Central Europe",
	"Europe/Oslo",		"Central Europe",
	"Europe/Berlin",		"Central Europe",
	"Europe/Amsterdam",	"Central Europe",
	"Europe/Copenhagen",	"Central Europe",
	"Europe/Berlin",		"Central Europe",
	"Europe/Madrid",		"Central Europe",
	"Europe/Rome",		"Central Europe",
	"Europe/Zagreb",		"Central Europe",
	"Europe/Gibraltar",	"Central Europe",
	"Europe/Ljubljana",	"Central Europe",
	"Africa/Tunis",		"Central Europe",
	"MET",			"Central Europe",
	"Europe/Lisbon",		"Western Europe",
	"Atlantic/Madeira",	"Western Europe",
	"Atlantic/Canary",	"Western Europe",
	"Atlantic/Faeroe",	"Western Europe",
	"Atlantic/Cape_Verde",	"GMT-1", 
	"Atlantic/St_Helena",	"GMT+0", 
	"Africa/Casablanca",	"GMT+0", 
	"Africa/Nouakchott",	"GMT+0", 
	"Africa/Sao_Tome",	"GMT+0", 
	"Africa/Bissau",		"GMT+0", 
	"Africa/Banjul",		"GMT+0", 
	"Africa/Conakry",		"GMT+0", 
	"Africa/Accra",		"GMT+0", 
	"Africa/Abidjan",		"GMT+0", 
	"Africa/Timbuktu",	"GMT+0", 
	"Africa/Dakar",		"GMT+0", 
	"Africa/Freetown",	"GMT+0", 
	"Africa/Lome",		"GMT+0", 
	"Africa/Monrovia",	"GMT+0", 
	"Africa/Cairo",		"Egypt",
	"Africa/Windhoek",	"Namibia",
	"Africa/Brazzaville",	"GMT+1",
	"Africa/Kinshasa",	"GMT+1",
	"Africa/Libreville",	"GMT+1",
	"Africa/Kinshasa",	"GMT+1",
	"Africa/Luanda",		"GMT+1",
	"Africa/Douala",		"GMT+1",
	"Africa/Malabo",		"GMT+1",
	"Africa/Lagos",		"GMT+1",
	"Africa/Johannesburg",	"GMT+2",
	"Africa/Maputo",		"GMT+2",
	"Africa/Dar_es_Salaam",	"GMT+3",
	"Africa/Nairobi",		"GMT+3",
	"Africa/Asmera",		"GMT+3",
	"Africa/Mogadishu",	"GMT+3",
	"Africa/Djibouti",	"GMT+3",
	"Africa/Aden",		"GMT+3",
	"Asia/Kuwait",		"GMT+3",
	"Asia/Bahrain",		"GMT+3",
	"Asia/Riyadh",		"GMT+3",
	"Asia/Qatar",		"GMT+3",
	"Asia/Aden",		"GMT+3",
	"Asia/Tehran", 		"Asia/Tehran (appr.)",
	"Indian/Mayotte",		"GMT+3",
	"Indian/Antananarivo",	"GMT+3",
	"Indian/Reunion",		"GMT+4",
	"Asia/Dubai",		"GMT+4",
	"Asia/Muscat",		"GMT+4",
	"Australia/Queensland",	"Australia/Eastern",	/* Queensland */
	"Australia/Brisbane",	"Australia/Eastern",	/* Queensland */
	"Australia/Hobart",	"Australia/S.Eastern",	/* Tasmania */
	"Australia/Sydney",	"Australia/S.Eastern",	/* New South Wales */
	"Australia/Melbourne",	"Australia/S.Eastern",	/* Victoria */
	"Australia/Darwin",	"Australia/Central",	/* Northern Territory */
	"Australia/Adelaide",	"Australia/S.Central",	/* South Australia */
	"Australia/Perth",	"Australia/Western",	/* Western Australia */
	"Australia/Lord_Howe",	"GMT+10:30",
	"Etc/GMT-5",		"GMT-5",
	"Etc/GMT-6",		"GMT-6",
	"Chile/EasterIsland",	"GMT-6",
	"Pacific/Easter",		"GMT-6",			/* I presume ... */
	"Pacific/Gambier",	"GMT-9",
	"Pacific/Marquesas",	"GMT-9:30",
	"Pacific/Honolulu",	"GMT-10",
	"US/Hawaii",		"GMT-10",
	"Pacific/Tahiti",		"GMT-10",
	"Pacific/Rarotonga",	"GMT-10",
	"Pacific/Johnston",	"GMT-10",
	"Etc/GMT-11",		"GMT-11",
	"Pacific/Apia",		"GMT-11",
	"Pacific/Niue",		"GMT-11",
	"Pacific/Midway",		"GMT-11",
	"Pacific/Pago_Pago",	"GMT-11",
	"Pacific/Tongatapu",	"GMT+13",
	"Pacific/Nauru",		"GMT+12",
	"Pacific/Fiji",		"GMT+12",
	"Pacific/Tarawa",		"GMT+12",
	"Pacific/Majuro",		"GMT+12",
	"Pacific/Kwajalein",	"GMT+12",
	"Pacific/Funafuti",	"GMT+12",
	"Pacific/Wallis",		"GMT+12",
	"Pacific/Wake",		"GMT+12",
	"Pacific/Norfolk", 	"GMT+11:30",
	"Pacific/Efate", 		"GMT+11",
	"Pacific/Noumea",		"GMT+11",
	"Pacific/Ponape",		"GMT+11",
	"Pacific/Guadalcanal",	"GMT+11",
	"Pacific/Guam",		"GMT+10",
	"Papua New Guinea",	"GMT+10",
	"Pacific/Port_Moresby",	"GMT+10",
	"Pacific/Guam",		"GMT+10",
	"Pacific/Truk",		"GMT+10",
	"Etc/GMT+10",		"GMT+10",
	"Pacific/Palau",		"GMT+9",
	"Japan",			"GMT+9",
	"Asia/Tokyo",		"GMT+9",
	"Asia/Seoul",		"GMT+9",
	"Asia/Pyongyang",		"GMT+9",
	"Asia/Jayapura",		"GMT+9",
	"Singapore",		"GMT+8",
	"PRC",			"GMT+8",
	"Asia/Shanghai",		"GMT+8",
	"Asia/Hong_Kong",		"GMT+8",
	"Hongkong",			"GMT+8",
	"Asia/Manila",		"GMT+8",
	"Asia/Taipei",		"GMT+8",
	"Asia/Singapore",		"GMT+8",
	"Asia/Kuala_Lumpur",	"GMT+8",
	"Asia/Ujung_Pandang",	"GMT+8",
	"Antarctica/Casey",	"GMT+8",
	"Asia/Jakarta",		"GMT+7",
	"Asia/Saigon",		"GMT+7",
	"Asia/Bangkok",		"GMT+7",
	"Asia/Phnom_Penh",	"GMT+7",
	"Indian/Christmas",	"GMT+7",
	"Antarctica/Davis",	"GMT+7",
	"Indian/Cocos",		"GMT+6:30",
	"Asia/Rangoon",		"GMT+6:30",
	"Asia/Dacca",		"GMT+6",
	"Antarctica/Mawson",	"GMT+6",
	"Asia/Colombo",		"GMT+5:30",
	"Asia/Calcutta",		"GMT+5:30",
	"Etc/GMT+5",		"GMT+5",
	"Asia/Karachi",		"GMT+5",
	"" };

