/* sysinclude.h
   Global defines for Tide Tool
    Also defines that help make the tide stuff work
     but don't seem to be anywhere in the GCC package

Tide Tool - A tide and current prediction program for the USR PalmPilot
Copyright (c) 1997, 2003 Walt Bilofsky

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#define PILOT

#define GTD_FIXED_PREC				/* Use Jeff Dairiki's code */
#define GTD_DO_ASM
#define GTD_COSINE

#define Double float				/* To use single or double in computations */
//#define Double double

#define free MemPtrFree
void *malloc(int n);

struct tm
{
  int   tm_sec;
  int   tm_min;
  int   tm_hour;
  int   tm_mday;
  int   tm_mon;
  int   tm_year;
  int   tm_wday;
  int   tm_yday;
  int   tm_isdst;
};

/*
 *	Using this union is terrible code, but the two structures happen to be identical,
 *	EXCEPT:
 *	
 *	1) Months are 1-12 on Pilot, 0-11 in UNIX.
 *	2) Pilot doesn't include the yday or isdst fields.
 *	3) Note that time_t's on Pilot are local time - either Standard or Savings -
 *	    and not corrected for Daylight Savings.  We need to make them GMT internal
 *	    to this program.
 *	
 *	So our UNIX code better not use either of these without doing the computations ...
 */

typedef union {
  struct tm ttm;
  DateTimeType dtm;
       } DateTimeUnion;

typedef DWord time_t;
#include <unix_stdio.h>
#include <unix_string.h>

extern struct tm * gmtime( time_t* );
extern struct tm * localtime( time_t* );
extern time_t mktime( struct tm* );
extern void assertfail( char *, int );
extern char abortflag;

#undef assert
#define assert(p) ErrFatalDisplayIf(!(p), "Assertion '" #p "' failed")

/* Floating point functions */
extern char * sfpr(float f, int width, int prec, int minus);
extern void fpr(float f, int width, int prec, int minus);
extern double fabs(double);

extern int printf (const char * fmt, ...);

extern char tolower(char);
extern char toupper(char);

