/* 
THINGS TO TEST:
	Beaming application, from RAM and/or card - and make sure it's on menu
		wherever available.
	Test Lord Howe time and make sure it didn't mess anything up.

TRANSLATION CATCHUPS:
	In What's New (readme and index page) - add 5-way controller moves arrow in graph.
	Beginning of About Tide Tool

TO DO:
	BUG: MEMORY ERROR CHANGING UNITS WITH ALERT SCREEN UP
	ELIMINATE RED NEGATIVE TIDE COLOR IF GRAYSCALE (CONFUSES CUSTOMERS)
	Graffiti in lat/lon entry?
	Hot areas for date and place
	Try again to number the x-axis

Problems fixed:
	Can load by double-clicking
	Note put back in UK database

KNOWN BUGS:
	If system date is outside 1998-2031, initial run crashes.
	Displaying 12/31/2031 hangs
	Anomaly: St. Pete Dec. 10 has low higher than the high.
	If a database is in RAM, ignore a copy in flash ROM.

FIXMES:
	How handle more than 4 tide events? (Southampton, Eng., 9/7-8/00)

FUTURE VERSIONS AND SUGGESTIONS:
	Let users define graph colors
	Notes to tide locations
	Accommodate time zones from OS 5?
	Accommodate larger screens
	Extend Place screen for large screen PDAs

USER REQUESTS:
	Neap and spring tides over a period
	Display "high/low water" when near same.
*/

/* tide.c:  Pilot version of xtide tide table program
 *	Main program, event loop, menu and database routines.

Tide Tool - A tide and current prediction program for Palm handheld computers
Copyright (c) 1997-2003 Walt Bilofsky

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <PalmOS.h>
#include <PalmCompatibility.h>

#include "sysinclude.h"
#include "tide.h"
#include "tidelib.h"
#include "PalmNavigator.h"

#define __MC68K__
#include "GPSLib68K.h"

int AbortFlag = 0;		/* Indicates a fatal error has happened */
					/* Positive => it's an error object number */
					/* -1 => an assertion failed */
DateTimeUnion dt;
static void EventLoop(void);
static void StopApplication(void);
static int CheckAbortFlag(void);
static Boolean HandleLoadEvent(EventType *);
static Boolean HandleEveryEvent (EventType *);

static void InitFindForm();
static void SelectFindDigit(int buttonID, Boolean setup);
static void SelectFindCheck(int buttonID);
static void SetFindPreferences(void);
static Boolean FindIfCurrent;
static WinHandle MainFormWindow = NULL;

static void InitUnitsForm();
static void SetUnitPrefs();
static void InitPrefForm();
static void SetPreferences();
static void InitRegsForm(void);
static void CloseRegForm(void);
static Boolean TblSelectHandle(int, int);
static void RegsScroll(int);
static void DebriefTable(void);

#if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL
static DWord startFreeDynamicMemory, endFreeDynamicMemory;
DWord junk;
#endif

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    if (cmd != sysAppLaunchCmdNormalLaunch) return sysErrParamErr;
#if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL
//    MemHeapFreeBytes(0, &startFreeDynamicMemory, &junk);
#endif
    FtrGet(sysFtrCreator, sysFtrNumROMVersion, &RomVersion);
    OpenDatabase();
    InitializeTime();				/* Must follow database init */
    InitializeDisplay();

    if (CheckAbortFlag() >= 0) {		/* If database error, display.  If fatal, quit here. */
    	FrmGotoForm(MainForm);
      EventLoop();				/* Event loop */
	}

    StopApplication ();				/* Application stop code */
    return 0;
}

static void EventLoop(void)
{
  short err;
  EventType e;
  long l;
  Boolean handled;
  LocalID formid = 0;
  LocalTime_t t, today;
  int datediff;

   while(1) {

/* 
 *	Check for an event.  
 *	If not computing, but in main window, wait for next minute to update display.
 */
	l = (TimeToDisplay - TimGetSeconds()) * SysTicksPerSecond();
	if (l < 0) l = 0;
	l = (ComputeState != CSIdle) ? 0 : (LocListState != LocIdle) ? 0 :
	    MainFormWindow && WinGetDrawWindow() == MainFormWindow ? l : evtWaitForever;
//#define DISPLAY_EVENTS		/* Debugging display - events and wait time */
#ifdef DISPLAY_EVENTS
{char s[30],*p; StrIToA(s,l); p = s+strlen(s);
strcat(p," E("); StrIToA(p+3,e.eType); strcat(p,")"); if (WinGetDrawWindow()) WinDrawChars (s,strlen(s),0,0);}
#endif
	EvtGetEvent(&e,(int)l);
#ifdef DISPLAY_EVENTS
{static char c; static char ind[] = "-/|\\"; if (WinGetDrawWindow()) WinDrawChar (ind[(++c)&3],150,0);}
#endif
	if (e.eType == appStopEvent) return;
	if (SysHandleEvent(&e) || 			/* If system or menus handled event */
		MenuHandleEvent((void *)0, &e, &err) ||
		HandleLoadEvent(&e)) 			/*  or it was a form open, */
			{
			continue;			/*  immediately look for another event. */
			}
	handled = FrmDispatchEvent(&e);
#ifdef DISPLAY_EVENTS
if (!handled && WinGetDrawWindow() && e.eType) {char s[50]; 
strcpy(s," E("); StrIToA(s+3,e.eType); strcat(s,")"); WinDrawChars (s,strlen(s),0,0); 
SysTaskDelay(25);}
#endif
	formid = FrmGetActiveFormID();

	switch (CheckAbortFlag()) {
      		case -1: return;			/* If fatal error, exit */
	  	case 1: BackToMidnight();		/* If error, force regeneration of tides */
		}
	if (handled) 
		continue;				/* If event handled, go get next event. */
								/* Continue computation if in progress */
#ifndef TRYING_SOMETHING_NEW
	if (MainFormWindow && WinGetDrawWindow() == MainFormWindow) {
#else
	if (FrmGetActiveFormID() == MainForm) {
#endif
#if 0
		recentbugcheck();				/**** TEMP - check for error in recent places list ***/
#endif
		t = TimGetSeconds();
		today = (t / daysInSeconds) * daysInSeconds;
		if (DisplayingToday && 			/* If today changed */
				(datediff = (today - DateBeingDisplayed) / daysInSeconds)) {  
			MoveDays(datediff);	 	/* reset the date */
			}
		if (PlaceIndex < 0) GetPlaceData(Preferences.Plastplace);
		if (ComputeState != CSIdle) list_tides();
		if (t >= TimeToDisplay) 		/* Next minute - update display */
			DisplayCurrentCondx(0);
	  	}
	  else if (LocListState != LocIdle) {		/* If in the middle of loclist initialization */
		if (InitializingLocList()) 		/*   do another piece of it. */
			FrmReturnToForm (MainForm); 	/* If failed, avoid crash by closing form quick. */
		}	
	}
   }

#define NON_PORTABLE			/* So we can load hardware flag defs for IrDA check */
#include <HwrMiscFlags.h>

static Boolean HandleLoadEvent(EventType * event)
{
   FormPtr frm;
   UInt32 irda;

   if (event->eType == frmLoadEvent)
   	{

       FrmSetActiveForm(frm = FrmInitForm(event->data.frmLoad.formID));
/*
 * Set the event handler for the form.  The handler of the currently
 * active form is called by FrmHandleEvent each time it receives an event.
 *
 *  FIXME:  The code would be more rational if there were different handlers
 *	for each form.
 */
      FrmSetEventHandler(frm, HandleEveryEvent);
/*
 * Set the menu to reflect whether beaming is available.  Don't use MenuHideItem
 *  in order to remain compatible with versions earlier than 3.5 (sigh).
 */
	switch (event->data.frmLoad.formID) {
	   case MainForm: case FindForm: case LocForm:	/* These forms have menus */
		if (sys_30) {				/* If OS < 3, no Exchg mgr.  Delete menu item. */
		   if (!FtrGet(sysFtrCreator, sysFtrNumHwrMiscFlags, &irda))
			break;				/* If can't check IrDA, keep menu item */
		   if ((irda & (hwrMiscFlagHasMbdIrDA | hwrMiscFlagHasCardIrDA)))
			break;				/* If can and IRDA present, keep menu item */
		   }
		FrmSetMenu(frm,MENUID_NOBEAM);	/* Otherwise, delete menu item */
		} 
      return (true);
   	}
   return (false);
}

#define HANDLED 	return true
#define NOT_HANDLED 	return false

static Boolean HandleEveryEvent (EventType * e)
{

	FormPtr form, formp;
	LocalID formid = 0;
	Word cid;
  	int i;
      if ((form = FrmGetActiveForm()) != 0) formid = FrmGetActiveFormID();

	switch (e->eType) {

      case frmOpenEvent:
	     	FrmDrawForm(form);

		switch (formid) {
			case MainForm:
				if (WeekStartDay)				/* If week starts with Monday */
				   for (i = 0; i < 7; i++)		/*  relabel the buttons */
					CtlSetLabel(GetObjectPtr(WeekMon + i), & T_WEEKDAYS[2*i]);
				FrmSetControlGroupSelection (FrmGetFormPtr(MainForm), WEEKGROUP, 
									WeekMon + DayofWeek );
				CtlSetLabel(GetObjectPtr(OTHERTYPE), disgraph ? T_TABLE : T_GRAPH);
				MainFormWindow  = FrmGetWindowHandle(FrmGetFormPtr(MainForm));
				HANDLED;
			case LocForm:
				InitLocForm();
				HANDLED;					/* Go check for error */
			case FindForm:
				InitFindForm();
				HANDLED;
			case RegsForm:
				InitRegsForm();
			}
			NOT_HANDLED;

	case winExitEvent:						/* Keep flag if in LocForm */
		if (e->data.winExit.exitWindow == (WinHandle) FrmGetFormPtr(LocForm)) {
			InLocForm = false;				/* Need this to tell if menu is up */
			HANDLED; }
 	case winEnterEvent:
		if (e->data.winEnter.enterWindow == (WinHandle) FrmGetFormPtr(LocForm) && 
				e->data.winEnter.enterWindow == (WinHandle) FrmGetFirstForm ()) {
			InLocForm = true;
			HANDLED; }

	case frmUpdateEvent:
		if (e->data.frmUpdate.formID == MainForm)		/*   if in main form, set to redraw tides */
			ComputeState = CSStart;
		NOT_HANDLED;						/* ... then let form handler redraw form */

	case ctlSelectEvent:  			/* A control button was pressed and released. */
		if (nPlaceDBs == 0) { 
			barf(NOLOCNBASE);	HANDLED; }
		cid = e->data.ctlSelect.controlID;
		if (cid > WeekBack && cid < WeekFwd) {		/* Click on week bar */
			i = (cid - WeekMon) - DayofWeek;
			if (i == 0) {					/* If clicked on a selected day */
				if (cid == WeekMon) i = -1;		/*  at end of bar, */
				if (cid == WeekMon+6) i = 1;		/*  move to next week */
				}
		   changedate:
			if (i) { MoveDays(i);
				   DisplayDate();	}			/* Get date changed for repeating arrows */
		   recompute:
 			FrmSetControlGroupSelection (FrmGetFormPtr(MainForm), WEEKGROUP, 
									WeekMon + DayofWeek );
			/* FIXME:  Do we need to explicitly force recompute here???? */
			NOT_HANDLED;					/* used to call FrmHandleEvent for this too */
			}
		else 
		  if (cid >= LatDTens && cid <= LatDn) {
			SelectFindDigit(cid,0); NOT_HANDLED; }	/* Click on lat/lon digit in find screen */
		else switch (cid) {
	/* Buttons in main form */
		   case NEWDATE:
			if (SelectADate()) goto recompute; 
			HANDLED;
		   case NEWLOC:
			FrmPopupForm(LocForm); 
			HANDLED;
		   case OTHERTYPE:
			disgraph = !disgraph;
			if (disgraph && Preferences.Pfirstgraph) {
				FrmCustomAlert(		/* First time, tell them how */
					UseColor == grayColor ? GRAPHTAPMEG : GRAPHTAPME, NULL, NULL, NULL);
				Preferences.Pfirstgraph = false;	/* to get a prediction */
				}
			CtlSetLabel(GetObjectPtr(OTHERTYPE), disgraph ? T_TABLE : T_GRAPH);
			ComputeState = CSStart;				/* Reset computation to redisplay */
			HANDLED;
		   case PREVLOC:						/* Swap to the last place seen */
			if (Preferences.PnRecentPlaces < 2) {
				/*** Error msg? ***/
				HANDLED;
				}
			GetPlaceData(Preferences.RecentPlaces[1]);
			ComputeState = CSStart;				/* Reset computation to redisplay */
			HANDLED;
		   case MainHelpPB:
			FrmHelp (MainHelp); HANDLED;
	/* Buttons in regional databases form */
		   case OKREG:
			CloseRegForm();
			HANDLED;
		   case CANREG:
			FrmReturnToForm(0);
			HANDLED;
	/* Buttons in Find Location form */
		   case OKFIND:
			SetFindPreferences();
			FrmReturnToForm (LocForm); 
			Preferences.Pselectprev = false;		/* Make sure not in the prev list */
			choosecurrents = -1;
			InitLocList(FindIfCurrent);			/* and looking at the right type */
			HANDLED;						/* Go check for error */
		   case CANFIND:
			FrmReturnToForm (LocForm); 
			HANDLED;
		   case FCPROX:
		   case FCSRCH:
		   case FCSET:
		   case FINDGPS:
			SelectFindCheck(cid);
			HANDLED;
	/* Buttons in Location Choice form */
		   case LISTIDES: 
		   case LISTCURR:
		   case LISTREC:
			InitLocList(cid - LISTIDES);
			HANDLED;						/* Go check for error */
		   case FINDLOCS:
			FrmPopupForm(FindForm); HANDLED;
		   case OKLOCS:						/* Exit the location select form */
			Preferences.Pselectprev = (choosecurrents == 2);
			if (LocListState != LocIdle)			/* If the list wasn't up yet, */
				cid = CANLOCS;				/*  switch to Cancel function. */
		   case CANLOCS:						/* Cancel the location select form */
			i = CloseLocList();
			if (cid == OKLOCS) { 
				GetPlaceData(i);				/* Switch to a new location */
				ComputeState = CSStart; 		/* Void old tides & recompute */
				}
			FrmReturnToForm (MainForm); 
			HANDLED;
			default:  NOT_HANDLED;
			}			
		HANDLED;

     case ctlRepeatEvent:		// A repeat button is being held down
		if (nPlaceDBs == 0) { barf(NOLOCNBASE);	HANDLED; }
 		cid = e->data.ctlRepeat.controlID;
		switch (cid) {
			case WeekBack: i = -7; goto changedate;
			case WeekFwd: i = 7; goto changedate;
			case LatUp: case LatDn:
					SelectFindDigit(cid,0); 
					NOT_HANDLED; 
			}
		NOT_HANDLED;

	case ctlEnterEvent:  	// A repeat button was pressed
		if (nPlaceDBs == 0) { barf(NOLOCNBASE);	HANDLED; }
		cid = e->data.ctlEnter.controlID;
		switch (cid) {
		   case MainHelpPB:
			FrmHelp (MainHelp); HANDLED;
			}
		NOT_HANDLED;

	case lstSelectEvent:
		if (e->data.lstSelect.listID == LocList) {
			ShowLocList(); HANDLED; }
		NOT_HANDLED;

	case sclExitEvent:
		if (formid == RegsForm) {
			RegsScroll(e->data.sclExit.newValue);
			HANDLED; }
		NOT_HANDLED;

	case sclRepeatEvent:
		if (e->data.sclRepeat.scrollBarID == LocScrollbar) {
			LocScroll(&e->data.sclRepeat);
			}
		NOT_HANDLED;

	case tblSelectEvent:		/* Only one table - in the regions form */
		switch (e->data.tblSelect.tableID) {
			case LocList:
				LocSelect(e->data.tblSelect.row);
				NOT_HANDLED;
			case REGTABLE:
				if (TblSelectHandle(e->data.tblSelect.row,e->data.tblSelect.column)) 
					HANDLED;
			}
			NOT_HANDLED;

	case fldEnterEvent:
		if (e->data.fldEnter.fieldID == FCField) {
				SelectFindCheck(FCSRCH);	
				HANDLED; }
		NOT_HANDLED;

	case keyDownEvent:
		if (nPlaceDBs == 0) { barf(NOLOCNBASE);	HANDLED; }
		if (e->data.keyDown.chr) {
			switch (FrmGetActiveFormID()) {
				case FindForm:
					SelectFindCheck(FCSRCH); 	/* Select entry field */
					NOT_HANDLED;			/* Let form code handle graffiti */
				/* Handle scroll buttons */
				case LocForm: if (ScrollLocList(e->data.keyDown.chr)) HANDLED;
					NOT_HANDLED;
				case MainForm:				/* Scroll date */
					switch (e->data.keyDown.chr) {
	   				  case 11: i = -1; goto changedate;
	   				  case 12: i = 1; goto changedate;
					  case vchrNavChange:		/* Maybe fiveway hit in graph? */
						if(e->data.keyDown.keyCode & navBitRight) MoveArrow(1);
						   else if(e->data.keyDown.keyCode & navBitLeft) MoveArrow(-1);
						   else NOT_HANDLED;
						HANDLED;
					  }
				}
			}
		NOT_HANDLED;

	case penDownEvent:				/* Did user pen the sun or moon, or the date line? */
		if (formid == MainForm && (i = HitSunMoon(e->screenX,e->screenY))) {
				WantedTheSky(i);
				HANDLED; }
		NOT_HANDLED;

	case menuEvent:				/* Handle menu event */
		if (nPlaceDBs == 0) { barf(NOLOCNBASE);	HANDLED; }
		MenuEraseStatus(0);
		switch (e->data.menu.itemID) {
		   case M_DBASE:	
			FrmPopupForm(RegsForm);
			HANDLED;
		   case M_ABOUT:	FrmHelp(AboutTT); HANDLED;
		   case M_MEM:	DisplayDBInfo(-1); HANDLED;
		   case M_LOCATION: DisplayDBInfo(-2); HANDLED;
		   case M_LICENSE:FrmHelp (TTWarning); HANDLED;
		   case M_TZONE:	DisplayTZInfo(); HANDLED;
		   case M_UNITS: 
			formp = FrmInitForm(UnitsForm);
			InitUnitsForm();
			if (FrmDoDialog(formp) == OKUNITS) {
				SetUnitPrefs();
				ComputeState = CSStart; /* Void old tides & recompute */
				}
			FrmDeleteForm( formp );
			HANDLED;
		   case M_PREFS: 
			formp = FrmInitForm(PrefForm);
			InitPrefForm();
			if (FrmDoDialog(formp) == OKPREFS) {
				SetPreferences();
				if (FrmGetActiveFormID() == LocForm) {
					i = choosecurrents;
					choosecurrents = -1;
					InitLocList(i);
					}
				ComputeState = CSStart; 	/* Void old tides & recompute */
				}
			FrmDeleteForm( formp );
			HANDLED;
		   case M_MOON:	WantedTheSky(1); HANDLED;
		   case M_SUN:	WantedTheSky(2); HANDLED;
		   case M_BEAM:	BeamTideTool(); HANDLED;
			}
      	NOT_HANDLED;

	default: 							/* Default: Let form handler do it */
		return false;					/* FrmDispatchEvent will pass event to FrmHandleEvent */
		}
   return false;							/* Shouldn't get here */
   }

static int CheckAbortFlag(void) {				/* Check for an error condition */
	int i;
	char *p;
	union { 
	  char stg[5];
          struct { UInt32 creator; char zero; } s;
          } creatorstg;

	if (!AbortFlag) return 0;				/* If no error, handle next event */

	switch (AbortFlag) {
		char buf[150];
		default: 
			FrmAlert (AbortFlag); break;
		case MISSINGYEAR:
			sprintf(buf,T_TIDES_CAN_ONLY_BE_COMPUTED,
				first_year, first_year + num_epochs - 1);
			FrmCustomAlert (ERRSTRING, buf, NULL, NULL);
			break;
		case BADDBASE: 
		case OLDDBASE:
			p = buf + sprintf(buf,T_THE_DATABASE_FILE,dbfilename);
			if (dbcreator != TideAppID) {
				creatorstg.s.creator = dbcreator;
				creatorstg.s.zero = 0;
				p += sprintf(p,T_ERR_CODE,creatorstg.stg);
				}
			sprintf(p,T_IS_BLOOEY,
				AbortFlag == BADDBASE ? T_CORRUPT : T_FROM_OLD_VERSION);
			FrmCustomAlert (ERRSTRING, buf, NULL, NULL);
			break;				
#ifdef MYOWNASSERTS
		case ASSERTFAIL:
			sprintf(buf,T_FATAL_ERR,AssertFile,AssertLine);
			FrmCustomAlert (ERRSTRING, buf, NULL, NULL);
			break;
#endif
		}
	i = (AbortFlag >= FATALERRORS ? -1 : 1);
	AbortFlag = 0;
	return i;
	}


/*
 *	Find Place form handling
 */

static FormPtr Findfrm;
static char ll_max[14] = { 8, 9, 5, 9, 5, 9, 1, 17, 9, 5, 9, 5, 9, 1 }; 
static struct place *FindPlace;				/* Current selection in place list */

static struct FindStatStruct FindStatus = {-1, 0};

static UInt16 gGpsLibRef;               /* GPS Library reference                */
static char GpsLibLoaded = 0;		/* 1 if GPS OK, -1 if not but lib was loaded */

static void InitFindForm() {
	int i,f;
	char buf[2*LATLONLEN+4];

	Findfrm = FrmGetFormPtr(FindForm);

	if (!GpsLibLoaded 
		&& (SysLibFind(gpsLibName, &gGpsLibRef) == errNone 	/* Try to load Garmin library */
		    || SysLibLoad(gpsLibType, gpsLibCreator, &gGpsLibRef) == errNone))
			{						/* If there, make note of it. */
			GpsLibLoaded = (GPSOpen(gGpsLibRef) == errNone) ? 1 : -1;
			}
	if (GpsLibLoaded > 0) FrmShowObject (Findfrm, 
				FrmGetObjectIndex(Findfrm, FINDGPS));	/* If not there, hide Garmin button */
	MemMove(&FindStatus, & Preferences.Pfind, sizeof FindStatus);
//printf("Pref %d %d %ld\n",Preferences.Pfind.checksel,Preferences.Pfind.latlong[5],Preferences.Pfind.latitude);
//printf("Fsta %d %d %ld\n",FindStatus.checksel,FindStatus.latlong[5],FindStatus.latitude);
	if (FindStatus.checksel < 0) FindStatus.checksel = 0;
 	FrmSetControlGroupSelection (Findfrm, FXGROUP, FCPROX + FindStatus.checksel);
	f = FindStatus.focus;
	for (i = LatDTens; i <= LonEW; ++i) SelectFindDigit(i, 1);
 	FrmSetControlGroupSelection (Findfrm, LLGROUP, LatDTens + (FindStatus.focus=f));

	FindPlace = GetLocSelection();		/* Current selection */
	FindIfCurrent = (P_ISCURRENT & FindPlace->flags) != 0;
	FntSetFont(0);
	WinDrawChars(FindPlace->location,strlen(FindPlace->location),
				FCTXT_X,FCSET_Y+FntCharHeight());
	LatLonToA(buf, UnpackLat(FindPlace), T_NORTH_SOUTH);
	LatLonToA(lonstg, UnpackLon(FindPlace), T_EAST_WEST);
	strcat(buf,NumspaceStr);
	strcat(buf,lonstg);
	WinDrawChars(buf,strlen(buf),FCTXT_X,FCSET_Y+2*FntCharHeight());
	}

/* Auxiliary routines for Find form */

static long makelat(char *p) {			/* Make latitude (or longitude) from chars */
	long ans;
	ans = 10 * p[0] + p[1];
	ans *= 60L;
	ans += 10 * p[2] + p[3];
	ans *= 60L;
	ans += 10 * p[4] + p[5];
	if (p[6]) ans = -ans;
	return ans;
	}

#if 0
static long trim_ll(long ll) {			/* Truncate lat/lon to prevent overflow in compare */
	ll = ll >= 0 ? ll+30L : ll-30L;
	return ll >> LLSHIFT;
	}
#define TRIM360 ((360L * 60L * 60L) >> LSHIFT)
#define TRIM180 ((180L * 60L * 60L) >> LSHIFT)
#endif

static void SelectFindCheck(int buttonID) {	/* Handle check boxes in Find form */
	int i;
 	FrmSetControlGroupSelection (Findfrm, FXGROUP, buttonID);
	switch (buttonID) {
		case FCPROX: 
			FindStatus.checksel = F_LATLON;
			if (FindStatus.focus >= LatDn-LatDTens) {
				FindStatus.focus = 0;
 				FrmSetControlGroupSelection (Findfrm, LLGROUP, LatDTens );
				}
			FrmSetFocus (Findfrm, noFocus);
			return;
		case FCSRCH:
			FrmSetFocus (Findfrm, FrmGetObjectIndex (Findfrm, FCField));
			InsPtEnable (true);
			FindStatus.checksel = F_STG;
			return;
		case FINDGPS:
			{GPSPositionDataType g;
			if (GPSGetPosition(gGpsLibRef, &g)) {	/* Get GPS position if avail */
				FrmAlert(NOGPSPOS); 			/* Warn if not */
				FrmSetControlGroupSelection (Findfrm, FXGROUP, FCPROX);
				break; }
			setllprefs(&FindStatus, g.lat/ 3314, g.lon / 3314);
			}				/* and fall through */
		case FCSET:				/* Note sign of longitude is flipped in FindStatus */
			if (buttonID == FCSET)
				setllprefs(&FindStatus, UnpackLat(FindPlace), UnpackLon(FindPlace));
		 	FindStatus.checksel = F_LATLON;
			for (i = LatDTens; i <= LonEW; ++i) SelectFindDigit(i, 1);
			FrmSetFocus (Findfrm, noFocus);
	}	}

char llstgs[2][2][2] = {{ T_NORTH, T_SOUTH}, { T_EAST, T_WEST}};
char labelnumbers[19][3] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", 
			"9", "10", "11", "12", "13", "14", "15", "16", "17", "18" };

static void SelectFindDigit(int buttonID, Boolean setup) {	/* Process a button in Find form */
	int i = -1, j, f = 0;
	if (!setup) {						/* Any of these puts status back here */
 		FrmSetControlGroupSelection (Findfrm, FXGROUP, FCPROX );
		FindStatus.checksel = 0;
		SelectFindCheck(FCPROX);			/* Make sure focus is in digits */
		}
	switch (buttonID) {
		case LonEW: f = (LonEW-LatDTens); i = 1; goto NSEW;	/* Hit the N/S or EW buttons -  */
		case LatNS:
			f = (LatNS-LatDTens); i = 0;
		  NSEW:						/* Set N/S or E/W buttons */
			if (!setup)
				FindStatus.latlong[f] = 1 - FindStatus.latlong[f];
			CtlSetLabel (GetObjectPtr(buttonID), llstgs[i][(Byte)(FindStatus.latlong[f])]);
 			FrmSetControlGroupSelection (Findfrm, LLGROUP, LatDTens + FindStatus.focus);
			return;
		case LatUp: i = 1;				/* Arrow - inc or dec by 1 */
		case LatDn:
			f = FindStatus.focus;
			j = FindStatus.latlong[f] + i;
			if (j > ll_max[f]) j = 0;
			if (j < 0) j = ll_max[f];
			FindStatus.latlong[f] = j;
			CtlSetLabel (GetObjectPtr(FindStatus.focus + LatDTens), labelnumbers[j]);
 			FrmSetControlGroupSelection (Findfrm, LLGROUP, LatDTens + FindStatus.focus);
			return;
		default:						/* Select a digit field */
			FindStatus.focus = f = buttonID - LatDTens;
			if (setup) 
				CtlSetLabel (GetObjectPtr(FindStatus.focus + LatDTens),
									labelnumbers[(int)FindStatus.latlong[f]]);
 			FrmSetControlGroupSelection (Findfrm, LLGROUP, buttonID);
			return;
	}	}

static void SetFindPreferences(void) {		/* Hit OK in Find form  - store prefs now */
	int i;
	char *p,*q;
	if (FindStatus.checksel == FCSET-FCPROX) FindStatus.checksel = 0;
	p = FldGetTextPtr (GetObjectPtr(FCField));
	strcpy(FindStatus.findstring,p ? p : "");
	FindStatus.findstrlen = strlen(FindStatus.findstring);
	switch (Preferences.Pfind.checksel = FindStatus.checksel) {
	   case F_LATLON:
		for (i = 0; i < 14; ++i)
			Preferences.Pfind.latlong[i] = FindStatus.latlong[i];
		Preferences.Pfind.latitude = makelat(&FindStatus.latlong[0]);
		Preferences.Pfind.slat = Preferences.Pfind.latitude / 10;
		Preferences.Pfind.longitude = makelat(&FindStatus.latlong[7]);
		Preferences.Pfind.slon = Preferences.Pfind.longitude / 20;
		return;
	   case F_STG:
		p = Preferences.Pfind.findstring;
		q = FindStatus.findstring;
		while ((*p++ = tolower(*q++)) != 0)
                    ; 
		Preferences.Pfind.findstrlen = FindStatus.findstrlen;
	}  }

/*
 *	Preferences form handling
 */

static FormPtr Preffrm;
static ListType *Plistp;
static FieldPtr Pfieldp;
static VoidPtr GetPrefPtr(LocalID id) {
	return FrmGetObjectPtr(Preffrm,FrmGetObjectIndex(Preffrm,id));
	}
static enum Units lev_to_units[] = { feet, meters, inches };	/* Convert level units index to internal units index */
static char units_to_lev[] = { 0, 1, 0, 0, 2 };				/*  and back */

static void InitPrefForm() {
	ControlType *colorcheckbox;
	Handle fieldtext;
	char *p;

	Preffrm = FrmGetFormPtr(PrefForm);
	colorcheckbox = GetPrefPtr(PPCOLOR);
	if (scr_color)				/* If color display, change checkbox lbl from grayscale */
		CtlSetLabel (colorcheckbox,T_DISPLAY_COLORS);
	if (!sys_30) 				/* If system doesn't support color/gray, hide checkbox */
		CtlSetUsable (colorcheckbox, false);
 	FrmSetControlGroupSelection (Preffrm, PPGROUP, PPIGNORE - Preferences.Puseoffsets);
	FrmSetControlValue ( Preffrm, FrmGetObjectIndex (Preffrm, DPREFDST), !Preferences.PignoreDST);
	if (sys_30) 
		FrmSetControlValue ( Preffrm, FrmGetObjectIndex (Preffrm, PPCOLOR), Preferences.Pcolor);
	FrmSetControlValue ( Preffrm, FrmGetObjectIndex (Preffrm, DPRALON), Preferences.Puselevel);
	Plistp = GetPrefPtr(DPRALUNT);
	LstSetSelection(Plistp,Preferences.Plevelunits);
	CtlSetLabel(GetPrefPtr(DPRALTRG),
		LstGetSelectionText(Plistp,units_to_lev[Preferences.Plevelunits]));
	Pfieldp = GetPrefPtr(DPRALVAL );		/* Set field text value */
	p = sfpr(Preferences.Plevel,0,known_units[Preferences.Plevelunits].places,1);
 	fieldtext = MemHandleNew(strlen(p) + 1);
	strcpy( (CharPtr)MemHandleLock( fieldtext ), p); 
	MemHandleUnlock( fieldtext ); 
	FldSetTextHandle(Pfieldp, fieldtext); 
	}

static float SToF(char *p) {	/* Return float value */
	float ans = 0.0;
	float mult = 1.0;
	Boolean fract = false;
	float sign = 1.0;
	int i;

	switch (*p) {
		case '-': sign = -1.0;
		case '+': ++p;
		}
	do {
		if ((i = (*p - '0')) >= 0  && i <= 9) {
			if (fract) ans += i * (mult /= 10.0);
			    else ans = 10.0 * ans + i;
			}
		  else if (!fract & (*p == '.' || *p == ',')) fract = true;
		  else { barf(BADFLOAT); return 0.0; }
	   } while (*++p);
	if (ans > 999) { barf(BADFLOAT); return 0.0; }
	return ans * sign;
	}

static void SetPreferences() {
	Preferences.Puseoffsets = PPIGNORE - 
		FrmGetObjectId (Preffrm, FrmGetControlGroupSelection ( Preffrm, PPGROUP));
	Preferences.PignoreDST  = !FrmGetControlValue ( Preffrm, FrmGetObjectIndex (Preffrm, DPREFDST));
	Preferences.Pcolor  = !sys_30 ? false : 
		FrmGetControlValue ( Preffrm, FrmGetObjectIndex (Preffrm, PPCOLOR));
	Preferences.Puselevel = FrmGetControlValue ( Preffrm, FrmGetObjectIndex (Preffrm, DPRALON));
	Preferences.Plevelunits = lev_to_units[LstGetSelection(Plistp)];
	Preferences.Plevel = SToF(FldGetTextPtr(Pfieldp));
	}

/*
 *	Units form handling
 */

static FormPtr Unitsfrm;

static void InitUnitsForm() {
	Unitsfrm = FrmGetFormPtr(UnitsForm);
 	FrmSetControlGroupSelection (Unitsfrm, TPGROUP, TPREF0 + Preferences.Ptunits);
 	FrmSetControlGroupSelection (Unitsfrm, HPGROUP, HPREFFT + Preferences.Phunits);
	FrmSetControlGroupSelection (Unitsfrm, CPGROUP, CPREFKT + Preferences.Pvunits);
	FrmSetControlValue ( Unitsfrm, FrmGetObjectIndex (Unitsfrm, CPREF), Preferences.Pusecoeffs);
	}

static void SetUnitPrefs() {
	Preferences.Ptunits = FrmGetObjectId (Unitsfrm,FrmGetControlGroupSelection ( Unitsfrm, TPGROUP))
								 - TPREF0;
	Preferences.Phunits = FrmGetObjectId (Unitsfrm,
			FrmGetControlGroupSelection ( Unitsfrm, HPGROUP)) - HPREFFT;
	Preferences.Pvunits = FrmGetObjectId (Unitsfrm,
			FrmGetControlGroupSelection ( Unitsfrm, CPGROUP)) - CPREFKT;
	Preferences.Pusecoeffs = FrmGetControlValue ( Unitsfrm, FrmGetObjectIndex (Unitsfrm, CPREF));
	}

/*
 *	Regions form handling
 */

static char regsettings[NPLACEDBS];				/* Temp array for user's selections */

static void LoadTable() {
	TablePtr table;
	int row, max;

	table = GetObjectPtr(REGTABLE);
	for (row = 0; row < RTBLROWS; ++row)
	   if (RegTableTop + row >= nPlaceDBs) TblSetRowUsable (table, row, false);
	    else {
		TblSetItemStyle (table, row, 0, customTableItem);
		TblSetItemStyle (table, row, 1, customTableItem);
		TblSetItemStyle (table, row, 2, popupTriggerTableItem);
		TblSetItemPtr (table, row, 2, GetObjectPtr(REGLIST));
		TblSetRowHeight (table, row, 14);
		TblSetRowUsable (table, row, true);
		TblSetRowID (table, row, PlaceHdrs[row+RegTableTop]->db_ID);
		TblSetItemInt (table, row, 2, regsettings[row+RegTableTop]);
		TblMarkRowInvalid (table, row);		/* Mark invalid to force draw */
		}
	TblSetCustomDrawProcedure (table, 0, DrawTableItem);
	TblSetCustomDrawProcedure (table, 1, DrawTableItem);
	TblSetColumnUsable (table, 0, true); 
	TblSetColumnUsable (table, 1, true); 
	TblSetColumnUsable (table, 2, true); 
	TblHasScrollBar (table, true);
	if ((max = nPlaceDBs - RTBLROWS) < 0) max = 0;
	SclSetScrollBar (GetObjectPtr(REGSCROLL), RegTableTop, 0, max, RTBLROWS);
	/* Update scroll bar */
	TblRedrawTable(table);
	}

static void RegsScroll(int where) {
	assert (where < nPlaceDBs);
	DebriefTable();					/* Save current selections */
	RegTableTop = where;
	LoadTable();					/* Probably don't have to do all that stuff every time */
	}

static void InitRegsForm(void) {
	int i;
	RegTableTop = 0;
	for (i = 0; i < nPlaceDBs; ++i) regsettings[i] = !DBEnabled(i);		
	LoadTable();
	}

static void DebriefTable(void) {			/* Get values from table */
	TablePtr table;
	int row;

	table = GetObjectPtr(REGTABLE);
	for (row = 0; row < RTBLROWS; ++row)
	   if (RegTableTop + row < nPlaceDBs) 
		regsettings[RegTableTop + row] = TblGetItemInt (table, row, 2);
	}

static Boolean CheckAnyRegions(void) {
	int i, total;
	for (i = total = 0; i < nPlaceDBs; ++i) total += !regsettings[i];
	if (total == 0) {
		barf(CANTDISABLE);
		return true;
		}
	return false;
	}

static void CloseRegForm(void) {
	int i;
	Boolean changed = false;
	DebriefTable();					/* Save values in regsettings */
	if (CheckAnyRegions()) return;		/* Make sure some database is still live */
	FrmReturnToForm(0);				/* Restore previous form (to deal with loc list) */
	for (i = 0; i < nPlaceDBs; ++i) 
		if (regsettings[i] != 2) changed |= DBEnable(i, !regsettings[i]);
	if (changed) DeleteOrReloadDatabases(-1);
	}

static Boolean TblSelectHandle(int row, int col) {	/* Return whether handled */
	TablePtr table;
	Boolean err;
	int oldval;

	table = GetObjectPtr(REGTABLE);
	if (col == 2) {						/* Popup trigger selection made */
		if (TblGetItemInt (table, row, 2) != 2)	/* If selection isn't Delete */
			return false;				/* Let system just select it. */
		oldval = regsettings[row+RegTableTop];
		DebriefTable();
		err = CheckAnyRegions();			/* Check some database is left */
		if (!err) {						/* If OK */
			CloseRegForm();				/* Save other settings */
			DeleteDatabase(row + RegTableTop);	/* Delete the database */
			FrmPopupForm(RegsForm);			/* Put the table up anew */
			return true;
			}
		TblSetItemInt (table, row, 2, regsettings[row+RegTableTop] = oldval);/* Restore setting */
		TblMarkRowInvalid (table, row);
		}
	if (col == 0) DisplayDBInfo(row + RegTableTop);
	TblReleaseFocus(table);
	TblMarkRowInvalid (table, row);
	TblRedrawTable(table);
	return true;
	}

VoidPtr GetObjectPtr(Int objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

static void StopApplication(void)
{
  FreeLocList();
  ReleaseDisplayHandle();
  FrmSaveAllForms ();
  FrmCloseAllForms ();
  if (GpsLibLoaded && GPSClose( gGpsLibRef ) == gpsErrNone ) SysLibRemove( gGpsLibRef );
  SavePreferences();
  CloseDatabases();
  RestoreDisplay();
  }

#if 0				/* Not needed at the moment, but hang onto it ... */
static void DisplayMemInfo(void) {
	char *p, buf[256];
	long free, max;
	p = malloc(1);
	MemHeapFreeBytes ( MemPtrHeapID (p), &free, &max);
	sprintf(buf,"%ld free bytes available to program.\nLargest block %ld.\n",free,max);
	FrmCustomAlert (DBINFO2, buf, NULL, NULL);
	free(p);
	}
#endif

/* 
 *	Error Functions
 */

#ifdef MYOWNASSERTS
void assertfail( char *file, int line) {
	AssertFile = file;
	AssertLine = line;
	AbortFlag = ASSERTFAIL;
	}
#endif

void barf(int errcode) {
	if (errcode > AbortFlag)				/* Save the flag for the error */
		AbortFlag = errcode;				/* If multiple errors, make sure it's the most severe one */
	}

#if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL
void CheckForLeak(int n) {
  DWord junk; 

  StopApplication ();
  MemHeapFreeBytes(0, &endFreeDynamicMemory, &junk);
  if (startFreeDynamicMemory != endFreeDynamicMemory) {
	char foo[100];
	sprintf (foo,"Memory leak at %d - %ld bytes.", n, startFreeDynamicMemory - endFreeDynamicMemory);
  	ErrNonFatalDisplayIf(1,foo);
  	}
   }
#endif

void *malloc(int n) {
	char *p, *q;
	if ((p = q = MemPtrNew(n)) == NULL) {
		AbortFlag = OUTOFMEM;
		return NULL;
		}
	while (n--) *q++ = 0;
	return p;
	}

char tolower(char c) { return c < 'A' ? c : c > 'Z' ? c : c + 'a' - 'A'; }

char toupper(char c) { return c < 'a' ? c : c > 'z' ? c : c - ('a' - 'A'); }

static void dummyfloats(double a, double b) {		/* To force early load of double math */
	dummyfloats(a + b, a - b);
	dummyfloats(a * b, a / b);
	}

