/* tidelib.h - external declarations and data structures for Tide Tool.

Tide Tool - A tide and current prediction program for the USR PalmPilot
Copyright (c) 1997-2003 Walt Bilofsky

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#define LANG_ENGLISH 1
#define LANG_FRENCH 2

#if LANG == LANG_FRENCH
#include "tidetext_fr.h"
#else
#include "tidetext.h"
#endif

#include "VFSMgr.h"

//#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
//#define ERROR_CHECK_LEVEL ERROR_CHECK_NONE

//#define PRINT_DEBUG
//#define PRECISE					/* Force precision output of results */
//#define LABELGRAPH				/* Put tide events on the graph view */

#define SCROLLBAR

#define PREFS_IN_SYSTEM				/* If no separate preference database - uses sys prefs */

#define MAXLOCSTR 60				/* Must be same as in database definitions */
#define MAXTZSTR 12
#define LATLONLEN 14				/* Length of lat/lon ascii string */

#define MAX_CSTS 192				/* Maximum number of factors allowed */
#define CST_ARRAY_SIZE 6			/* MAX_NUM_CSTS / 32, rounded up */

typedef unsigned int DBIndex;	/* an index is (index in db) | (db index in array << ID_SHIFT) */

extern int AbortFlag;			/* If program is aborting */
extern unsigned long RomVersion;	/* Version of Palm OS ROM */

/********************************************************
 *
 *	Time and date routines and globals
 *
 ********************************************************/

typedef time_t GMTime_t;		/* Define separate types for GMT and local time */
typedef time_t LocalTime_t;
enum TimePrefs { t5_01pm = 0, t17_01, t_1701, t_17h01 };

extern int year;
extern GMTime_t faketime;		/* Current date/time for tide computation, in GMT */
extern LocalTime_t loctime;		/* Current date/time user has selected, in local time */
extern LocalTime_t TimeToDisplay;	/* Time for next update of display of current data */
extern LocalTime_t LocalDateNow;	/* Today's date (in seconds 'till last midnight) */
extern LocalTime_t DateBeingDisplayed; 	/* Date being displayed */
extern int DayofWeek;			/* Week day of current faketime */
extern char WeekStartDay;		/* User's global preference */
extern DateFormatType LongDateFormat;
extern char decimal_point;		/* Comma or period, according to system preference */

					/* Values returned by sunrise/sunset functions */
#define ALWAYS_LIGHT 1				/* Never get this time_t, we assume */
#define ALWAYS_DARK 2
#define SUN_MASK (~3)

extern void LatLonToA(char *latlon, long ll, char *which);
extern void InitializeTime(void);
extern void MoveDays(int);
extern Boolean MoveFaketime(int i);
extern Boolean SelectADate(void);
extern void BackToMidnight(void);
extern GMTime_t local2GMT(LocalTime_t loctime, Boolean midnight);
extern GMTime_t localtm2GMT(struct tm *ht, Boolean midnight);
extern struct tm * LocalTimeToDST( LocalTime_t* );
extern VoidPtr GetObjectPtr(Int objectID);
extern Boolean IsDST(GMTime_t t);
extern void DisplayTZInfo(void);
extern char *ZoneName(int IsDST);
extern int MoonPhase(GMTime_t date);
extern void WantedTheSky(int sm);

extern GMTime_t sunrise, sunset;

extern void Fiddler(LocalTime_t date);
extern GMTime_t SunStuff(LocalTime_t, Boolean);

/* To use GTD's trig functions. */

#ifdef GTD_COSINE
#define sinf fixed_sin
#define mycos fixed_cos
#else
extern double sinf(double arg);
extern double mycos(double x);
#endif

extern double myacos(double f);
extern double tanf(double f);

extern double atanf(double f);

/********************************************************
 *
 *	Tide computation algorithm routines and globals
 *
 ********************************************************/

extern void sigabort ( void );  /* Leave out parms for portability */
extern int hhmm2seconds (char *hhmm);
extern void free_data (void);
extern void load_data (void);
extern void check_epoch (void);
extern void InitializeEpoch(void);
extern time_t parse_time_string (char *time_string);
//extern int yearoftimet (time_t t);
//extern int happy_new_year (int new_year);
extern void figure_multipliers ( void );
extern float time2atide (time_t t);
extern float TideAnyplace(GMTime_t t, int *s);
extern void do_timestamp (char *buf, char *dbuf, struct tm *t);
extern char *TimeToString(char *, int, Boolean, Boolean, char, enum TimePrefs);
extern struct tm *tmtime (time_t t);
extern int update_high_tide (void);
extern void list_tides (void);
extern void tide2ascii (void);
extern void tide2ppm (char *);
extern time_t prev_hour (time_t);
extern time_t prev_day (time_t);
extern time_t increment_day (time_t);
extern void change_time_zone (char *);
extern void barf (int);
extern char *stradoop (char *);
extern char ToLower(char c);
#ifdef GTD_COSINE
extern float fixed_cos (double arg);
extern float fixed_sin (double arg);
#else
extern Double cos_degrees (double arg);
extern Boolean InitCosine(int doit);
#endif
extern float mysqrt(float f);
extern time_t midnight_start;	/* Start display at this time */

#ifdef PRINT_DEBUG
/* For debugging */
extern void printime(time_t t);
extern void prindate(time_t t);
#endif
#if ERROR_CHECK_LEVEL == ERROR_CHECK_FULL
extern void CheckForLeak(int);
#endif

/********************************************************
 *
 *	Database routines and globals
 *
 ********************************************************/

/* Data loaded from tide factors database file */
extern int num_csts;
extern int first_year;
extern int num_epochs;
extern int num_nodes;
extern float loc_amp[MAX_CSTS], loc_epoch[MAX_CSTS];
/* These are pointers into a record in the file */
extern float *cst_nodes, *cst_speeds, *cst_epochs;
/* These hold data for French coefficient calculation */
extern float coeff_amp[MAX_CSTS], coeff_epoch[MAX_CSTS];
extern long coeff_tmeridian;
extern double coeff_datum;
extern DBIndex coeff_PlaceIndex;
extern Boolean coeff_swapped;
extern Boolean coeff_computing;
extern Boolean coeff_displaying;
extern time_t coeff_time;
struct coeff_value {
	time_t time;
	unsigned char val;
	};
#define MAX_COEFFS 6
extern struct coeff_value coeff_values[MAX_COEFFS];
extern int ncoeffs;
extern Boolean need_zeros;		/* Whether 0 crossings are needed (current or alarm) */

/* Data loaded from the locations database file */
extern DBIndex PlaceIndex;		/* current place index in database file */
extern long meridian;
extern long tmeridian;			/* Meridian for tide computation; may be offset from time zone */
extern double DATUM;
extern int unit;
extern long latitude, longitude;
extern int minoff, maxoff;		/* Max and min time corrections for an offset location */
extern char dst[5],std[5];
extern float absmax, absmin;
extern UInt32 dbcreator;		/* Data for dbase error messages */
extern Char dbfilename[33];
extern UInt32 TideAppID;

/* Data structures for locations database */

#define P_UNITS 7		/* Flag byte: bits for units */
#define P_ISCURRENT 0x80	/* If it's a current (otherwise tide) */
#define P_ISOFFSET  0x40	/* If it's an offset of another location */
#define P_HIDDEN    0x20	/* A hidden reference station - never displayed */
#define P_COEFF     0x10	/* Display French tidal coefficient */

struct place {			/* Location information for a place (rec. 1) */
	int latitude;		/* In seconds / 10 */
	int longitude;		/* In seconds / 20 */
	int absmax, absmin;	/* Max and min tidal range * 100 */
	unsigned char lloflow;	/* 10 * longitude overflow + latitude overflow */
	char timezone;		/* >= 0: index in time zone array.  < 0: */
					/*	GMT offset in half hours starting with -1=+14, -2=+13:30, ... */
	char flags;
	struct {			/* How location name is split for display */
	  unsigned char font:1;	/* Font to use */
	  unsigned char loc:7;	/* Where to split the string */
	  } split;			/* NOTE: order of fields is opposite of that on PC */
	char location[1];		/* Actual size may vary; void where prohibited */
	};				/*  (Size in database varies - maximum is MAXLOCSTR+1) */

struct placedat {			/* Tidal computation data for a place (rec. 2) */
	float datum;		
	char meridian;		/* Meridian for tide computation - in half hours */
	unsigned char ndata;	/* Number of nonzero data */
	long datamask[CST_ARRAY_SIZE];	/* Bit array indicating which data are nonzero */
	float amp[1];		/* Actual struct is float amp[num_csts]; float epoch[num_csts]; */
	};				/*    with zero entries squeezed out */

struct placeoff {			/* Second record in case it's an offset */
	DBIndex primary;		/* Where it's offset from */
	int offsets[4];		/* Time offsets (in min) for h/l or fl/ebb/low sl/hi sl */
	int mulfactors[2];	/* Mult factors (* 100) for h/l or fl/ebb */
	int addfactors[2];	/* Additive factors (* 100) */
	};

extern struct place *Place;
extern struct placedat *PlaceDat;
extern struct placeoff *PlaceOff;

extern Boolean OpenDatabase(void);
extern Boolean DeleteDatabase(int );
extern void DisplayDBInfo(int);
extern Boolean DBEnabled(int ) ;
extern Boolean DBEnable(int n, Boolean enable) ;
extern void SwapCoefficientData(void);
extern int GetPlaceData(DBIndex new_place);
extern void CloseDatabases(void);
extern void DeleteOrReloadDatabases(int);
extern int GetYearData(int);
extern void SavePreferences();
extern int GetDBEntry(int dbid);
extern void BeamTideTool(void);
#ifdef EDITOR
extern void DBDeletePlace(void);
#endif
long UnpackLat(struct place *);
long UnpackLon(struct place *);

/********************************************************
 *
 *	Display and event caching routines and globals
 *
 ********************************************************/

/* TideEvent values are relied upon in several place - do not change. */
extern enum TideEvent {
	HighTide=0, LowTide=1, LowSlack=2, HighSlack=3, MaxFlood=4, MaxEbb=5, Slack=6, 
	NoLabel }
 dis_high, dis_low, dis_rise, dis_fall;

struct DisplayEvent {			/* Data for one event */
	int eTime : 13;			/* Offset in minutes from midnight */
	enum TideEvent eType : 3;
	float eValue;
	};

struct ShortEvent{			/* Event without value stored - must start same as DisplayEvent */
	int eTime : 13;			/* Offset in minutes from midnight */
	enum TideEvent eType : 3;
	};

#define EVALID 1			/* display is valid (has any good events in it) */
#define ECOMPLETE 2		/* display is complete (all the day's events are in it) */
#define EVENTSPERDAY 8		/* How many events in a day */

extern Boolean ShowResult(time_t time, enum TideEvent eventno, float param);
extern void DisplayDate(void);
extern void RedoDisplay(Boolean);
extern void DisplayCurrentCondx(time_t when);
extern void InitializeGraph(time_t);
extern void DisplayGraphLine(int i, float value);
extern int HitSunMoon(int, int);
extern void MoveArrow(int);
extern void ReleaseDisplayHandle(void);
extern void InitThermometer(void);
extern void SetThermometer(int);
extern void EraseThermometer(void);
extern void InitializeDisplay();
extern void RestoreDisplay();

extern Boolean sys_35;		/* Whether Palm OS 3.0 and 3.5 feature sets are present */
extern Boolean sys_30;
extern Boolean scr_color;		/* Whether screen supports color */

extern enum State { 
	CSNoState = -1,
	CSIdle = 0,			/* No computation */
	CSStart,			/* Start a fresh recomputation */
	CSRestart,			/* Restart after computing coefficient */
	CSGraphing,			/* Graphing, if we're in graph mode */
	CSCoeff,			/* Computing tide coefficient */
	CSComputing,			/* Start computing another event */
	} 
  ComputeState;			/* State of computation, which happens in chunks */

extern char *AssertFile;
extern int AssertLine;
extern char NumspaceStr[2];
#define Numspace NumspaceStr[0]

#define SLACKER 90000.0		/* For slack tide: this amplitude displays blank */
#define T_EARLIER 80000.0	/* These specify an event on the previous (next) day */
#define T_LATER 70000.0

typedef enum {LENGTH, VELOCITY, BOGUS} unit_type;
typedef struct {
/*  char *name;  */
  char abbrv[7];
  unit_type type;
  float conv_factor;
  int places;
} units;

/* Units corresponding to preference form selections */
#define NUMUNITS 9
enum Units {
	feet, meters, knots, bogoknots,
	inches, mph, kph, msec, fsec, Default };
extern units known_units[NUMUNITS];		/* The conversion table */

#define DBTYPE_FIRST	25			/* Increase this by 3 every time db file format changes */
#define DBTYPE_ARGS	DBTYPE_FIRST	/* First byte of database header is type */
#define DBTYPE_LOC	(DBTYPE_FIRST+1)	/* Present values are for 2.3b1 and later */
#define DBTYPE_USER	(DBTYPE_FIRST+2)

struct ArgsDBHeader {		/* Header for tide factors database */
	char db_type;		/* Always 1 for factors DB */
	int params[4];		/* first_year, num_epochs, num_csts, num_nodes */
	time_t created;
	};

struct LocDBHdr {			/* Header for the locations database files */
	char db_type;				/* DBTYPE_LOC for place databases */
	char db_ID;				/* Unique identifier <= 63 */
	time_t db_created_date;
	int db_num_places;			/* Number of visible places */
	int db_default_tide_place;
	int db_default_current_place;
	int db_num_currents;
	int db_num_offsets;
	int db_num_curroffsets;
	int db_num_total_places;		/* Number of total places (including hidden) */
	long latfrom, latto, longfrom, longto;
	int db_first_year, 			/* To check for compatibility */
		db_num_csts, db_num_nodes; 
	DBIndex db_coeff;				/* Index of coefficient "location" */
	int db_notice;				/* Resource number of any notice the user needs to see */
	char db_name[MAXLOCSTR];		/* Name of database file */
	};

DmOpenRef ram;	
FileRef card;

struct DBdescriptor {			/* Structures for the transparent database routines */
	union { DmOpenRef ram;		/* Ref pointer either for RAM DB or VFS file */
		FileRef card;
	      } ref;
	UInt16 volume;			/* Volume number for VFS database; vfsInvalidVolRef for RAM DB */
	};
#define IN_RAM vfsInvalidVolRef

struct DBHandle {			/* Handle pointer for transparent database reference */
	union {
		VoidHand handle;
		struct LocDBHdr *hdr;
		struct Prefstruct *pref;	
		struct ArgsDBHeader *arghdr;
	      } p;
	Boolean in_ram;			/* Whether in RAM or on VFS */
	};
MemPtr QueryRecord(struct DBHandle *, struct DBdescriptor *, UInt16);
void HandleUnlock(struct DBHandle *);

extern int nPlaceDBs, nEnabledDBs;

#define NPLACEDBS 32
extern struct DBdescriptor PlaceDBs[NPLACEDBS];
extern struct DBHandle PlaceHdrHandles[NPLACEDBS];
extern struct LocDBHdr *PlaceHdrs[NPLACEDBS];

extern int num_places;
extern int num_currents;
extern int num_offsets;
extern int num_offcurrents;

#define F_NONE	-1		/* "Find" selection methods: */
#define F_LATLON	0		/*	Use proximity to lat/lon */
#define F_STG	1		/* 	Use substring search */
#define LLSHIFT	3		/* Shift lat/long by this to prevent overflow in compares */

struct FindStatStruct {		/* Structure for the status of the Find screen */
	char checksel;			/* Which selection method is checked */
	char focus;				/* Which lat/lon digit field is hot */
	char latlong[14];
	long latitude, longitude;	/* Okay, this is confusing.  In the database, lats are scaled */
	int slat;				/* by 10 and longs by 20.  Here they are the full long and slat */
	int slon;				/* and slon are the scaled versions (i.e., like in struct place) */
	int findstrlen;			/* Length of the string */
	char findstring[MAXLOCSTR];	/* String for string match selection - lowercased */
	};

typedef enum {
	dSun, dMon, dTue, dWed, dThu, dFri, dSat} week_days;		/* 0 through 6 */

#define M_WEEKDAY 7		/* Mask for day of the week */
#define ON_AFTER 0x08		/* Flags: First designated day >= the given date */
#define ON_DATE 0x10	 	/*        Exactly this date */

struct DSTswitch {
	char month;		/* January = 1; 0 means no DST */
	int hour;		/* Time of switch (in minutes after midnight) */
	char wkday;		/* Day of week plus flags */
	char mday;
	};

struct TimeZone {				/* 50 bytes per entry */
	char longname[20];
	char stdtime[L_ZONENAME];	/* Short names for standard time */
	char DSTtime[L_ZONENAME];	/*   and daylight time */
	long meridian;
	int DSTswitch;			/* Time added for DST, in seconds */
	struct DSTswitch toDST;
	struct DSTswitch fromDST;
	};

extern struct TimeZone Zones[];
extern struct TimeZone *timezone;

extern Boolean InLocForm;		/* Flag to determine if safe to draw in LocForm */

/**********************************************************************************
***
***	OK, real important here.  Whenever the Preference structure is changed to be
***	incompatible with older ones, PREFVERSTG needs to be changed.  Otherwise,
***	the old Preferences database will be retained when the user updates.
***
**********************************************************************************/

#define PREFVERSTG "2.2(4)"		/* Changed in v2.2 - tiny tweak to timezone offset */
#define NRECENT 32			/* Number of places to store in the recent places list */

extern struct Prefstruct {		/* Saved program state and user preferences */
	char db_type;
	char ver_stg[8];		/* Version number */
	Boolean Pfirstime;	/* If first time through program */
	Boolean Pfirstgraph;	/* If first time displaying a graph */
	unsigned int Phunits;	/* Height units preference */
	unsigned int Pvunits;	/* Velocity units preference */
	enum TimePrefs Ptunits;	/* Time units preference  - 0=hh:mm am, 1= hh:mm, 2=hhmm*/
	Boolean PignoreDST;	/* Whether to ignore Daylight Savings */
	time_t Ptoday;		/* The date program was last run */
	time_t Pdatedisplayed;	/* Date last displayed */
	Boolean Pdistoday;	/* Whether the current date was being displayed */
	Boolean Piscurrent;	/* Whether a tide or current was displayed */
	Boolean Pdisgraph;	/* Whether displaying a list of events or a graph */
	Boolean Puseoffsets;	/* Whether to list offsets in place list */
	Boolean Pusecoeffs;	/* Whether to display French tidal coefficients */
	DBIndex Plastplace;	/* The index of the last place displayed */
	DBIndex Plastcurrent;	/* The index of the last current place */
	DBIndex Plasttide;	/* The index of the last tide place */
	Boolean Pselectprev;	/* Whether Select form checkbox was "Previous" */
	enum { unknownColor = -1, noColor = 0, grayColor, fullColor }
 		Pcolor;		/* Whether color or not */
	Boolean Puselevel;	/* Whether the user has defined an alert level */
	int Plevelunits;		/*  and the units (ft., meters, in.) */
	float Plevel;		/*  and the value of that level */
	int PnRecentPlaces;	/* Number of places recently displayed */
	DBIndex RecentPlaces[NRECENT];	/*  and the places */
	struct FindStatStruct Pfind;	/* Most recent Find selections */
	UInt32 PDBDisabled[2];	/* Whether the database is enabled or not */
	UInt32 PDBNotified[2];	/* Set when the use has received any notice in the database */
	} Preferences;
/*
 * Note that this defines max database ID (64) and max number database entries (1024)
 *	and a lot of code assumes that the max database ID has this value.
 */
#define ID_SHIFT 10		/* an index is (index in db) | (db index in array << ID_SHIFT) */
#define ID_MASK ((1 << ID_SHIFT) - 1)

#define iscurrent Preferences.Piscurrent
#define DisplayingToday Preferences.Pdistoday
#define DateBeingDisplayed Preferences.Pdatedisplayed
#define disgraph Preferences.Pdisgraph
#define UseColor Preferences.Pcolor

/********************************************************
 *
 *	Place screen - tideplace.c: routines and globals
 *
 ********************************************************/

extern void InitLocList(int newchoice);
extern Boolean InitializingLocList(void);
extern void ShowLocList(void);
extern int CloseLocList(void);
extern void FreeLocList(void);
extern Boolean ScrollLocList(int direction);
extern struct place *GetLocSelection();
extern void InitLocForm(void);
extern void MoveLocList(int);
extern void setllprefs(struct FindStatStruct *, long, long);
extern Boolean LocFormOnTop();
extern void DrawTableItem(void *, Int16, Int16, RectangleType *);	/* Used to draw all tables */
extern void LocSelect(int row);
extern void LocScroll(struct sclRepeat *);

extern char choosecurrents;			/* Place chooser list: 0 =tides, 1 currents, 2 = alpha */
extern char latstg[LATLONLEN];		/* Shared between several routines which print lat/longs */
extern char lonstg[LATLONLEN];
extern int RegTableTop;				/* Top row of Regions table */

extern enum LocState { 			/* State variable for initializing location list */
	LocIdle = 0,			/* No computation */
	LocInit,			/* Initializing */
	LocDraw,			/* Draw the initial list */
	LocStuffNewDB,			/* Stuffing locations into list - next database */
	LocStuffDoingDB,		/* Stuffing locations into list - in middle of database */
	LocInsertStart,			/* Begin inserting in tree */
	LocInsert,			/* Continue inserting in tree */
	LocWalk,			/* Walk the tree */
	LocFinish,			/* Done - put up the list */
	} 
  LocListState;

#define strncmp(s1, s2, n) StrNCompare(s1,s2,n)

void far_InitializeTime();		/* Routine to overcome far call limitation */
extern struct tm * far_LocalTimeToDST( LocalTime_t* );
extern void far_do_timestamp (char *buf, char *dbuf, struct tm *t);
extern int far_CloseLocList(void);
extern void far_LatLonToA(char *latlon, long ll, char *which);
extern void far_HandleUnlock(struct DBHandle *);
