/* tideplace.c:  Pilot version of xtide tide table program
 *	Routines to handle Place screen.

Tide Tool - A tide and current prediction program for Palm handheld computers
Copyright (c) 1997-2003 Walt Bilofsky

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <PalmOS.h>
#include <PalmCompatibility.h> 

#include "sysinclude.h"
#include "tide.h"
#include "tidelib.h"
#include "Libraries\PalmOSGlue\PalmOSGlue.h"

/*
 *	Locations chooser form handling
 */
/*
 * Data for locations form
 */
FormPtr Locfrm;
enum LocState LocListState = LocIdle;	/* State variable for init location list */
static WinHandle LocFormWindow = NULL;
Boolean InLocForm = false;

static char **locptrs = NULL;		/* Pointer to array of locations */
static int *locindex = NULL;		/* Index in Locplaces for each list entry */
//static VoidHand dummylocs = NULL;	/* Strings to display "Sorting ..." */
static int loclisttop = 0;		/* Where the top of the list is now */
static int loclistselected = -1;	/* The current loclist selected item */
static Boolean init_table = true;	/* Whether table needs to be initialized */

static struct DBHandle *lochands = NULL;	/* Handles - needed for freeing up */
static int nlocitems;
static int treedex;			/* Index for tree walk */
static TablePtr Loctp = NULL;
typedef int ListIndex;
static struct listentry {
	DBIndex index;				/* index of record in database */
	struct place *place;			/* pointer to record content */
	ListIndex left;				/* Indexes in tree, for sorting */
	ListIndex right;
	char flag;					/* For sorting, if contains the search string */
	} *Locplaces = NULL;

char latstg[LATLONLEN] = "";			/* For several routines which print lat/longs */
char lonstg[LATLONLEN] = "";

char choosecurrents;

static void LocTideOrCur(void) {
 	FrmSetControlGroupSelection (FrmGetFormPtr(LocForm), TCGROUP, 
									LISTIDES + choosecurrents );
	FrmSetTitle (FrmGetFormPtr(LocForm),
		choosecurrents == 0 ? T_SELECT_LOCATION_FOR_TIDE :
		choosecurrents == 1 ? T_SELECT_LOCATION_FOR_CURRENT : 
						T_LOCNS_VIEWED_RECENTLY );
	}

void InitLocForm(void) {
	choosecurrents = -1;
	/* On entering locations form, always sort on current place */
	Preferences.Pfind.checksel = F_LATLON;	/* Use latitude and longitude */
	setllprefs(&Preferences.Pfind,latitude,longitude);
	Loctp = GetObjectPtr (LocList);
	InitLocList(Preferences.Pselectprev ? 2 : iscurrent);
	LocFormWindow  = FrmGetWindowHandle(FrmGetFormPtr(LocForm));
	}

#define TRIM360 ((360L * 60L * 60L) / 10L)
#define TRIM180 ((180L * 60L * 60L) / 10L)

/* Comparison functions for two listentries */
static long deltaLL(int Lat, int Long) {	      /* Return square of "distance" from point zero */
	long dlat, dlong;					/* Approximates by assuming lat & long are in same units */
	dlat = Lat - (long)Preferences.Pfind.slat;		/* Scale lat & long the same */
	dlong = 2L * (Long - (long)Preferences.Pfind.slon);
	if (dlong < 0) dlong = - dlong;		/* If near the date line, this will fix it */
	if (dlong > TRIM180) dlong -= TRIM360;
//printf("%d %ld %d %ld\n",Lat,dlat,Long,dlong);
#if 1
	if (dlat < -32767L || dlat > 32767L || dlong < -32767L || dlong > 32767L) {
		return 0x37777777; }
#else
/* Largest possible difference is 64800.  So if difference is over 16384, scale back */
/*  to 16383 + difference / 4 so square remains in range of long */
	if (dlat < -16383L || dlat > 16383L || dlong < -16383L || dlong > 16383L) {
		dlat = (dlat >> 2) + (32767 ^ (dlat >> 31));
		dlong = (dlong >> 2) + (32767 ^ (dlong >> 31));
		}
#endif
	return dlat*dlat + dlong*dlong;
	}

/* Return whether p contains the findstring - case insensitive */
static int stgin(char *p) {
	int i;
	for (i = StrLen(p) - Preferences.Pfind.findstrlen; i >= 0; --i, ++p) {
#if 1
		if (!StrNCaselessCompare(p,Preferences.Pfind.findstring,Preferences.Pfind.findstrlen))
			return 1;
#else
		char *q, *r;
		q = p;
		r = Preferences.Pfind.findstring;
		while (*r)						/* Step to end of findstring */
			if (tolower(*q) != *r) {		/* If a mismatch */
				if (*q) {
					break; }
				else return 1; } 			/* it's a find at the end of findstring */
			 else ++r, ++q;
		 if (!*r) return 1; 
#endif
		}
	return 0;							/* If q also exhausted it's a match at end */
	}

static long deltaentry;

static int compfn(ListIndex pentry, ListIndex proot) {
	long diff;
	struct place *entry = Locplaces[pentry].place;
	struct place *root = Locplaces[proot].place;
	switch (Preferences.Pfind.checksel) {
	   case F_LATLON:
		diff = deltaentry - deltaLL(root->latitude, root->longitude);
		return diff < 0 ? -1 : diff ? 1 : 0;
	   case F_STG:					/* String comparison: */
		if (Locplaces[pentry].flag != Locplaces[proot].flag)
			return Locplaces[pentry].flag ? -1 : 1;	/* One contains the search string */
		return strcmp(entry->location,root->location); 	/* Both or neither do - */
		}								/*  use alphabetical order */
	return 0;								/* Should not occur */
	}

static void InsertInList(ListIndex *root, ListIndex new) {
	Locplaces[new].left = Locplaces[new].right = -1;
	deltaentry = deltaLL(Locplaces[new].place->latitude, Locplaces[new].place->longitude);
	while (*root != -1)
		if (compfn(new,*root) < 0) 
			root = &(Locplaces[*root].left);  
		else root = &(Locplaces[*root].right);
	*root = new;
	}

/* Walk tree to construct sorted list.  Only recurses to left of tree, so comparison */
/*  functions should preferably bias to the right (i.e., >= 0) */
static void WalkTree(ListIndex p) {
	while (p != -1) {
		WalkTree(Locplaces[p].left);
		locptrs[treedex] = Locplaces[p].place->location;	/* Store place name */
		locindex[treedex++] = p;					/* and Locplaces index */
		p = Locplaces[p].right;
	}	}
		
/* Helper function: Put a qualifying location in the list.
   Return 1 for success, 0 for failure and -1 for error.  */
static int stuffloclist(int j, int db, int i) {
	struct place *p;
	struct DBHandle handle;
	Boolean ignoreme = false;
	if (db < 0 || db >= nPlaceDBs || !(p = QueryRecord(&handle, &PlaceDBs[db],2 * i + 1))) {
		barf(BADLOCNBASE);
		return -1; }
	if (choosecurrents != 2) {
		ignoreme = (choosecurrents == !(P_ISCURRENT & p->flags));
		if (p->flags & P_HIDDEN) ignoreme = true;	/* Rule this place out for present criteria? */
		if (!Preferences.Puseoffsets && (P_ISOFFSET & p->flags))
			ignoreme = true;
		if (ignoreme) {
			far_HandleUnlock(&handle);
			return 0;
		}	}
	assert (j < nlocitems);
	lochands[j] = handle;
	Locplaces[j].place = p;				/* Save the pointer */
	Locplaces[j].index = i | (PlaceHdrs[db]->db_ID << ID_SHIFT);
	Locplaces[j].left = Locplaces[j].right = -1;
	return 1;
	}

#define THERMBASE 750
void InitLocList(int newchoice) {
	if (newchoice < 0) newchoice = 0;
	if (choosecurrents == newchoice) return;
	if (newchoice == 1 && !num_currents) {
		FrmAlert (NOCURRENTS);
		newchoice = 0;
		}
	choosecurrents = newchoice;
	LocTideOrCur();
	FreeLocList();			/* In case it's active already */
	LocListState = LocInit;
	}

/*
 *	InitializingLocList stuffs all locations into loclist for sorting and display.
 *		Because it takes so long, the routine is called repeatedly so the
 *		event loop can continue responding.
 *	Returns true if an error has occurred.  In this case, event loop needs to
 *		terminate stuffing and display the error.
 */

Boolean InitializingLocList(void) {	/* Called repeatedly to perform loclist initialization */
  static int i, j, db, edb;
  static ListIndex listroot;
  static int temperature, tempinc, littleinc;
  switch(LocListState) {
   case LocIdle: return false;
   case LocInit:
	listroot = -1;
	nlocitems = choosecurrents == 2 ? Preferences.PnRecentPlaces :
			choosecurrents ? num_currents : num_places - num_currents;
	if (choosecurrents != 2 && !Preferences.Puseoffsets) nlocitems -= 
			choosecurrents ? num_offcurrents : num_offsets - num_offcurrents;
	locptrs = malloc(nlocitems * sizeof (char *));
	locindex = malloc(nlocitems * sizeof (int));
	lochands = malloc(nlocitems * sizeof (struct DBHandle));
	Locplaces = malloc(nlocitems * sizeof (struct listentry));
	if (AbortFlag) {
		AbortFlag = NOSORTMEM;
		FreeLocList();
		return true;
		}
	if (choosecurrents == 2) {	/* List is recent places - this is simple */
		for (i = 0; i < Preferences.PnRecentPlaces; ++i) {
			if (stuffloclist(i,GetDBEntry(Preferences.RecentPlaces[i] >> ID_SHIFT),
						Preferences.RecentPlaces[i] & ID_MASK) < 0) return true;
			locptrs[i] = Locplaces[i].place->location;	/* Store place name */
			locindex[i] = i;						/* and Locplaces index */
			}
		LocListState = LocFinish;
		return false;
		}
	LocListState = LocDraw;
	return false;
   case LocDraw:
	if (WinGetDrawWindow() != LocFormWindow) return false;	/* Don't proceed unless on top */
	ShowLocList();								/* Show "Sorting ..." */
	InitThermometer();
/*
 * Place applicable locations into the locations list
 */
	edb = j = temperature = 0;
	db = -1;
	tempinc = THERMBASE / nEnabledDBs;
	LocListState = LocStuffNewDB;
	return false;
   case LocStuffNewDB:				/* Process next database */
	for (;;) {					/* Loop until active database is found */
		if (++db >= nPlaceDBs) {	/* Done with databases */
			LocListState = LocInsertStart;
			return false;
			}
		if (!DBEnabled(db)) continue;
		SetThermometer(temperature = (THERMBASE * edb++) / nEnabledDBs);
//printf("DB %d: %d places\n",db,PlaceHdrs[db]->db_num_total_places);
		littleinc = (tempinc * 32)/ PlaceHdrs[db]->db_num_total_places;
		LocListState = LocStuffDoingDB;
		i = 0;
   case LocStuffDoingDB:			/* In the middle of processing a database */
		while (i < PlaceHdrs[db]->db_num_total_places) {
			switch (stuffloclist(j, db, i)) {
				case -1: return true;		/* Bad entry */
				case 1: ++j;
				}
			if (!(++i & 31)) {	/* Every 32 items, bump thermometer */
				SetThermometer(temperature += littleinc);
				if (PlaceDBs[db].volume != IN_RAM)
					return false;	/* and if on card, take a breath */
		   }    }
		}
	LocListState = LocStuffNewDB;		/* Go on to next database */
	return false;
   case LocInsertStart:
//printf("j=%d, nlocitems=%d\n",j,nlocitems); SysTaskDelay(100);
	assert (j == nlocitems);
/* 
 * Precompute whether locations contain the search string
 */
	if (Preferences.Pfind.checksel == F_STG)		/* Precompute whether contains srch stg */
		for (i = 0; i < nlocitems; ++i)
			Locplaces[i].flag = stgin(Locplaces[i].place->location); 
/*
 * Insert each location in the sort tree
 */
	i = 0;
	LocListState = LocInsert;
   case LocInsert:
	while (i < nlocitems) {
		InsertInList(&listroot, i);
		++i;
		if (!(i & 0x1F)) {				/* Move thermometer in x20 chunks */
			SetThermometer(THERMBASE + (int)(((1000L - THERMBASE) * i)/nlocitems));
			return false;				/* Compute in x80 chunks */
		}	}
	treedex = 0;
	SetThermometer(1000);
	LocListState = LocWalk;
	return false;
/*
 * Walk the sort tree in order to generate the sorted list to display
 */
   case LocWalk:
	WalkTree(listroot);
	LocListState = LocFinish;
	return false;
   case LocFinish:
	if (WinGetDrawWindow() == LocFormWindow) {	/* Don't finish unless loc form on top */
		if (!InLocForm) return false;		/*  and any menu has really gone away */
		EraseThermometer();			/*  or else menu will blank out list */
		loclistselected = 0;			/* Select top item */
		ShowLocList();				/* Display sorted list */
//		TblSelectItem(GetObjectPtr(LocList),0,0);
		LocListState = LocIdle;			/* Signal that list creation is done */
	}	}
   return false;
   }

Boolean LocFormOnTop() { return WinGetDrawWindow() == LocFormWindow; }

Boolean ScrollLocList(int key) {		/* Process keydown in places form */
	switch (key) {				/* Return whether handled */
	   case 11:
		MoveLocList(loclisttop - LOCNITEMS + 1);
		break;
	   case 12:
		MoveLocList(loclisttop + LOCNITEMS - 1);
		break;
	   default:
		return 0;
		}
	return 1;
	}

void MoveLocList(int where) {
	if (where > nlocitems - LOCNITEMS) where = nlocitems - LOCNITEMS;
	if (where < 0) where = 0;
	loclisttop = where;
	ShowLocList();
	}

void LatLonToA(char *latlon, long ll, char *which) {
	char *p = latlon;
	long deg, min;
	int i = 0;
	if (ll < 0) { i = 1; ll = -ll; }
	deg = ll / (60*60);
	ll -= deg * (60*60);
	min = ll/60;
	ll -= min * 60;
	if (deg < 100) *p++ = Numspace;  /* Number width blank */
	if (deg < 10) *p++ = Numspace;
	p += sprintf(p,"%ld\260 ",deg);
	if (min < 10) *p++ = Numspace;
	p += sprintf(p,"%ld' ",min);
	if (ll < 10) *p++ = Numspace;
	sprintf(p,"%ld\"%c",ll,which[i]);
	}

/* Returns number of database record selected */
int CloseLocList(void) {
	int i = loclistselected == -1 ? PlaceIndex :
			Locplaces[locindex[loclistselected]].index; 
	FreeLocList();
	Loctp	 = NULL;
	latstg[0] = lonstg[0] = 0;
	return i;
	}

void FreeLocList(void) {
	int i;
	if (lochands) {
		for (i = 0; i < nlocitems; ++i) 
			if (lochands[i].p.handle) far_HandleUnlock(&lochands[i]);
		free(lochands);
		lochands = NULL;
		}
	if (Locplaces) {
		free(Locplaces);
		Locplaces = NULL;
		}
	if (locptrs) {
		free(locptrs);
		locptrs = NULL;
		}
	if (locindex) {
		free(locindex);
		locindex = NULL;
		}
//	if (dummylocs) {
//		MemHandleFree(dummylocs);
//		dummylocs = NULL;
//		}
	LocListState = LocIdle;
	loclisttop = 0;
	loclistselected = -1;
	init_table = true;
	}

struct place *GetLocSelection() {			/* Only called from Find screen */
	return loclistselected == -1 ? Place :	/* If nothing selected, use current place */
			Locplaces[locindex[loclistselected]].place; 
	}

static void	unmakelat(long l, char *p) {		/* Convert lat/long to digit array for form */
	int i;
	if (l >= 0) p[6] = 0;
	  else {l = -l; p[6] = 1; }
	i = l % 60;
	l /= 60;
	*p = l / 600;
	l -= *p++ * 600L;
	*p = l / 60; l -= *p++ * 60L;
	*p++ = l / 10;
	*p++ = l % 10;
	*p++ = i / 10;
	*p = i % 10;
	}

void setllprefs(struct FindStatStruct *p, long latitude, long longitude) {
	p->latitude = latitude;	
	p->slat = latitude / 10;
	p->longitude = longitude;
	p->slon = longitude / 20;
	unmakelat(latitude,&p->latlong[0]);
	unmakelat(longitude,&p->latlong[7]);
	}

#define LLX 80
#define LATY 106
#define LONY LATY + 12
#define SLIDE_IGNORE 25					/* Ticks to ignore an arrow and a scroller, resp. */
static int ScrollMax;
static time_t lastslidetime = 0;

static void LocShowLatLong() {
	FntSetFont(0);
	WinEraseChars (latstg, (int)strlen(latstg), LLX, LATY);
	WinEraseChars (lonstg, (int)strlen(lonstg), LLX, LONY);
	if (loclistselected >= 0) {
		LatLonToA(latstg, UnpackLat(Locplaces[locindex[loclistselected]].place), T_NORTH_SOUTH);
		WinDrawChars (latstg, (int)strlen(latstg), LLX, LATY);
		LatLonToA(lonstg, UnpackLon(Locplaces[locindex[loclistselected]].place), T_EAST_WEST);
		WinDrawChars (lonstg, (int)strlen(lonstg), LLX, LONY);
		WinDrawLine(0, LATY - 3 ,159, LATY - 3);
	}	}

void LocSelect(int row) {
	loclistselected = loclisttop + row;
	LocShowLatLong();
	}

void LocScroll(struct sclRepeat *ss) {	/* Handle a scroll repeat event */
	int delta;
	time_t t = TimGetTicks();
	delta = ss->value - ss->newValue;
	if (delta < 0) delta = -delta;
	if (delta == 1 					/* If arrow, and the last slide was just a moment ago */
			&& t - lastslidetime < SLIDE_IGNORE && ss->newValue != ScrollMax) {
		SclSetScrollBar (GetObjectPtr(LocScrollbar), loclisttop, 0, ScrollMax, LOCNITEMS);
//printf("Ign %d val %d dt %ld\n",delta, ss->newValue, t - lastslidetime);
		return; }				/*   put bar back to where it was and ignore. */
//printf("Mvd %d val %d dt %ld\n",delta, ss->newValue, t - lastslidetime);
	lastslidetime = t;
	MoveLocList(ss->newValue);
	}

void ShowLocList(void) { 
	TablePtr table;
	int row;

	LocShowLatLong();

	table = GetObjectPtr(LocList);
	if (init_table) {
		init_table = false;
		for (row = 0; row < LOCNITEMS; ++row)
		   if (loclisttop + row >= nlocitems) TblSetRowUsable (table, row, false);
		    else {
			TblSetItemStyle (table, row, 0, customTableItem);
			TblSetRowHeight (table, row, LOCROWHT);
			TblSetRowUsable (table, row, true);
			TblSetRowID (table, row, row+loclisttop);
			}
		TblSetCustomDrawProcedure (table, 0, DrawTableItem);
		TblSetColumnUsable (table, 0, true); 
		}
	/* Update scroll bar */
	if ((ScrollMax = nlocitems - LOCNITEMS) < 0) ScrollMax = 0;
	TblHasScrollBar (table, true);
	SclSetScrollBar (GetObjectPtr(LocScrollbar), loclisttop, 0, ScrollMax, LOCNITEMS);
	TblUnhighlightSelection(table);
	TblDrawTable(table);
	if ((row = loclistselected - loclisttop) >= 0 && row < LOCNITEMS)
		TblSelectItem(table,row,0);	 /* Select the right row */
	}

void DrawTableItem(void *table, Int16 row, Int16 col, RectangleType *bounds) {
	char *p;
	Boolean InLocForm = (FrmGetActiveFormID() == LocForm);

	WinEraseRectangle(bounds,0);
	if ((row += loclisttop) >= (InLocForm ? nlocitems : nPlaceDBs)) return;
	if (col == 0 && !InLocForm) {				
		FntSetFont(4);					/* Draw i icon for Regions tbl */
		p = "\004";
		}
	  else {							/* Draw text item */
		FntSetFont(0);
		p = (!InLocForm ? 				/* Get the text string for table entry */
			((row += RegTableTop) >= nPlaceDBs ? "" : PlaceHdrs[row]->db_name) : 
			loclistselected >= 0 ? 	
				(row >= nlocitems ? "" : Locplaces[locindex[row]].place->location) :
			row == 4 ? T_SORTING : "");		/* If sorting, the entry displays that */
//if (loclistselected >= 0) { printf("Draw %d loc?%d %s\n",row-loclisttop,InLocForm,p); SysTaskDelay(50);}
		}
	WinGlueDrawTruncChars (p,strlen(p),bounds->topLeft.x,bounds->topLeft.y,
				TblGetColumnWidth(table,col));
	FntSetFont(0);
	}

