/* tidetext.h:  Text strings for Tide Tool

Tide Tool - A tide and current prediction program for Palm handheld computers
Copyright (c) 1997-2003 Walt Bilofsky

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
/*
 * CAUTIONS TO TRANSLATOR:
 * 
 * 1. Only text between quotes "..." should be translated - not comments 
 *    (between / * and * /) or the names in capital letters.
 *
 * 2. Do not change special characters and characters following "\" or %.
 *	These are used in formatting strings for printing out.  Wherever
 *    %d appears in a string, the program replaces it by a number.  %s will
 *    be replaced by either a text string or, sometimes, a number.
 *
 * 3. Try to keep strings short.  They may be fitting into a small
 *	space on the screen.  When in doubt, run Tide Tool and look at
 *	the display.
 *
 * 4. For terms having to do with tides, look at http://www.shom.fr .
 */

/* Strings in tide.c
 *	General error messages
 */
#define T_WEEKDAYS "M\0T\0W\0T\0F\0S\0S"		/* 1 letter abbreviations of weekdays */
#define T_TABLE "Table"					/* MU: Used in multiple places */
#define T_GRAPH "Graph"					/* MU */
#define T_TIDES_CAN_ONLY_BE_COMPUTED "Tides can only be computed for years %d through %d."
#define T_THE_DATABASE_FILE "The database file\"%s\""
#define T_IS_BLOOEY " is %s.  Delete Tide Tool and reinstall."
#define T_CORRUPT "corrupt"
#define T_FROM_OLD_VERSION "from an old version of Tide Tool"
#define T_FATAL_ERR "Fatal error in file %s line %d"
#define T_NORTH_SOUTH "NS"				/* MU */
#define T_EAST_WEST "EW"				/* MU */
#define T_NORTH "N"
#define T_SOUTH "S"
#define T_EAST "E"
#define T_WEST "W"
#define T_DISPLAY_COLORS "Display colors                  "
#define T_ERR_CODE " (code %s)"

/* Strings in tidedb.c 
 *	Database error messages
 *	Information displays from menu commands
 */

#define T_ONLYDB "  It is the only one installed."
#define T_TTAPP "Tide Tool application"
#define T_TTDBASE "Tide Tool harmonic factors database"
#define T_REGIONAL_DB " regional database"
#define T_BEAMING_INCOMPLETE "Beaming did not complete."
#define T_REFERENCE_STA "Reference station: %s\n"
#define T_HAS_N_HARMONIC_FACTORS "Station has %d harmonic factors.\n"
#define T_NO_ZERO_RECORD "A database has no 0 record."
#define T_REGION "Region: %s\n"
#define T_LOC_DB_NR "Location Database #%d\n%s\n"
#define T_TIDES_CURRENTS_AREA "%d tides, %d currents\nArea of coverage:\n    %s to\n"
#define T__TO "    %s to\n"
#define T_ALGORITHM_DATABASE_COVERS "Algorithm database covers %d to %d using %d factors.\n"
#define T_CREATED "Created %s"
#define T_STORED "Stored %s\n"
#define T_IN_RAM "in RAM"
#define T_ON_CARD "on card"

/* Strings in tidedis.c 
 *	Tide and current display
 */

#define T_HIGHTIDE "\200High Tide:"
#define T_LOWTIDE "\200 Low Tide:"
#define T_SLACKWATER "\200\200\200   Slack:"
#define T_MAX_FLOOD "Max Flood:"
#define T_MAX_EBB "\200 Max Ebb:"
#define T_FEET_ABBR " ft"
#define T_METERS_ABBR " m"
#define T_KNOT_ABBR " kt"
#define T_KNOTSQ_ABBR " kt^2"
#define T_INCH_ABBR " in"
#define T_MPH_ABBR " mph"
#define T_KMPH_ABBR " km/h"
#define T_MTRS_SEC_ABBR " m/sec"
#define T_FT_SEC_ABBR " f/sec"
/* For the "Tide is now" line on the tide/current display */
#define T_TIDE_CURRENT_DESCR {". Tide", ". Current", ", tide", ", current"}  /* For present state of tide/curr */
#define T_AT "At "
#define T_NOW "Now "
#define T_IS "is"
#define T_WILLBE_WAS "will be" : "was"
#define T_RISING_FALLING " rising." : " falling."
#define T_COEFF "Coeff."

/* Strings in tidedis.c 
 *	Tide and current display
 */

#define T_SELECT_LOCATION_FOR_TIDE "Select Location for Tide"
#define T_SELECT_LOCATION_FOR_CURRENT "Select Location for Current    " 
#define T_LOCNS_VIEWED_RECENTLY "Locations Viewed Recently    "
#define T_SORTING "                               Sorting ..."

/* Strings in tidetime.c 
 *	Locations, astronomical functions
 */

#define T_AMERICA_PACIFIC "America/Pacific"
#define T_AMERICA_CENTRAL "America/Central"
#define T_AMERICA_EASTERN "America/Eastern"
#define T_US_ALASKA "US/Alaska"
#define T_US_HAWAII_ALEUTIAN "US/Hawaii-Aleutian"
#define T_AMERICA_MOUNTAIN "America/Mountain"
#define T_CANADA_PACIFIC "Canada/Pacific"
#define T_AMERICA_HAVANA "America/Havana"
#define T_BRAZIL_EAST "Brazil/East"
#define T_ATLANTIC_STANLEY "Atlantic/Stanley"
#define T_AMERICA_SANTIAGO "America/Santiago"
#define T_AMERICA_ATLANTIC "America/Atlantic"
#define T_CANADA_NEWFOUNDLAND "Canada/Newfoundland"
#define T_W_GREENLAND "W. Greenland"
#define T_GREENWICHMEANTIME "Greenwich Mean Time"
#define T_EUROPE_LONDON "Europe/London"
#define T_EUROPE_DUBLIN "Europe/Dublin"
#define T_ATLANTIC_AZORES "Atlantic/Azores"
#define T_WESTERNEUROPE "Western Europe"
#define T_CENTRALEUROPE "Central Europe"
#define T_EUROPE_MOSCOW "Europe/Moscow"
#define T_EGYPT "Egypt"
#define T_ASIA_TEHRAN "Asia/Tehran (appr.)"
#define T_ASIA_BAGHDAD "Asia/Baghdad"
#define T_NAMIBIA "Namibia"
#define T_ASIA_VLADIVOSTOK "Asia/Vladivostok"
#define T_ASIA_MAGADAN "Asia/Magadan"
#define T_ASIA_KAMCHATKA "Asia/Kamchatka"
#define T_NEW_ZEALAND "New Zealand"
#define T_AUSTRALIA_EASTERN "Australia/Eastern"
#define T_AUSTRALIA_CENTRAL "Australia/Central"
#define T_AUSTRALIA_WESTERN "Australia/Western"
#define T_AUSTRALIA_SEASTERN "Australia/S.Eastern"
#define T_AUSTRALIA_SCENTRAL "Australia/S.Central"
#define T_AUSTRALIA_LORDHOWE "Australia/Lord Howe"
#define T_GMTZ "GMT"			/* MU */
#define T_CEST "CEST"
#define T_CEDT "CEDT"
#define L_ZONENAME 6			/* Longest timezone plus 1 */

#define T_GO_TO_DATE "Go To Date"
#define T_MONTHS_OF_YEAR "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"
#define T_DAYS_OF_WEEK "Sun","Mon","Tue","Wed","Thu","Fri","Sat"
#define T_GMT "GMT%s%d"
#define T_FIRST "1st"			/* Weeks in the month */
#define T_SECOND "2nd"
#define T_THIRD "3rd"
#define T_FOURTH "4th"
#define T_LAST "last"
#define T_THE_DAY_ON_OR_AFTER "the %s. on or after %s. %d"
#define T_THE_NTH_WKDAY_OF_MONTH "the %s %s. of %s."
/* 
 * Display of regional database information
 */
#define T_USES "%s uses "
#define T_TIMEZONE_WHICH_IS "%s time, which is "
#define T_HOUR " hour"
#define T_HOURS " hours"
#define T_DST_NOT_OBSERVED "Daylight savings is not observed."
#define T_YOU_DISABLED_DST "You disabled DST.  Normally, "
#define T_SWITCHES_TO_DST "%s switches to %s by adding "
#define T_HOUR_AT " hour at "
#define T_ON " on "
#define T_UNTIL ", until "
#define T_AYEM " am"
#define T_PM " pm"
/*
 * Sun and moon information
 */
#define T_MOONPHASES "\n   Last qtr.: ", "\nNew moon: ","\n      1st qtr.: ", "\n   Full moon: ",
#define T_ON_DATE_MOON "On "
#define T_MOON_INTRO ":\n"
#define T_MOON "    Moon %s:"
#define T_RISES_SETS "rises" : "sets "
#define T_MOON_IS_ALL_DAY "    Moon is %s all day\n"
#define T_DOWN_UP "down" : "up"
#define T_ON_DATE_SUN "On "
#define T_SUN_INTRO ":\n\n"
#define T_SUN_RISES_AT " Sun rises at "
#define T_SUN_SETS_AT " Sun sets at  "
#define T_SUN_IS " Sun is "
#define T_UP_DOWN "up" : "down"
#define T_ALL_DAY_TODAY " all day today.\n"
#define T_HRS_MIN_OF_DAYLIGHT "\n%dh %dm of daylight"


