/* tidetext.h:  Text strings for Tide Tool

Tide Tool - A tide and current prediction program for Palm handheld computers
Copyright (c) 1997-2003 Walt Bilofsky

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
/*
 * CAUTIONS TO TRANSLATOR:
 * 
 * 1. Only text between quotes "..." should be translated - not comments 
 *    (between / * and * /) or the names in capital letters.
 *
 * 2. Do not change special characters and characters following "\" or %.
 *	These are used in formatting strings for printing out.  Wherever
 *    %d appears in a string, the program replaces it by a number.  %s will
 *    be replaced by either a text string or, sometimes, a number.
 *
 * 3. Try to keep strings short.  They may be fitting into a small
 *	space on the screen.  When in doubt, run Tide Tool and look at
 *	the display.
 *
 * 4. For terms having to do with tides, look at http://www.shom.fr .
 */

/* Strings in tide.c
 *	General error messages
 */
#define T_WEEKDAYS "L\0M\0M\0J\0V\0S\0D"		/* 1 letter abbreviations of weekdays */
#define T_TABLE "Table"					/* MU: Used in multiple places */
#define T_GRAPH "Graphe"				/* MU */
#define T_TIDES_CAN_ONLY_BE_COMPUTED "Les mares ne peuvent tre calcules que pour les annes %d  %d."
#define T_THE_DATABASE_FILE "Le fichier base de donnes\"%s\""
#define T_IS_BLOOEY " est %s.  Supprimez Tide Tool et rinstallez."
#define T_CORRUPT "corrompu"
#define T_FROM_OLD_VERSION "d'une version antrieure de Tide Tool"
#define T_FATAL_ERR "Erreur fatale sur le fichier %s ligne %d"
#define T_NORTH_SOUTH "NS"				/* MU */
#define T_EAST_WEST "EO"				/* MU */
#define T_NORTH "N"
#define T_SOUTH "S"
#define T_EAST "E"
#define T_WEST "O"
#define T_DISPLAY_COLORS "Afficher les couleurs                  "
#define T_ERR_CODE " (code %s)"

/* Strings in tidedb.c 
 *	Database error messages
 *	Information displays from menu commands
 */

#define T_ONLYDB "  C'est le seul install."
#define T_TTAPP "l'application Tide Tool"
#define T_TTDBASE "la base de donnes des constantes harmoniques de Tide Tool"
#define T_REGIONAL_DB "la base de donnes rgionale "
#define T_BEAMING_INCOMPLETE "Le transfert n'a pu tre finalis."
#define T_REFERENCE_STA "Port de reference : %s\n"
#define T_HAS_N_HARMONIC_FACTORS "Le port a %d constantes harmoniques.\n"
#define T_NO_ZERO_RECORD "Une base de donnes n'a pas 0 enregistrement."
#define T_REGION "Rgion : %s\n"
#define T_LOC_DB_NR "Base de ports #%d\n%s\n"
#define T_TIDES_CURRENTS_AREA "%d mares, %d courants\nZone de couverture :\n    %s \n"
#define T__TO "    %s \n"
#define T_ALGORITHM_DATABASE_COVERS "La base d'algorithmes couvre %d  %d en utilisant les constantes %d.\n"
#define T_CREATED "Cr %s"
#define T_STORED "%s\n"
#define T_IN_RAM "Stock en RAM"
#define T_ON_CARD "Sur carte mmoire"

/* Strings in tidedis.c 
 *	Tide and current display
 */

#define T_HIGHTIDE "Pleine mer :"
#define T_LOWTIDE "Basse mer :"
#define T_SLACKWATER "\200\200\200  Etale :"
#define T_MAX_FLOOD "\200\200\200\200  Flot :"
#define T_MAX_EBB "\200\200 Jusant :"
#define T_FEET_ABBR " ft"
#define T_METERS_ABBR " m"
#define T_KNOT_ABBR " nd"
#define T_KNOTSQ_ABBR " nd^2"
#define T_INCH_ABBR " in"
#define T_MPH_ABBR " mph"
#define T_KMPH_ABBR " km/h"
#define T_MTRS_SEC_ABBR " m/s"
#define T_FT_SEC_ABBR " ft/s"

/* For the "Mare actuelle" line on the tide/current display */
#if 1
#define T_TIDE_CURRENT_DESCR {" la mare", " le courant", " la mare", " le courant" }  /* For present state of tide/curr */
#define T_AT " "
#define T_NOW " "
#define T_IS "est "
#define T_WILLBE_WAS "sera " : "tait "
#define T_RISING_FALLING " mont" : " desc"
#else
#define T_TIDE_CURRENT_DESCR {" - hauteur", " - vitesse", " - hauteur", " - vitesse" }  /* For present state of tide/curr */
#define T_AT ""
#define T_NOW ""
#define T_IS ":"
#define T_WILLBE_WAS ":" : ":"
#define T_RISING_FALLING " - mont" : " - desc"
#endif

#define T_COEFF "Coeff"

/* Strings in tidedis.c 
 *	Tide and current display
 */

#define T_SELECT_LOCATION_FOR_TIDE "Choix du port pour la mare     "
#define T_SELECT_LOCATION_FOR_CURRENT "Choix du port pour le courant   " 
#define T_LOCNS_VIEWED_RECENTLY "Ports visualiss rcemment     "
#define T_SORTING "                           Tri en cours..."

/* Strings in tidetime.c 
 *	Locations, astronomical functions
 */

#define T_AMERICA_PACIFIC "Amerique/Pacifique"
#define T_AMERICA_CENTRAL "Amerique/Centrale"
#define T_AMERICA_EASTERN "Amerique/Est"
#define T_US_ALASKA "US/Alaska"
#define T_US_HAWAII_ALEUTIAN "US/Hawa-Alout'nes"
#define T_AMERICA_MOUNTAIN "Amerique/Montagne"
#define T_CANADA_PACIFIC "Canada/Pacifique"
#define T_AMERICA_HAVANA "Amerique/Havane"
#define T_BRAZIL_EAST "Brsil/Est"
#define T_ATLANTIC_STANLEY "Atlantique/Stanley"
#define T_AMERICA_SANTIAGO "Amerique/Santiago"
#define T_AMERICA_ATLANTIC "Amerique/Atlantique"
#define T_CANADA_NEWFOUNDLAND "Canada/Newfoundland"
#define T_W_GREENLAND "O. Groenland"
#define T_GREENWICHMEANTIME "Temps Universel GMT"
#define T_EUROPE_LONDON "Europe/Londres"
#define T_EUROPE_DUBLIN "Europe/Dublin"
#define T_ATLANTIC_AZORES "Atlantique/Aores"
#define T_WESTERNEUROPE "Europe de l'Ouest"
#define T_CENTRALEUROPE "Europe Centrale"
#define T_EUROPE_MOSCOW "Europe/Moscou"
#define T_EGYPT "Egypte"
#define T_ASIA_TEHRAN "Asie/Thran (appr.)"
#define T_ASIA_BAGHDAD "Asie/Baghdad"
#define T_NAMIBIA "Namibie"
#define T_ASIA_VLADIVOSTOK "Asie/Vladivostok"
#define T_ASIA_MAGADAN "Asie/Magadan"
#define T_ASIA_KAMCHATKA "Asie/Kamchatka"
#define T_NEW_ZEALAND "Nouvelle Zlande"
#define T_AUSTRALIA_EASTERN "Australie/Est"
#define T_AUSTRALIA_CENTRAL "Australie/Centre"
#define T_AUSTRALIA_WESTERN "Australia/Ouest"
#define T_AUSTRALIA_SEASTERN "Australie/Sud Est"
#define T_AUSTRALIA_SCENTRAL "Australie/Sud Centre"
#define T_AUSTRALIA_LORDHOWE "Australie/Lord Howe"
#define T_GMTZ "TU"			/* MU */
#define T_CEST "TU+1"
#define T_CEDT "TU+2"
#define L_ZONENAME 6			/* Longest timezone plus 1 */

#define T_GO_TO_DATE "Aller  la date"
#define T_MONTHS_OF_YEAR "jan","fv","mars","avr","mai","juin","juil","aot","sept","oct","nov","dc"
#define T_DAYS_OF_WEEK "dim","lun","mar","mer","jeu","ven","sam"
#define T_GMT "TU%s%d"
#define T_FIRST "1er"			/* Weeks in the month */
#define T_SECOND "2me"
#define T_THIRD "3me"
#define T_FOURTH "4me"
#define T_LAST "dernier"
#define T_THE_DAY_ON_OR_AFTER "%s de ou aprs %s %d"
#define T_THE_NTH_WKDAY_OF_MONTH "%s %s de %s"
/* 
 * Display of regional database information
 */
#define T_USES "%s utilise l'heure "
#define T_TIMEZONE_WHICH_IS "%s, soit "
#define T_HOUR " heure"
#define T_HOURS " heures"
#define T_DST_NOT_OBSERVED "L'heure d't n'existe pas."
#define T_YOU_DISABLED_DST "Vous avez dsactiv l'heure d't.  Normalement, "
#define T_SWITCHES_TO_DST "%s passe en %s par ajout d' "
#define T_HOUR_AT " heure ( "
#define T_ON ") le "
#define T_UNTIL " et ce jusqu'u "
#define T_AYEM " am"
#define T_PM " pm"
/*
 * Sun and moon information
 */
//#define T_MOONPHASES "Dernier quartier: ", "Nouvelle lune : ","1er quartier: ", "Pleine lune: ",
#define T_MOONPHASES "\nDernier qtr : ", "\nNouv. lune : ","\n1er qtr : ", "\nPleine lune : ",
#define T_ON_DATE_MOON "Infos lune: "
#define T_MOON_INTRO "\n"
#define T_MOON "%s"
#define T_RISES_SETS "Lever  :     " : "Coucher  :"
#define T_MOON_IS_ALL_DAY "    La lune est %s toute la journe\n"
#define T_DOWN_UP "bas" : "haut"
#define T_ON_DATE_SUN "Infos soleil: "
#define T_SUN_INTRO "\n\n"
#define T_SUN_RISES_AT "Lever  :      "
#define T_SUN_SETS_AT "Coucher  :"
#define T_SUN_IS "Est "
#define T_UP_DOWN "haut" : "bas"
#define T_ALL_DAY_TODAY " toute la journe aujourd'hui.\n"
#define T_HRS_MIN_OF_DAYLIGHT "\nDure du jour : %dh %dm"


