/* tidetime.c:  Display, time and math routines for Pilot version 
 *	of xtide tide table program
 *
Tide Tool - A tide and current prediction program for the USR PalmPilot
Copyright (c) 1997-2000 Walt Bilofsky

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#define MOONRISE

#include <PalmOS.h>
#include <PalmCompatibility.h>

/* Includes for doing Unix time functions on PalmOS - Walt's own */
#include "sysinclude.h"
#include "tide.h"
#include "tidelib.h"

char WeekStartDay;
DateFormatType LongDateFormat;	

static void computeDST(DateTimeUnion *dt);
static void fixtimedates(Boolean);

struct TimeZone Zones[] = {
	{ T_AMERICA_PACIFIC, 	"PST", "PDT", -8*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_AMERICA_CENTRAL, 	"CST", "CDT", -6*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_AMERICA_EASTERN, 	"EST", "EDT", -5*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_US_ALASKA, 		"AKST","AKDT",-9*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_US_HAWAII_ALEUTIAN,	"HAST","HADT",-10*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_AMERICA_MOUNTAIN,	"MST", "MDT", -7*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_CANADA_PACIFIC, 	"PST", "PDT", -8*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_AMERICA_HAVANA, 	"CST", "CDT", -5*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,1} },
	{ T_BRAZIL_EAST,   	"EST","EDT", -3*60,60,{10,0*60,ON_AFTER|dSun,11},{2,0*60,ON_AFTER|dSun,11} },
	{ T_ATLANTIC_STANLEY, 	"FKST","FKDT", -4*60,60,{9,0*60,ON_AFTER|dSun,8}, {4,0*60,ON_AFTER|dSun,16} },
	{ T_AMERICA_SANTIAGO, 	"CLST","CLDT", -4*60,60,{10,0*60,ON_AFTER|dSun,9}, {3,0*60,ON_AFTER|dSun,9} },
	{ T_AMERICA_ATLANTIC, 	"AST", "ADT", -4*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_CANADA_NEWFOUNDLAND, "NST", "NDT", -(3*60+30),60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_W_GREENLAND, 		"WGST","WGDT",-3*60,60,{4,2*60,ON_AFTER|dSun,1},{10,2*60,ON_AFTER|dSun,25} },
	{ T_GREENWICHMEANTIME,	T_GMTZ, "",     0,0,{0,0,0,0},{0,0,0,0} },
	{ T_EUROPE_LONDON,   	T_GMTZ, "BST",        0,60,{3,1*60,ON_AFTER|dSun,25}, {10,1*60,ON_AFTER|dSun,25} },
	{ T_EUROPE_DUBLIN,   	T_GMTZ, "IST",        0,60,{3,1*60,ON_AFTER|dSun,25}, {10,1*60,ON_AFTER|dSun,25} },
	{ T_ATLANTIC_AZORES,   	"AZOST","AZODT",-1*60,60,{3,2*60,ON_AFTER|dSun,25}, {10,2*60,ON_AFTER|dSun,25} },
	{ T_WESTERNEUROPE,   	"WEST","WEDT",    0,60,{3,2*60,ON_AFTER|dSun,25}, {10,2*60,ON_AFTER|dSun,25} },
	{ T_CENTRALEUROPE,   	T_CEST,T_CEDT, 1*60,60,{3,2*60,ON_AFTER|dSun,25}, {10,2*60,ON_AFTER|dSun,25} },
	{ T_EUROPE_MOSCOW,   	"MSK","MSD",   3*60,60,{3,2*60,ON_AFTER|dSun,25}, {10,2*60,ON_AFTER|dSun,25} },
	{ T_EGYPT,   		"EEST","EEDT",   2*60,60,{4,0*60,ON_AFTER|dFri,24}, {9,0*60,ON_AFTER|dFri,24} },
	{ T_ASIA_TEHRAN,		"IRST","IRDT", 3*60+30,60,{3,0*60,ON_DATE,21}, {9,0*60,ON_DATE,23} },
	{ T_ASIA_BAGHDAD,   	"AST","ADT",   3*60,60,{4,3*60,ON_DATE,1}, {10,3*60,ON_DATE,1} },
	{ T_NAMIBIA, 		"SAST", "SADT", 2*60,60,{4,2*60,ON_AFTER|dSun,1},{9,2*60,ON_AFTER|dSun,1} },
	{ T_ASIA_VLADIVOSTOK,   "VLAST","VLADT",10*60,60,{3,2*60,ON_AFTER|dSun,25},{10,2*60,ON_AFTER|dSun,25} },
	{ T_ASIA_MAGADAN,   	"MAGST","MAGDT",11*60,60,{3,2*60,ON_AFTER|dSun,25},{10,2*60,ON_AFTER|dSun,25} },
	{ T_ASIA_KAMCHATKA,   	"PETST","PETDT",12*60,60,{3,2*60,ON_AFTER|dSun,25},{10,2*60,ON_AFTER|dSun,25} },
	/* Following repaired in 2.1b */
	{ T_NEW_ZEALAND,		"NZST", "NZDT",12*60,60,{10,2*60+45,ON_AFTER|dSun,1},{3,2*60+45,ON_AFTER|dSun,15} },
	/* Following repaired in 1.1b4 (second goaround) */
	{ T_AUSTRALIA_EASTERN, "EST", "", 10*60,0,{0,0,0,0},{0,0,0,0} },
	{ T_AUSTRALIA_CENTRAL, "CST", "", 9*60+30,0,{0,0,0,0},{0,0,0,0} },
	{ T_AUSTRALIA_WESTERN, "WST", "", 8*60,0,{0,0,0,0},{0,0,0,0} },
	/* Following timezones added in 1.1b4 to allow Australian daylight time */
	{ T_AUSTRALIA_SEASTERN,	"EST", "EDT", 10*60,60,{10,2*60,ON_AFTER|dSun,25},{3,2*60,ON_AFTER|dSun,25} },
	{ T_AUSTRALIA_SCENTRAL,	"CST", "CDT",9*60+30,60,{10,2*60,ON_AFTER|dSun,25},{3,2*60,ON_AFTER|dSun,25} },
	{ T_AUSTRALIA_LORDHOWE,	"LHST","LHDT",10*60+30,30,{10,2*60,ON_AFTER|dSun,25},{3,2*60,ON_AFTER|dSun,25} },
   	{""} };

#ifdef PI
#undef PI
#endif
#define PI 3.14159265358979323846
#define DEGRAD (PI/180.0)
#define RADDEG (180.0/PI)

static void far_sky(int);
void WantedTheSky(int flag) { far_sky(flag); }			/* Relay function - for far call */

static void
far_timestamp (char *buf, char *dbuf, struct tm *t);
void
do_timestamp (char *buf, char *dbuf, struct tm *t) {
	far_timestamp (buf, dbuf, t); }

static double cosz = -0.014543897;  /* for sunrise and sunset */
GMTime_t sunrise, sunset;

#ifdef MOONRISE
int moonrise, moonset;
static void ThatsAmore(char *, int);
#endif

/* LocalTimeToDST - given local time, returns a struct tm with the local time,
 *  corrected for Daylight Savings.
 */
static DateTimeUnion tval;		/* Shared temp struct for returning results */

struct tm * LocalTimeToDST( LocalTime_t* secs ) {
	TimSecondsToDateTime(*secs, &tval.dtm);		/* Get data for local std time */
	computeDST(&tval);
	return &tval.ttm;
	}

/* helper function - reconverts a local time struc tm back to a GMT time_t */
GMTime_t localtm2GMT(struct tm *ht, Boolean midnight) {
	GMTime_t gmtime;
	if (midnight) {
		ht->tm_min = ht->tm_hour = ht->tm_sec = 0;
		computeDST((DateTimeUnion *)ht);
		}
	gmtime = TimDateTimeToSeconds((DateTimeType*) ht);
	if (ht->tm_isdst) gmtime -= 60L*timezone->DSTswitch;	/*  - "fall back" if DST */
	gmtime -= meridian;				/* Convert to GMT time: local time + zone */
	return gmtime;
	}

/* Compute time in GMT, rounded back to last minute
 *  (or, if midnight!=0, back to last midnight).
 */
GMTime_t local2GMT(LocalTime_t loctime, Boolean midnight) {
 	struct tm *ht;
	ht = LocalTimeToDST(&loctime);		/*  to find out if it's DST right now */
	return localtm2GMT(ht,midnight);
}

/* BackToMidnight - backs up faketime (which is somewhere in current computation day
 *	in GMT) to be local midnight in GMT.
 * Resets compute state to let computation for that day to restart.
 */
void BackToMidnight(void) {
 	struct tm *ht;
	faketime = local2GMT(DateBeingDisplayed, 1);
	ht = localtime(&faketime);
 	if ((DayofWeek = ht->tm_wday - WeekStartDay) < 0) /* Save day of week and light its button */
		DayofWeek = 6;
	ComputeState = CSStart;				/* Void old tides & recompute */
	}

void InitializeTime(void) {
	char dummy;
	WeekStartDay = PrefGetPreference(prefWeekStartDay);
	LongDateFormat = PrefGetPreference(prefLongDateFormat);
	fixtimedates(true);
	if (LocalDateNow != Preferences.Ptoday) {	/* If date changed since program last run */
		DisplayingToday = true;			/*  display today the first time through */
		fixtimedates(true);
		}
	LocGetNumberSeparators(PrefGetPreference(prefNumberFormat),&dummy,&decimal_point);
	}

static void fixtimedates(Boolean init) {				/* Set weekday and LocalDateNow */
	FormPtr mainf;
	LocalTime_t t = TimGetSeconds ();
	TimeToDisplay = 0;				/* Force display of conditions now */
	LocalDateNow = (t / (60L * 60L * 24L)) * (60L * 60L * 24L);
	if (init && DisplayingToday) 			/* If displaying today, reset display date */
		DateBeingDisplayed = LocalDateNow;  /* (This is initial default) */
	faketime = local2GMT(DateBeingDisplayed, 1);	/* Initialize faketime */
	BackToMidnight();					/* Back up to midnight */
	if ((mainf = FrmGetFormPtr(MainForm)))	/* If main form has been initialized, */
		FrmSetControlGroupSelection (mainf, 
		  WEEKGROUP, WeekMon + DayofWeek );	/* Fix date display */
	}

void MoveDays(int i) {					/* Move i days forward or back */
	if (!i || MoveFaketime(i)) {			/* Change faketime */
		fixtimedates(false);			/* Adjust local date, etc. */
	  	DisplayingToday = (LocalDateNow == DateBeingDisplayed);
		}
	}

/** FIXME - since BackToMidnight now depends on DateBeingDisplayed instead of vice versa,
**   take most of this stuff (but not the error check) out.  **/
/*
 * This routine is only called directly from the library routine to get midnight of the next night.
 *  	In that case, everything done here gets put back immediately after.
 * It is also called for real from MoveDays.
 */
Boolean MoveFaketime(int i) {
	/* Move i days forward/backward.  Assumes faketime is on a day boundary. */
	/* Moves 12 extra hours forward, then to midnight, to allow for DST boundaries. */
	/* Note - int is ok here - allows for +/- 89 years. */
	GMTime_t l, f;
	DateTimePtr ht;
	l = i;
	l *= (60L * 60L * 24L);
	f = faketime + l + 12L*hoursInSeconds;		/* Move GMT, allowing slop for DST */
	ht = (DateTimePtr)localtime(&f);
	if (ht->year < first_year || ht->year >= first_year + num_epochs) {
		barf (MISSINGYEAR);
		return false;
  		}
	faketime = f; 
	DateBeingDisplayed += 					/* Move the current date (in local time) */
			i * (60L * 60L * 24L);
	BackToMidnight();
	return true;
	}

Boolean SelectADate() {
	DateTimePtr ht;
	LocalTime_t t;
	ht = (DateTimePtr)localtime(&faketime);
	if (!SelectDay ( selectDayByDay, &ht->month, &ht->day,
						&ht->year, T_GO_TO_DATE)) return 0;
	if (ht->year < first_year || ht->year >= first_year + num_epochs) {
		barf (MISSINGYEAR);
		return 0;
  		}
	ht->hour = 2;			/* Move forward a couple hours to avoid day boundary */
	t = TimDateTimeToSeconds((DateTimeType*) ht);
	DateBeingDisplayed = (t / (60L * 60L * 24L)) * (60L * 60L * 24L);
  	DisplayingToday = (LocalDateNow == DateBeingDisplayed);
	fixtimedates(false);
	return 1;
	}

#if LANG == LANG_FRENCH
#define L_MONTH_ABBR 5
#else
#define L_MONTH_ABBR 4
#endif
static char Nmonths[12][L_MONTH_ABBR] = { T_MONTHS_OF_YEAR  };
static char Ndays[7][4] = { T_DAYS_OF_WEEK };

char *ZoneName(int IsDST) {
	static char gmtname[20];
	int i;
	if (IsDST != -1 && timezone) 
		return IsDST ? timezone->DSTtime : timezone->stdtime;
	i = meridian / 1800L;
	if (i < 0) i = -i;
	sprintf(gmtname,T_GMT,meridian < 0 ? "-" : "+", i/2);
	if (i & 1) strcat(gmtname,":30");
	if (IsDST == -1) 				/* If -1, want to add "hour" or "hours" */
		strcat(gmtname, (i/2) == 1 ? T_HOUR : T_HOURS);
	return gmtname;
	}

static char *DSTPrint(char *p, struct DSTswitch *s) {
	char *w;
	if (s->wkday & ON_DATE) 
		p += sprintf(p,"%s %d",Nmonths[s->month-1],s->mday);
	else if (s->wkday & ON_AFTER) {
		switch (s->mday) {
			case 1: w = T_FIRST; break;
			case 8: w = T_SECOND; break;
			case 15: w = T_THIRD; break;
			case 22: w = T_FOURTH; break;
			case 24: case 25: w = T_LAST; break;
			default:
				p += sprintf(p,T_THE_DAY_ON_OR_AFTER,
					Ndays[s->wkday & M_WEEKDAY],Nmonths[s->month-1],
					s->mday);
				return p;
			}
		p += sprintf(p,T_THE_NTH_WKDAY_OF_MONTH,
				w, Ndays[s->wkday & M_WEEKDAY],Nmonths[s->month-1]);
		}
	return p;
	}

void DisplayTZInfo(void) {
  	char *p, buf[400];
	p = buf + sprintf(buf,T_USES,Place->location);
	if (timezone) p += sprintf(p,T_TIMEZONE_WHICH_IS,timezone->longname);
	p += sprintf(p,"%s.  ",ZoneName(-1));
	if (!timezone || !timezone->toDST.month)
		p += sprintf(p, T_DST_NOT_OBSERVED);
       else {
		if (Preferences.PignoreDST) 
			p += sprintf(p, T_YOU_DISABLED_DST);
		p += sprintf(p,T_SWITCHES_TO_DST,timezone->stdtime,
			timezone->DSTtime);
		p = TimeToString(p, timezone->DSTswitch,false,false,0,t17_01);
		p += sprintf(p,T_HOUR_AT);
		p = TimeToString(p, timezone->toDST.hour,true,false,0,Preferences.Ptunits);
		p += sprintf(p,T_ON);
		/* We sloppily rely here on the day of week enumeration in DateTime.h */
		p = DSTPrint(p, &timezone->toDST);
		p += sprintf(p,T_UNTIL);
		p = DSTPrint(p, &timezone->fromDST);
		strcat(p,".");
		}
 	FrmCustomAlert (DBINFO, buf, NULL, NULL);
  	}

/* 
 *	Some useful C time and trig functions that don't work yet in GCC
 */

static char time_separators[] = {':', ':', 0, 'h'};

/*
 *	Formats a time in minutes into the string at p, according to the format given.
 *	NOTE: Returns the END of the string.
 */
char *TimeToString(char *p, int minutes, Boolean show_mins, Boolean pad, char padchar, enum TimePrefs tpref) {
	int i;
	char *pm;
	pm = T_AYEM;
	if (minutes < 0) minutes = -minutes;
#if 0
	p += sprintf(p,"%d",minutes/60);
	if ((i = minutes % 60) != 0 || mins) {
		*p++ = ':';
		if (i < 10) *p++ = '0';
		p += sprintf(p,"%d",i);
		}
#else
	i = minutes / 60;
	minutes = minutes % 60;
	if (tpref == t5_01pm) {
		if (i > 11) { pm = T_PM; i -= 12; }
		if (i == 0) i = 12; }
	  else {				/* Pad non-am/pm display */
		pm = "";
		if (pad) { 
			*p++ = Numspace;
			*p++ = Numspace;
		}	}
	if (i < 10 && pad) *p++ = padchar;
	p += sprintf(p,"%d",i);
	if (minutes || show_mins) {
		if (time_separators[tpref]) 
			*p++ = time_separators[tpref];
		if (minutes < 10) *p++ = '0';
		p += sprintf(p,"%d%s",minutes, pm);
		}
#endif
	return p;
	}

static void
far_timestamp (char *buf, char *dbuf, struct tm *t) {
	char padchar;
	padchar = Preferences.Ptunits == t_1701 ? '0' : Numspace;
	if (buf != NULL) {
		TimeToString(buf, 60*t->tm_hour + t->tm_min, true, true, padchar, Preferences.Ptunits);
#ifdef PRECISE
		sprintf(buf + strlen(buf),":%d",t->tm_sec);
#endif
		}
	if (dbuf != NULL) {
		DateToAscii(t->tm_mon,t->tm_mday,t->tm_year,LongDateFormat,dbuf);
	}	}

time_t mktime(struct tm *t) {
	/* Note this does not set tm_isdst, etc. */
	ULong seconds;
	seconds = TimDateTimeToSeconds((DateTimeType*) t);
	DateSecondsToDate(seconds, (DatePtr) t);
	return seconds;
	}

Boolean IsDST(GMTime_t t) {
	/* Returns whether that time (in GMT) is in local DST or not */
	struct tm *ht;
	ht = LocalTimeToDST(&t);
	return ht->tm_isdst;
	}

/* Note that time functions do not bother with yday */

/* localtime - given time in GMT, returns a struct tm with the local time,
 *  corrected for Daylight Savings.
 */
struct tm * localtime( GMTime_t* secs ) {
	struct tm *ht;
	LocalTime_t t = *secs + meridian;
	ht = LocalTimeToDST(&t);
	if (ht->tm_isdst)				/* If it's DST, */
		TimAdjust( (DateTimeType*) ht, 60L*timezone->DSTswitch);	/*  spring forward. */
	return ht;
	}

/* Return 1 if the DST switch is past the cutover - right month is assumed */
static Boolean pastswitch(struct DSTswitch *s) {
  int dl;
  if (s->wkday & ON_DATE) dl = s->mday;
    else {
	/* Compute the date of the month we're looking for */
	dl = (M_WEEKDAY & s->wkday) + 1 				/* Day we're seeking */
		- DayOfWeek (tval.dtm.month, 1, tval.dtm.year);	/* Day of week of 1st */
	while (dl < s->mday) dl += 7;			/* Get 1st one past mday */
	}
  if (tval.dtm.day < dl) return 0;
  if (tval.dtm.day > dl) return 1;
  return (60*tval.dtm.hour+tval.dtm.minute > s->hour);
  }

static void computeDST(DateTimeUnion *dt) {		/* Set tm_isdst flag in date/time struct */
	struct DSTswitch *from, *to;
	dt->ttm.tm_isdst = 0;
	/* Handle Daylight Savings. */
	if (!Preferences.PignoreDST && timezone && timezone->toDST.month) {
		to = &timezone->toDST;
		from = &timezone->fromDST;
		if (to->month > from->month) {
			/* Southern hemisphere - reverse the sense */
			from = to;
			to = &timezone->fromDST;
			dt->ttm.tm_isdst = 1;
			}
		if ((dt->dtm.month > to->month && dt->dtm.month < from->month) 
		 || (dt->dtm.month == to->month && pastswitch(to))
		 || (dt->dtm.month == from->month && !pastswitch(from)))
					dt->ttm.tm_isdst = ! dt->ttm.tm_isdst;
	}	}

struct tm * gmtime( GMTime_t* secs ) {
	TimSecondsToDateTime(*secs, &tval.dtm);
	tval.ttm.tm_isdst = 0;
	return &tval.ttm;
	}

/* Limited f format printf - only values up to +- 32767 */
/* Note- this code only works for fairly small orders of magnitude because
    of the multipliers, and of multiple uses of the buffer */

char * sfpr(float f, int width, int prec, int rjust) {
	static float pmults[] = { 10.0, 100.0, 1000.0, 10000.0 };
	static float pround[] = { 0.00001, 0.0499,  0.00499,  0.000499,  0.0000499 };
	static char buf[20];
	int i, j, sign = 0;
	char *p = buf, *start;
	/* Set aside spaces for right justification.  This is kludgy because
	   if the number is positive, the first blank is actually two spaces
	   to equal the minus sign, which is 4 pixels wide.  The rest are
	   Numspaces, which are as wide as a number. */
	for (i = 3; i--; ) *p++ = Numspace;
	*p++ = ' '; *p++ = ' ';
	start = p;
	if (f < 0) {				/* If negative */
		sign = 1;
		p[-1] = p[-2] = Numspace;		/* don't need to padd with thin spaces */
		f = -f;
		*p++ = '-';
		}
	prec = prec < 0 ? 4 : prec > 4 ? 4 : prec; 
	f += pround[prec];
	i = f;
	p += sprintf(p,"%d",i);
	f -= i;
	if (prec) {
		f *= pmults[prec-1];
		i = f;
		j = sprintf(&buf[14],"%d",i);
		*p++ = decimal_point;
		if (j > prec) buf[14+prec] = 0;
		while ( j < prec-- ) *p++ = '0';
		strcpy(p, &buf[14]);
		p += j;
		}
	if (rjust) {
		p = start;
		while (strlen(start) < width && start > buf) --start;
		if (!sign && p != start) --start;		/* Skip extra space to allow for sign */
		}
	return start;
	}

void
fpr(float f, int width, int prec, int minus)
{
  printf("%s",sfpr(f, width, prec, minus));
}

#ifdef GTD_COSINE

#define pi2    1.570796326795
#define M_PI 3.141592654

#else

#define NPTS 1024
#define pi2    1.570796326795
#define M_PI 3.141592654
static Double pts[NPTS + 1];
static double sinf(double arg);

#if 0
static void showerr(void) {
	long i, ntries = 1000;
	double myerror = 0, ierror = 0, e, ma = 0, ia = 0, my, real, aa, ay;
	e = 4e7;
	for (i = 1; i < ntries; i += 1) {
		e *= 0.99;
		my = cos_degrees(e) - sinf(pi2 + e* M_PI / 180.0);
		if (my < 0) my = -my;
		if (my > ma) { ma = my; ay = e; }
		myerror += my;
		}
	printf("My cos error = %e (max %e at %e)\n",myerror/ntries, ma, ay);
	printf("Tested down to %e\n",e);
	}
#endif

#define INCR 128				/* Initialize in increments for event loop control */
Boolean InitCosine(int doit) {
	static int i = NPTS;		/* Number yet to initialize */
	int j;
	if (!i) return false;		/* If table has been initialized, return false */
	if (!doit) return true;		/* If just asking status, return true */
	pts[0] = 1.0; 			/* This one doesn't get computed */
	for (j = INCR; --j >= 0; ) {	/* Make cosine table */
		--i;
		pts[NPTS - i] = sinf(i * ( pi2 / (float)NPTS));
		}
	return true;			/* If not initialized yet */
	}

Double cos_degrees (double arg)	/* Sin function by interpolation */
{	double y;
	long e;
	int i,j,k;

	if (arg < 0) arg = -arg;
#if 0	/* This is now done in several places before this function is called */
	arg *= (1024.0 / 90.0);		/* Map 0 - 90 degrees to 0 - 1024 */
#endif
	e = arg;
	y = arg - e;
	if (y < 0 || y >= 1) { 
		barf(FLOATOFLOW);
		return 0; }
	j = e;
	i = j & 2047;			/* Modulo 1024 is position in 1st quadrant */
	k = 1;
	if (j & 1024) {			/* 3rd & 4th quadrants are reverse of 1 & 2 */
		i = 2048 - i;
		k = -1; }
	y = pts[i] + y * ( pts[i+k] - pts[i] );	/* Interpolate */
	if ((j &= (1024+2048)) == 1024 || j == 2048)
			y = -y;		/* 3rd, 4th quadrants are negative */
	return y;
}
#endif /* GTD_COSINE */

static Double F_poly(Double x, Double p[], int n)  /* evaluate nth deg. polynomial, n >= 1 */
{   static Double result;  static int i;
    for (result = p[i = n]; i; ) result = p[--i] + x * result;
    return result; }

#ifndef GTD_COSINE
double sinf(double arg)
{	/* Coefficients are #3370 from Hart & Cheney (18.80D). */
	static Double p[5] = {
	  .1357884e8, -.49429081e7, .440103053e6, -.138472724e5, .145968841e3 };
	static Double q[5] = {
	  .864455865e7, .408179225e6, .946309610e4, .132653491e3, 1.0 };
	static Double y,ysq;
	static long quad,e;

	quad = (arg >= 0.0) ? 0 : (arg = -arg, 2);
	arg *= .63661977;		/* 2 / pi */
	y = arg - (e = arg);
	quad=(e + quad) % 4;
	if (quad & 1) y = 1.0 - y;
	if(quad & 2) y = -y;

	ysq = y * y;
	return y * F_poly(ysq,p,4)/F_poly(ysq,q,4);
}
#endif /* not GTD_COSINE */

#if 0		/* Other sqrt actually larger by 12 bytes - */
		/* someday be nice to have it in a separate prc file though. */
typedef long int32_t ;
typedef unsigned long u_int32_t ;
#include "..\mathlib\E_sqrt.c"
float mysqrt(float f) {return __ieee754_sqrt(f);}
#else
float mysqrt(float f)
/*	Eqns. 6.1.3 & 6.1.7; Initial guess from table 0231 */
{	static Double p[] = {.58812e-2, .5267875, .5881229 };
	static Double f2, temp, scale;
	static int t;

	if (f < 0.0) return -1.0;
	if (f == 0.0) return 0.0;
	scale = 1.0;
	while (f > 1.0) { f = f * 0.01; scale = scale * 10.0; }
	while (f < 0.01) { f = f * 100.0; scale = scale * 0.1; }
	f2 = F_poly(f, p, 2) / (f + .999998e-1);
	for (t = 10; t--; ) {
		temp = 0.5 * (f2 + f / f2);
		if (temp == f2) break;
		f2 = temp; }
	return f2 * scale;
}
#endif

double fabs(double f) { return f < 0.0 ? -f : f; }

#ifndef GTD_COSINE
double mycos(double f) {
  	return sinf(fabs(f) + pi2);
	}
#endif

double atanf(double f)
/*	Eqns. 6.5.21, & 6.5.22	& Table 5097 */
{	static Double p[6] = {
		33.05861847399, 58.655751569, 32.3909748562,
		5.853195211263, .1952374193623, -.002434603300441 };
	static Double q[5] = {
		33.05861847399, 69.67529105952, 49.00434821822,
		12.97557886271, 1.0};
	int recip;

	Double f2, f2sq;
	if ((f2 = fabs(f)) == 0.0) return 0;
	if ((recip = (f2 > 1.0)) != 0) f2 = 1.0 / f2;	/* 0 < f2 < 1 */
	f2sq = f2 * f2;
	f2 *= F_poly(f2sq, p, 5) / F_poly(f2sq, q, 4);
	if (recip) f2 = pi2 - f2;
	return f < 0.0 ? -f2 : f2;
}

double myacos(double x) {
	if (x == 0.0) return 90.0;
	if (x == -1.0) return 180;
	x = atanf(mysqrt(1 - x*x) / x);
	if (x < 0.0) x += M_PI;
	return x;
	}

/***************************************************************/
/*                                                             */
/*  Sunrise and sunset functions.                              */
/*                                                             */
/*  Algorithm from "Almanac for computers for the year 1978"   */
/*  by L. E. Doggett, Nautical Almanac Office, USNO.           */
/*                                                             */
/*  This code also uses some ideas found in programs written   */
/*  by Michael Schwartz and Marc T. Kaufman.                   */
/*                                                             */
/*  Functions from REMIND.                                     */
/*  Copyright (C) 1992-1997 by David F. Skoll                  */
/*                                                             */
/*  REMIND source code is public under the GNU public license  */
/*    except that it may not be used under Microsoft Windows   */
/*    or any future version thereof, or by Cadabra Design      */
/*    Libraries, Inc., its subsidiaries or directors.          */
/*                                                             */
/***************************************************************/

/* FIXME:  A better sun algorithm would be ... er ... better. */

void Fiddler(LocalTime_t date) {
	sunrise = SunStuff(date, true);
	sunset = SunStuff(date, false);
	}

time_t SunStuff(LocalTime_t locdate, Boolean rise)		/* Returns time in minutes UTC */
{
    long jan0;
    int mins, hours;
    DateType date;

    double M, L, tanA, sinDelta, cosDelta, a, a_hr, cosH, t, H, T;
    double latit, longdeg, UT;

/* Get offset from UTC */
    mins = meridian / 60;

/* Get latitude and longitude - w latitude is positive */
    longdeg = - longitude / (3600.0*15.0);		/* Long in hours = degrees/15 */
    latit = latitude * (DEGRAD/3600.0);			/* Lat in radians */
 
    /* Correct the date for cases where time zone is on one side of date line */
    /*   but longitude is on the other.  Remember longdeg has flipped sign. */

    if (longdeg < -9.0 && meridian < 0) locdate += daysInSeconds;
	else if (longdeg > 9.0 && meridian > 0) locdate -= daysInSeconds;

/* Get number of days since January 1 */
	DateSecondsToDate ( locdate, &date);
	jan0 = DateToDays(date);
	date.month = date.day = 1;
	jan0 -=  DateToDays(date);

/* Following formula on page B6 exactly... (with optimization - WB) */
    t = longdeg / 24.0 + (double) jan0 + (rise ? 0.25 : 0.75);

/* Mean anomaly of sun for 1978 uses 0.985600 * t - 3.18 - ... how accurate for other years??? */
/* Several web pages use 3.18 instead of 3.251 - WB 12/03 */
    M = 0.985600 * t - 3.18;  /* In degrees */

/* Sun's true longitude */
    L = M + 1.916*sinf(DEGRAD*M) + 0.02*sinf(2*DEGRAD*M) + 282.565;
    if (L > 360.0) L -= 360.0;

/* Tan of sun's right ascension */
    tanA = 0.91746 * sinf(DEGRAD*L)/mycos(DEGRAD*L);
    a = RADDEG * atanf(tanA);

/* Move a into same quadrant as L */
    if (0.0 <= L && L < 90.0) {
	if (a < 0.0) a += 180.0;
    } else if (L < 270.0) {
	a += 180.0;
    } else {
	if (a > 0.0) a += 180.0;
    }

    if (a > 360.0)
	a -= 360.0;
    a_hr = a / 15.0;

/* Sine of sun's declination */
    sinDelta = 0.39782 * sinf(DEGRAD*L);
    cosDelta = mysqrt(1 - sinDelta*sinDelta);

/* Cosine of sun's local hour angle */
    cosH = (cosz - sinDelta * sinf(latit)) / (cosDelta * mycos(latit));

    if (cosH < -1.0) { /* Summer -- permanent daylight */
	if (rise) return ALWAYS_LIGHT ;
	else      return ALWAYS_LIGHT ;
    }
    if (cosH > 1.0) { /* Winter -- permanent darkness */
	if (rise) return ALWAYS_DARK ;
	else      return ALWAYS_DARK ;
    }

    H = RADDEG * myacos(cosH);
    if (rise) H = 360.0 - H;

    T = H / 15.0 + a_hr - 0.065710*t - 6.620;
    while (T >= 24.0) T -= 24.0;
    while (T < 0.0) T+= 24.0;

    UT = T + longdeg;
    jan0 = UT * 3600.0;
    jan0 += locdate;

    hours = UT + (double) mins / 60.0;
    if (hours < 0) hours += 24;
      else if (hours >= 24) hours -= 24;

    /* Sometimes, we get roundoff error.  Check for "reasonableness" of
       answer. */
    if (rise) {
	/* Sunrise so close to midnight it wrapped around -- permament light */
	if (hours >= 23) return ALWAYS_LIGHT ;
    } else {
	/* Sunset so close to midnight it wrapped around -- permament light */
	if (hours <= 1) return ALWAYS_LIGHT ;
    }
    return jan0;
}

float phases[] = {0.02, 	/* Last quarter */
			0.27, 	/* New */
			0.51, 	/* First quarter */
			0.77, 	/* Full moon */
			};

static double moon_phase(GMTime_t date) {
	double jdate;
	double phase;
	long l;
	jdate = (date - meridian) / 86400.0;			/* Convert to julian date */
	/* .743... equals fract(julian date conversion 2416480.5 / 29.53058868) */
	l = phase = .74359859595 + jdate / 29.53058868;
	phase -= l;
	return phase;
	}

static char phasenames[4][20] = { T_MOONPHASES };

# ifdef MOONRISE		/** MOONRISE ***/
/* Moonrise and moonset: Basic program hand-translated into C, for my sins.
900 REM  ***************************
910 REM  THIS PROGRAM COMPUTES THE
920 REM  TIMES OF MOONRISE AND MOON-
930 REM  SET ANYWHERE IN THE WORLD.
940 REM  FROM SKY & TELESCOPE, JULY,
950 REM  1989, PAGE 78.
960 REM  ***************************
*/
static float posits[4][4];					/* Position array */
static float fract(float f) { return f - (long) f; }
static float T;

static float threepoint(int i, float P) {
	float A, B;
	A=posits[2][i]-posits[1][i]; B=posits[3][i]-posits[2][i]-A;
	return posits[1][i]+P*(2*A+B*(2*P-1));
	}

#ifndef ORIGINAL_RECIPE
static float init1[] = {0.374897, 0.259091, 0.827362, 0.347343, 0.993126 };
static float init2[] = {0.03629164709, 0.03674819520, 0.03386319198, -0.00014709391, 0.00273777850};

static float initval(float a, float b) {
	return fract(a + b * T) * (2.0*PI); 
	}

/*
 *  Computation for moonrise/moonset algorithm.
 *  Yes, this structure-driven computation is butt-ugly.
 *  But it generates about 3.5k less code than the straight-line, ok?
 */
static float mfd[5];
#define M mfd[0]
#define F mfd[1]
#define D mfd[2]
#define N mfd[3]
#define G mfd[4]
struct mooncomp {
	float factor;
	char coeffs[5];
	};
struct mooncomp vfactors[] = {
	/*           M  F  D  N  G   */
	{ 0.39558, { 0, 1, 0, 1, 0 } },
	{ 0.08200, { 0, 1, 0, 0, 0 } },
	{ 0.03257, { 1,-1, 0,-1, 0 } },
	{ 0.01092, { 1, 1, 0, 1, 0 } },
	{ 0.00666, { 1,-1, 0, 0, 0 } },
	{ -0.00644,{ 1, 1,-2, 1, 0 } },
	{ -0.00331,{ 0, 1,-2, 1, 0 } },
	{ -0.00304,{ 0, 1,-2, 0, 0 } },
	{ -0.00240,{ 1,-1,-2,-1, 0 } },
	{ 0.00226, { 1, 1, 0, 0, 0 } },
	{ -0.00108,{ 1, 1,-2, 0, 0 } },
	{ -0.00079,{ 0, 1, 0,-1, 0 } },
	{ 0.00078, { 0, 1, 2, 1, 0 } },
	{ 0, { 0, 0, 0, 0, 0 } } };

struct mooncomp wfactors[] = {
	/*           M  F  D  N  G   */
	{ 0.10478, { 1, 0, 0, 0, 0, } },
	{ -0.04105, {0, 2, 0, 2, 0, } },
	{ -0.02130, {1, 0,-2, 0, 0, } },
	{ -0.01779, {0, 2, 0, 1, 0, } },
	{ 0.01774, { 0, 0, 0, 1, 0, } },
	{ 0.00987, { 0, 0, 2, 0, 0, } },
	{ -0.00338, {1,-2, 0,-2, 0, } },
	{ -0.00309, {0, 0, 0, 0, 1, } },
	{ -0.00190, {0, 2, 0, 0, 0, } },
	{ -0.00144, {1, 0, 0, 1, 0, } },
	{ -0.00144, {1,-2, 0,-1, 0, } },
	{ -0.00113, {1, 2, 0, 2, 0, } },
	{ -0.00094, {1, 0,-2, 0, 1, } },
	{ -0.00092, {2, 0,-2, 0, 0, } },
	{ 0, { 0, 0, 0, 0, 0 } } };

struct mooncomp ufactors[] = {
	/*           M  F  D  N  G   */
	{ -0.10828, {1, 0, 0, 0, 0, } },
	{ -0.01880, {1, 0,-2, 0, 0, } },
	{ -0.01479, {0, 0, 2, 0, 0, } },
	{  0.00181, {2, 0,-2, 0, 0, } },
	{ -0.00147, {2, 0, 0, 0, 0, } },
	{ -0.00105, {0, 0, 2, 0,-2, } },
	{ -0.00075, {1, 0,-2, 0, 1, } },
	{ 0, { 0, 0, 0, 0, 0 } } };

float mulfactors(struct mooncomp *, int);
float mulfactors(struct mooncomp *mp, int ival) {
	float sum, arg;
	int i;
	sum = ival;
	while (mp->factor) {
		arg = 0;
		for (i = 0; i < 5; ++i) switch (mp->coeffs[i]) {
			case 2: arg += mfd[i];
			case 1: arg += mfd[i]; break;
			case -2: arg -= mfd[i];
			case -1: arg -= mfd[i];
			}
		sum += (mp++)->factor * (ival ? mycos(arg) : sinf(arg));
		}
	return sum;
	}
#endif /* ORIGINAL_RECIPE */
/*
 * Testing note: For date line issues use Neiafu (GMT+13) and Niue (GMT-11).
 * 	Neiafu = Niue + 16 minutes + 1 day.  Further north use Constantine Harbor
 *	and Sweeper Cove, Alaska.
 */
static void ThatsAmore(char *buf, int buflen) {
	int i, j;
	char *p = buf;
	Boolean first = true;
	float latit, sthour, endhour;
	float S, T0, L, V, Z1, C0, C, Z, A0, D0, U, W,
		A2, D2, L0, L2, H0, H1, H2, D1, V0=0, V2=0, V1, A, B, E;
#ifdef ORIGINAL_RECIPE
   	float F, D, M, N, G; 
#endif
	GMTime_t GMTdate, starttime, endtime, event;
	struct tm *t; 
	moonrise = moonset = -1;

	starttime = local2GMT(DateBeingDisplayed, 1);			/* GMT for start of day */
	endtime = local2GMT(DateBeingDisplayed + daysInSeconds, 1); 
	GMTdate = 							/* Get midnight GMT somewhere in local day */
		((endtime - 60L * 15L) / daysInSeconds) * daysInSeconds;
	sthour = ((long)(starttime - GMTdate )) 		/* GMT offsets for local midnight to midnight */
		/ (60L * 60L);					/*  (takes care of 23 and 25 hour days for DST) */
	endhour = ((long)(endtime - GMTdate )) / (60L * 60L);
	endhour -= 0.5;
	if (meridian % 3600) {					/* For timezones not on even GMT hours */
		sthour -= 1.0;
		endhour += 1.0;					/* compute a 26 hour day */
		}

	T = GMTdate / (3600L * 24L);
	T += (2416480.5 - 2451545.0);				/* Convert to Julian date and shift */

//	Z0 = - meridian/86400.0;				/* Z0=timezone/24; we'll just use GMT*/

	T0=T / 36525.0;						/* LST at 0h zone time */
	S=24110.5 + 8640184.813 * T0;
//	S=S + 86636.6 * Z0;
	S=S/86400.0 + longitude/(3600.0 * 360.0);	 	/* longitude in fraction of a day */
	S=fract(S);
	T0=S*360*(PI/180.0);
//	T = T + Z0;
/* 50 REM      POSITION LOOP */
	for (i = 1; i <= 3; ++i) {
/* 495 REM   FUNDAMENTAL ARGUMENTS */
#ifdef ORIGINAL_RECIPE
		L=fract(0.606434+0.03660110129*T)*(2.0*PI); 
		M=fract(0.374897+0.03629164709*T)*(2.0*PI); 
		F=fract(0.259091+0.03674819520*T)*(2.0*PI); 
		D=fract(0.827362+0.03386319198*T)*(2.0*PI); 
		N=fract(0.347343-0.00014709391*T)*(2.0*PI); 
		G=fract(0.993126+0.00273777850*T)*(2.0*PI);
		V=0.39558*sinf(F+N);
		V=V+0.08200*sinf(F);
		V=V+0.03257*sinf(M-F-N);
		V=V+0.01092*sinf(M+F+N);
		V=V+0.00666*sinf(M-F);
		V=V-0.00644*sinf(M+F-2*D+N);
		V=V-0.00331*sinf(F-2*D+N);
		V=V-0.00304*sinf(F-2*D);
		V=V-0.00240*sinf(M-F-2*D-N);
		V=V+0.00226*sinf(M+F);
		V=V-0.00108*sinf(M+F-2*D);
		V=V-0.00079*sinf(F-N);
		V=V+0.00078*sinf(F+2*D+N);
		U=1-0.10828*mycos(M);
		U=U-0.01880*mycos(M-2*D);
		U=U-0.01479*mycos(2*D);
		U=U+0.00181*mycos(2*M-2*D);
		U=U-0.00147*mycos(2*M);
		U=U-0.00105*mycos(2*D-G);
		U=U-0.00075*mycos(M-2*D+G);
		W=0.10478*sinf(M);
		W=W-0.04105*sinf(2*F+2*N);
		W=W-0.02130*sinf(M-2*D);
		W=W-0.01779*sinf(2*F+N);
		W=W+0.01774*sinf(N);
		W=W+0.00987*sinf(2*D);
		W=W-0.00338*sinf(M-2*F-2*N);
		W=W-0.00309*sinf(G);
		W=W-0.00190*sinf(2*F);
		W=W-0.00144*sinf(M+N);
		W=W-0.00144*sinf(M-2*F-N);
		W=W-0.00113*sinf(M+2*F+2*N);
		W=W-0.00094*sinf(M-2*D+G);
		W=W-0.00092*sinf(2*M-2*D);
#else /* ORIGINAL_RECIPE */
		L=initval(0.606434, 0.03660110129); 
		for (j = 5; --j >= 0; ) mfd[j] = initval(init1[j],init2[j]);
		V = mulfactors(vfactors, 0);
		U = mulfactors(ufactors, 1);
		W = mulfactors(wfactors, 0);
#endif /* ORIGINAL_RECIPE */

/*  REM    COMPUTE RA, DEC, DIST */
		S=W/mysqrt(U-V*V);
		posits[i][1]=L+atanf(S/mysqrt(1-S*S));
		S=V/mysqrt(U);
		posits[i][2]=atanf(S/mysqrt(1-S*S));
		posits[i][3]=60.40974*mysqrt(U);
		T = T + 0.5;
		}
	if (posits[2][1]<=posits[1][1]) { posits[2][1]=posits[2][1]+(2.0*PI); }
	if (posits[3][1]<=posits[2][1]) { posits[3][1]=posits[3][1]+(2.0*PI); }
	Z1=(PI/180.0)*(90.567-41.685/posits[2][3]);
	latit = latitude * (PI / (180.0 * 3600.0));
	S=sinf(latit); C=mycos(latit);
	Z=mycos(Z1); 
	A0=posits[1][1]; D0=posits[1][2];
	for (C0=sthour; C0 <= endhour; ++C0) {
		A2 = threepoint(1,(C0+1)/24.0);
		D2 = threepoint(2,(C0+1)/24.0);
/* REM  TEST AN HOUR FOR AN EVENT */
		A0=A2;D0=D2; V0=V2;
		L0=T0+C0*(15*(PI/180.0)*1.0027379); L2=L0+(15*(PI/180.0)*1.0027379);
		if (A2<A0) { A2=A2+2*PI; }
		H0=L0-A0; H2=L2-A2;
		H1=(H2+H0)/2;                         /* HOUR ANGLE */
		D1=(D2+D0)/2;                         /* DEC */
		if (first) { 				  /* Initialize V0 here */
			V0=S*sinf(D0)+C*mycos(D0)*mycos(H0)-Z; 
			first = false;
			}
		V2=S*sinf(D2)+C*mycos(D2)*mycos(H2)-Z;
		if (V0 < 0 && V2 < 0) continue;		/* IF SGN(V0)=SGN(V2) THEN 440 */
		if (V0 >= 0 && V2 >= 0) continue;
		V1=S*sinf(D1)+C*mycos(D1)*mycos(H1)-Z;
		A=2*V2-4*V1+2*V0; B=4*V1-3*V0-V2;
		D=B*B-4*A*V0; if (D<0) continue;
		D=mysqrt(D);
/*
355 IF V0<0 AND V2>0 THEN PRINT R$;
360 IF V0<0 AND V2>0 THEN M8=1
365 IF V0>0 AND V2<0 THEN PRINT S$;
370 IF V0>0 AND V2<0 THEN W8=1
*/
		E=(-B+D)/(2*A);
		if (E>1 || E<0) { E=(-B-D)/(2*A); }
//printf("%ld %ld\n",(long)(E*1000000),(long)C0*3600L);
						/* The long cast fixes a sign issue here */
		event = (long)(3600.0 * (C0+E+(1.0/120.0))); 	 /* ROUND OFF */
//printf("%ld %ld\n",(long) E, event);
/*
390 H3=INT(T3): M3=INT((T3-H3)*60)
395 PRINT USING "##:##";H3;M3;
*/
		event += GMTdate;						/* GMT event time in seconds */
		if (event >= starttime && event <= endtime) {	/* Make sure event is today */
			if (V0 < 0) moonrise = i; else moonset = i;
			p += sprintf(p, T_MOON, V0 < 0 ? T_RISES_SETS);
			do_timestamp(p, NULL, t = localtime(&event));
//printf("%d/%d %d:%02d\n",t->tm_mon,t->tm_mday,t->tm_hour,t->tm_min); SysTaskDelay(500); 
			p += strlen(p);
			*p++ = '\n';
			}
		A0=A2;D0=D2;V0=V2;

/* If we want to know someday where the moon is rising and setting ...
400 H7=H0+E*(H2-H0)
405 N7=-COS(D1)*SIN(H7)
410 D7=C*SIN(D1)-S*COS(D1)*COS(H7)
415 A7=ATN(N7/D7)/R1
420 IF D7<0 THEN A7=A7+180
425 IF A7<0 THEN A7=A7+360
430 IF A7>360 THEN A7=A7-360
435 PRINT UsinG ",  AZ ###.#";A7
440 RETURN
445 REM
*/
		}							/* End 24 hours loop */
	if (moonrise < 0 && moonset < 0) 
		p += sprintf(p, T_MOON_IS_ALL_DAY,V2 < 0 ? T_DOWN_UP);
	*p = 0;
	assert (p < buf + buflen);
	}
#endif /*** MOONRISE ***/

static int old_sol(char *label,GMTime_t time, char *buf) {
	char dbuf[longDateStrLength];
	if (!(time & SUN_MASK)) {
		return 0;
		}
#if 1
	do_timestamp(dbuf, NULL, localtime(&time));
	strcat(buf,label);
#else  /* Debug sunrise */
	{ char b2[60]; do_timestamp(dbuf, b2, localtime(&time));
	strcat(buf,b2); }
#endif
	strcat(buf,dbuf);
	strcat(buf,"\n");
	return 1;
	}

static void far_sky(int flag) {			/* WantedTheSky - called by relay function */
#define fsNBUF 400
   char buf[fsNBUF+1], dbuf[longDateStrLength+1], todbuf[longDateStrLength+1];
   DateTimeType dt;
   int dim, i;
   double phase;
   int daylen;
   GMTime_t always;
   LocalID alert;

   buf[0] = 0;
   switch (flag) {
    default: return;
    case 1: 					/* Moon */
   	TimSecondsToDateTime(DateBeingDisplayed, &dt);
	DateToAscii(dt.month,dt.day,dt.year,LongDateFormat,todbuf);
# ifdef MOONRISE		/** MOONRISE ***/
	strcat(buf,T_ON_DATE_MOON);
	strcat(buf,todbuf);
	strcat(buf,T_MOON_INTRO);
	i = strlen(buf);
	ThatsAmore(buf + i, fsNBUF - i - 160);
#endif
      dt.day = 1;
	dim = DaysInMonth (dt.month, dt.year);
	phase = moon_phase(TimDateTimeToSeconds (&dt) - 3600L*24L);	/* Phase of last day last month */
	for (i = 0; i < 4; ++i) if (phase < phases[i]) break;	/* Starting phase */
	for (dt.day = 1; dt.day <= dim; ++dt.day) {
		if (i == 4) { i = 0; phase -= 1.0; }
 		phase += 0.03386319219;
		if (phase >= phases[i]) {
			strcat(buf,phasenames[i]);
			DateToAscii(dt.month,dt.day,dt.year,LongDateFormat,dbuf);
			strcat(buf,dbuf);
			++i;
		}	}
	alert = MOONINFO;
	break;
     case 2:
	Fiddler(DateBeingDisplayed);
	strcpy(buf,T_ON_DATE_SUN);
	do_timestamp(NULL, buf+strlen(buf), LocalTimeToDST(&DateBeingDisplayed));
	strcat(buf,T_SUN_INTRO);
	always = sunrise;
	if (!(i = old_sol(T_SUN_RISES_AT,sunrise,buf))) 
		sunrise = local2GMT(DateBeingDisplayed, 1);
	if (old_sol(T_SUN_SETS_AT,sunset,buf)) ++i;
		else sunset = local2GMT(DateBeingDisplayed + 24L * 60L * 60L,1);
	if ((daylen = (sunset - sunrise + 30) / 60) < 0) daylen = 24 * 60 - daylen;
//printf("u%lu d%lu %d\n",sunrise,sunset,daylen); SysTaskDelay(300);
	if (!i) {
		strcat(buf,T_SUN_IS);
		strcat(buf,always == ALWAYS_LIGHT ? T_UP_DOWN);
		strcat(buf,T_ALL_DAY_TODAY);
		daylen = always == ALWAYS_LIGHT ? 24 * 60 : 0;
		}
	if (i == 2) 
		sprintf(buf + strlen(buf),T_HRS_MIN_OF_DAYLIGHT,daylen/60, daylen%60);
	alert = SUNINFO;
	}
   FrmCustomAlert (alert, buf, NULL, NULL);
   }

#define NPHASES 12
int MoonPhase(GMTime_t date) {
	int i;
	float phase = moon_phase(date);
	phase = phase + 2.0 - phases[3];			/* Scale to full moon = 0*/
	i = (phase * NPHASES);
	i %= NPHASES;
//printf("phase = %d.%d i = %d\n", (int)moon_phase(date),((int) (moon_phase(date) * 100))%100, i);
	return i;
	}

